{
Copyright (C) 2001  Nicola Girardi <nikke@swlibero.org>

Original copyright and license information follows. This unit is distributed
under the terms of the GNU LGPL, as the original library.

GdkPixbuf library - Main header file

Copyright (C) 1999 The Free Software Foundation

Authors: Mark Crichton <crichton@gimp.org>
         Miguel de Icaza <miguel@gnu.org>
         Federico Mena-Quintero <federico@gimp.org>
         Havoc Pennington <hp@redhat.com>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
}

unit GdkPixbuf;

interface

uses Glib, GdkTypes, Gdk;

type
  PGdkColorSpace = ^GdkColorSpace;
  GdkColorSpace  = Integer;
{ Color spaces; right now only RGB is supported }
const
  GDK_COLORSPACE_RGB = 0;

type
  { All of these are opaque structures }
  PGdkPixbuf          = Pointer;
  PGdkPixbufFrame     = Pointer;
  PGdkPixbufAnimation = Pointer;
  { Handler that must free the pixel array }
  GdkPixbufDestroyNotify = ^procedure(Pixels : PGUChar; Data : GPointer);
  { Handler for the last unref operation }
  GdkPixbufLastUnref = ^procedure(Pixbuf : PGdkPixbuf; Data : GPointer);

{ Reference counting }

function GdkPixbufRef(Pixbuf : PGdkPixbuf) : PGdkPixbuf;
asmname 'gdk_pixbuf_ref';

procedure GdkPixbufUnref(Pixbuf : PGdkPixbuf);
asmname 'gdk_pixbuf_unref';

procedure GdkPixbufSetLastUnrefHandler(Pixbuf : PGdkPixbuf; LastUnrefFn : GdkPixbufLastUnref; LastUnrefFnData : GPointer);
asmname 'gdk_pixbuf_set_last_unref_handler';

procedure GdkPixbufFinalize(Pixbuf : PGdkPixbuf);
asmname 'gdk_pixbuf_finalize';

{ GdkPixbuf accessors }

function GdkPixbufGetColorspace(Pixbuf : PGdkPixbuf) : PGdkColorSpace;
asmname 'gdk_pixbuf_get_colorspace';

function GdkPixbufGetNChannels(Pixbuf : PGdkPixbuf) : Integer;
asmname 'gdk_pixbuf_get_nchannels';

function GdkPixbufGetHasAlpha(Pixbuf : PGdkPixbuf) : GBoolean;
asmname 'gdk_pixbuf_get_has_alpha';

function GdkPixbufGetBitsPerSample(Pixbuf : PGdkPixbuf) : Integer;
asmname 'gdk_pixbuf_get_bits_per_sample';

function GdkPixbufGetPixels(Pixbuf : PGdkPixbuf) : PGUChar;
asmname 'gdk_pixbuf_get_pixels';

function GdkPixbufGetWidth(Pixbuf : PGdkPixbuf) : Integer;
asmname 'gdk_pixbuf_get_width';

function GdkPixbufGetHeight(Pixbuf : PGdkPixbuf) : Integer;
asmname 'gdk_pixbuf_get_height';

function GdkPixbufGetRowstride(Pixbuf : PGdkPixbuf) : Integer;
asmname 'gdk_pixbuf_get_rowstride';

{ Create a blank pixbuf with an optimal rowstride and a new buffer }
function GdkPixbufNew(ColorSpace : GdkColorspace; HasAlpha : GBoolean; BitsPerSample, Width, Height : Integer) : PGdkPixbuf;
asmname 'gdk_pixbuf_new';

{ Copy a pixbuf }

function GdkPixbufCopy (Pixbuf : PGdkPixbuf) : PGdkPixbuf;

{ Simple loading }

function GdkPixbufNewFromFile(FileName : PGChar) : PGdkPixbuf;
asmname 'gdk_pixbuf_new_from_file';

function GdkPixbufNewFromData(Data : PGUChar; Colorspace : GdkColorspace; HasAlpha : GBoolean; BitsPerSample, Width, Height, Rowstride : Integer; DestroyFn : GdkPixbufDestroyNotify; DestroyFnData : GPointer) : PGdkPixbuf;
asmname 'gdk_pixbuf_new_from_data';

function GdkPixbufNewXpmFromData(Data : PPGChar) : PGdkPixbuf;
asmname 'gdk_pixbuf_new_from_xpm_data';

{ Adding an alpha channel }

function GdkPixbufAddAlpha(Pixbuf : PGdkPixbuf; SubstituteColor : GBoolean; r, g, b : GUChar) : PGdkPixbuf;
asmname 'gdk_pixbuf_add_alpha';

{ Copy an area of a pixbuf onto another one }
procedure GdkPixbufCopyArea(SrcPixbuf : PGdkPixbuf; SrcX, SrcY, Width, Height : Integer; DestPixbuf : PGdkPixbuf; DestX, DestY : Integer);
asmname 'gdk_pixbuf_copy_area';

{ Rendering to a drawable }

{ Alpha compositing mode }
type
  PGdkPixbufAlphaMode = ^GdkPixbufAlphaMode;
  GdkPixbufAlphaMode  = Integer;
const
  GDK_PIXBUF_ALPHA_BILEVEL = 0;
  GDK_PIXBUF_ALPHA_FULL    = 1;

procedure GdkPixbufRenderThresholdAlpha(Pixbuf : PGdkPixbuf; Bitmap : PGdkBitmap; SrcX, SrcY, DestX, DestY, Width, Height, AlphaThreshold : Integer);
asmname 'gdk_pixbuf_render_threshold_alpha';

procedure GdkPixbufRenderToDrawable(Pixbuf : PGdkPixbuf; Drawable : PGdkDrawable; Gc : PGdkGc; SrcX, SrcY, DestX, DestY, Width, Height : Integer; Dither : GdkRgbDither; XDither, YDither : Integer);
asmname 'gdk_pixbuf_render_to_drawable';

procedure GdkPixbufRenderToDrawableAlpha(Pixbuf : PGdkPixbuf; Drawable : PGdkDrawable; SrcX, SrcY, DestX, DestY, Width, Height : Integer; AlphaMode : GdkPixbufAlphaMode; AlphaThreshold : Integer; Dither : GdkRgbDither; XDither, YDither : Integer);
asmname 'gdk_pixbuf_render_to_drawable_alpha';

procedure GdkPixbufRenderPixmapAndMask(Pixbuf : PGdkPixbuf; var Pixmap : PGdkPixmap; var Mask : PGdkBitmap; AlphaThreshold : Integer);
asmname 'gdk_pixbuf_render_pixmap_and_mask';

{ Fetching a region from a drawable }
function GdkPixbufGetFromDrawable(Dest : PGdkPixbuf; Src : PGdkDrawable; Cmap : PGdkColormap; SrcX, SrcY, DestX, DestY, Width, Height : Integer) : PGdkPixbuf;
asmname 'gdk_pixbuf_get_from_drawable';

{ Scaling }

{ Interpolation modes }
const
  GDK_INTERP_NEAREST  = 0;
  GDK_INTERP_TILES    = 1;
  GDK_INTERP_BILINEAR = 2;
  GDK_INTERP_HYPER    = 3;
type
  PGdkInterpType = ^GdkInterpType;
  GdkInterpType  = Integer;

procedure GdkPixbufScale(Src, Dest : PGdkPixbuf; DestX, DestY, DestWidth, DestHeight : Integer; OffsetX, OffsetY, ScaleX, ScaleY : Real; InterpType : GdkInterpType);
asmname 'gdk_pixbuf_scale';

procedure GdkPixbufComposite(Src, Dest : PGdkPixbuf; DestX, DestY, DestWidth, DestHeight : Integer; OffsetX, OffsetY, ScaleX, ScaleY : Real; InterpType : GdkInterpType; OverallAlpha : Integer);
asmname 'gdk_pixbuf_composite';

procedure GdkPixbufCompositeColor(Src, Dest : PGdkPixbuf; DestX, DestY, DestWidth, DestHeight : Integer; OffsetX, OffsetY, ScaleX, ScaleY : Real; InterpType : GdkInterpType; OverallAlpha : Integer; CheckX, CheckY, CheckSize : Integer; Color1, Color2 : GUint32);
asmname 'gdk_pixbuf_composite_color';

function GdkPixbufScaleSimple(Src : PGdkPixbuf; DestWidth, DestHeight : Integer; InterpType : GdkInterpType) : PGdkPixbuf;
asmname 'gdk_pixbuf_scale_simple';

function GdkPixbufCompositeColorSimple(Src : PGdkPixbuf; DestWidth, DestHeight : Integer; InterpType : GdkInterpType; OverallAlpha, CheckSize : Integer; Color1, Color2 : GUint32) : PGdkPixbuf;
asmname 'gdk_pixbuf_composite_color_simple';

{ Animation support }

{ GIF-like animation overlay modes for frames }
const
  GDK_PIXBUF_FRAME_RETAIN  = 0;
  GDK_PIXBUF_FRAME_DISPOSE = 1;
  GDK_PIXBUF_FRAME_REVERT  = 2;
type
  PGdkPixbufFrameAction = ^GdkPixbufFrameAction;
  GdkPixbufFrameAction  = Integer;

function GdkPixbufAnimationNewFromFile(FileName : PGChar) : PGdkPixbufAnimation;
asmname 'gdk_pixbuf_animation_new_from_file';

function GdkPixbufAnimationRef(Animation : PGdkPixbufAnimation) : PGdkPixbufAnimation;
asmname 'gdk_pixbuf_animation_ref';

procedure GdkPixbufAnimationUnref(Animation : PGdkPixbufAnimation);
asmname 'gdk_pixbuf_animation_unref';

function GdkPixbufAnimationGetWidth(Animation : PGdkPixbufAnimation) : Integer;
asmname 'gdk_pixbuf_animation_get_width';

function GdkPixbufAnimationGetHeight(Animation : PGdkPixbufAnimation) : Integer;
asmname 'gdk_pixbuf_animation_get_height';

function GdkPixbufAnimationGetFrames(Animation : PGdkPixbufAnimation) : PGList;
asmname 'gdk_pixbuf_animation_get_frames';

function GdkPixbufAnimationGetNumFrames(Animation : PGdkPixbufAnimation) : Integer;
asmname 'gdk_pixbuf_animation_get_num_frames';

{ Frame accessors }

function GdkPixbufFrameGetPixbuf(Frame : PGdkPixbufFrame) : PGdkPixbuf;
asmname 'gdk_pixbuf_frame_get_pixbuf';

function GdkPixbufFrameGetXOffset(Frame : PGdkPixbufFrame) : Integer;
asmname 'gdk_pixbuf_frame_get_x_offset';

function GdkPixbufFrameGetYOffset(Frame : PGdkPixbufFrame) : Integer;
asmname 'gdk_pixbuf_frame_get_y_offset';

function GdkPixbufFrameGetDelayTime(Frame : PGdkPixbufFrame) : Integer;
asmname 'gdk_pixbuf_frame_get_delay_time';

function GdkPixbufFrameGetAction(Frame : PGdkPixbufFrame) : GdkPixbufFrameAction;
asmname 'gdk_pixbuf_frame_get_action';

{ !! General (presently empty) initialization hooks, primarily for gnome-libs NOT IMPLEMENTED. }

implementation

end.
