Program simple;

(* 
 * Copyright (C) 1998 Janne Lf <jlof@mail.student.oulu.fi>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *)

{$include "../units/gtk.inc"}
uses
  gdkgl, gtkglarea, gl;


var
  window, glarea : PGtkWidget;
  infoStr : pgchar;
  (* Attribute list for gtkglarea widget. Specifies a
     list of Boolean attributes and enum/integer
     attribute/value pairs. The last attribute must be
     GDK_GL_NONE. See glXChooseVisual manpage for further
     explanation.
  *)
  attrlist : array [0..8] of integer value (
    GDK_GL_RGBA,
    GDK_GL_RED_SIZE,1,
    GDK_GL_GREEN_SIZE,1,
    GDK_GL_BLUE_SIZE,1,
    GDK_GL_DOUBLEBUFFER,
    GDK_GL_NONE
  );
  dummy : GInt;

  

function init(widget : PGtkWidget) : gint;
begin
  (* OpenGL functions can be called only if make_current returns true *)
  if GtkGLAreaMakeCurrent(PGtkGLArea(widget)) <> G_FALSE then
    begin
      glViewport(0,0, widget^.allocation.width, widget^.allocation.height);
      glMatrixMode(GL_PROJECTION);
      glLoadIdentity;
      glOrtho(0,100, 100,0, -1,1);
      glMatrixMode(GL_MODELVIEW);
      glLoadIdentity
    end;
  init :=  G_TRUE
end;


(* When widget is exposed it's contents are redrawn. *)
function draw(widget : PGtkWidget; event : PGdkEventExpose) : gint;
begin
  (* Draw only last expose. *)
  if event^.count > 0 then return G_TRUE;

  (* OpenGL functions can be called only if make_current returns true *)
  if GtkGLAreaMakeCurrent(PGtkGLArea(widget)) <> G_FALSE then
    begin

      (* Draw simple triangle *)
      glClearColor(0,0,0,1);
      glClear(GL_COLOR_BUFFER_BIT);
      glColor3f(1,1,1);
      glBegin(GL_TRIANGLES);
      glVertex2f(10,10);
      glVertex2f(10,90);
      glVertex2f(90,90);
      glEnd;

      (* Swap backbuffer to front *)
      GtkGLAreaSwapBuffers(PGtkGLArea(widget));
        
    end;

  draw :=  G_TRUE
end;

(* When glarea widget size changes, viewport size is set to match the new size *)
function reshape(widget : PGtkWidget; event: PGdkEventConfigure ) : gint;
begin
  (* OpenGL functions can be called only if make_current returns true *)
  if GtkGLAreaMakeCurrent(PGtkGLArea(widget)) <> G_FALSE then
    glViewport(0,0, widget^.allocation.width, widget^.allocation.height);
  reshape := G_TRUE
end;



begin
  (* initialize gtk *)
  GtkInit;

  (* Check if OpenGL is supported. *)
  if GdkGLQuery = G_FALSE then
    begin
      Writeln('OpenGL not supported');
      Exit
    end;

  (* vendor dependent version info string *)
  infoStr := GdkGLGetInfo;
  GPrint(infoStr);
  GFree(infoStr);


  (* Create new top level window. *)
  window := GtkWindowNew( GTK_WINDOW_TOPLEVEL );
  GtkWindowSetTitle( PGtkWindow(window), 'Simple' );
  GtkContainerSetBorderWidth( PGtkContainer(window), 10 );

  (* Quit form main if got delete event *)
  dummy := GtkSignalConnect( PGtkObject(window), 'delete_event', 
                        @GtkMainQuit, Nil );


  (* You should always delete gtk_gl_area widgets before exit or else
     GLX contexts are left undeleted, this may cause problems (=core dump)
     in some systems.
     Destroy method of objects is not automatically called on exit.
     You need to manually enable this feature. Do gtk_quit_add_destroy()
     for all your top level windows unless you are certain that they get
     destroy signal by other means.
  *)
  GtkQuitAddDestroy(1, PGtkObject(window));


  (* Create new OpenGL widget. *)
  glarea := PGtkWidget(GtkGLAreaNew( PInteger(@attrlist) ));
  (* Events for widget must be set before X Window is created *)
  GtkWidgetSetEvents(glarea, GDK_EXPOSURE_MASK Or GDK_BUTTON_PRESS_MASK);
  
  (* Connect signal handlers *)
  (* Redraw image when exposed. *)
  dummy := GtkSignalConnect(PGtkObject(glarea), 'expose_event', @draw, Nil);
  (* When window is resized viewport needs to be resized also. *)
  dummy := GtkSignalConnect(PGtkObject(glarea), 'configure_event',
		                    @reshape, Nil);
  (* Do initialization when widget has been realized. *)
  dummy := GtkSignalConnect(PGtkObject(glarea), 'realize', @init, Nil);

  (* set minimum size *)
  GtkWidgetSetUsize(glarea, 100,100);

  (* put glarea into window and show it all *)
  GtkContainerAdd(PGtkContainer(window),glarea);
  GtkWidgetShow(glarea);
  GtkWidgetShow(window);

  GtkMain;
end.

