{ Report bugs to Nicola Girardi <nikke@swlibero.org> }
program Calendar;

{$include "../units/gtk.inc"}
{$ignore-function-results}

uses GPC;

const
  DEF_PAD       = 10;
  DEF_PAD_SMALL = 5;

type
  PCalendarData = ^TCalendarData;
  TCalendarData = record
    FlagCheckboxes : array[0..4] of PGtkWidget;
    Settings : array[0..4] of Boolean;
    Font : PGChar;
    FontDialog, Window, Prev2Sig, PrevSig, LastSig, Month : PGtkWidget;
  end;

procedure CalendarDateToString(Data : PCalendarData; var Buffer : String);
var
  Year, Month, Day : GUint;
begin
  GtkCalendarGetDate(PGtkCalendar(Data^.Window), Year, Month, Day);
  WriteStr(Buffer, Year, '/', Month, '/', Day);
end;

procedure CalendarSetSignalStrings(SigStr : String; Data : PCalendarData);
var
  PrevSig : PGChar;
begin
  GtkLabelGet(PGtkLabel(Data^.PrevSig), PrevSig);
  GtkLabelSetText(PGtkLabel(Data^.Prev2Sig), PrevSig);

  GtkLabelGet(PGtkLabel(Data^.LastSig), PrevSig);
  GtkLabelSetText(PGtkLabel(Data^.PrevSig), PrevSig);

  GtkLabelSetText(PGtkLabel(Data^.LastSig), SigStr);
end;

procedure CalendarMonthChanged(Widget : PGtkWidget; Data : PCalendarData);
var
  Buffer : TString;
begin
  CalendarDateToString(Data, Buffer);
  Buffer := 'Month changed: ' + Buffer;
  CalendarSetSignalStrings(Buffer, Data);
end;

procedure CalendarDaySelected(Widget : PGtkWidget; Data : PCalendarData);
var
  Buffer : TString;
begin
  CalendarDateToString(Data, Buffer);
  Buffer := 'Day selected: ' + Buffer;
  CalendarSetSignalStrings(Buffer, Data);
end;

procedure CalendarDaySelectedDoubleClick(Widget : PGtkWidget; Data : PCalendarData);
var
  Year, Month, Day : GUint;
  Buffer : TString;
begin
  CalendarDateToString(Data, Buffer);
  Buffer := 'Day selected with double click: ' + Buffer;
  CalendarSetSignalStrings(Buffer, Data);

  GtkCalendarGetDate(PGtkCalendar(Data^.Window), Year, Month, Day);

  if PGtkCalendar(Data^.Window)^.MarkedDate[Day-1] = 0 then
    GtkCalendarMarkDay(PGtkCalendar(Data^.Window), Day)
  else
    GtkCalendarUnmarkDay(PGtkCalendar(Data^.Window), Day);
end;

procedure CalendarPrevMonth(Widget : PGtkWidget; Data : PCalendarData);
var
  Buffer : TString;
begin
  CalendarDateToString(Data, Buffer);
  Buffer := 'Previous month: ' + Buffer;
  CalendarSetSignalStrings(Buffer, Data);
end;

procedure CalendarNextMonth(Widget : PGtkWidget; Data : PCalendarData);
var
  Buffer : TString;
begin
  CalendarDateToString(Data, Buffer);
  Buffer := 'Next month: ' + Buffer;
  CalendarSetSignalStrings(Buffer, Data);
end;

procedure CalendarPrevYear(Widget : PGtkWidget; Data : PCalendarData);
var
  Buffer : TString;
begin
  CalendarDateToString(Data, Buffer);
  Buffer := 'Previous year: ' + Buffer;
  CalendarSetSignalStrings(Buffer, Data);
end;

procedure CalendarNextYear(Widget : PGtkWidget; Data : PCalendarData);
var
  Buffer : TString;
begin
  CalendarDateToString(Data, Buffer);
  Buffer := 'Next year: ' + Buffer;
  CalendarSetSignalStrings(Buffer, Data);
end;

procedure CalendarSetFlags(Data : PCalendarData);
var
  I : Integer;
  Options : Integer = 0;
begin
  for I := 0 to 4 do
    if Data^.Settings[I] then
      Inc(Options, 1 shl I);
  if Data^.Window <> nil then
    GtkCalendarDisplayOptions(PGtkCalendar(Data^.Window), Options);
end;

procedure CalendarToggleFlag(Toggle : PGtkWidget; Data : PCalendarData);
var
  I, J : Integer;
begin
  J := 0;
  for I := 0 to 4 do
    if Data^.FlagCheckboxes[I] = Toggle then
      J := I;

  Data^.Settings[J] := not Data^.Settings[J];
  CalendarSetFlags(Data);
end;

procedure CalendarFontSelectionOk(Button : PGtkWidget; Data : PCalendarData);
var
  Style : PGtkStyle;
  Font : PGdkFont;
begin
  Data^.Font := GtkFontSelectionDialogGetFontName(PGtkFontSelectionDialog(Data^.FontDialog));
  if Data^.Window <> nil then
    begin
      Font := GtkFontSelectionDialogGetFont(PGtkFontSelectionDialog(Data^.FontDialog));
      if Font <> nil then
        begin
          Style := GtkStyleCopy(GtkWidgetGetStyle(Data^.Window));
          GdkFontUnref(Style^.Font);
          Style^.Font := Font;
          GdkFontRef(Style^.Font);
          GtkWidgetSetStyle(Data^.Window, Style);
        end;
    end;
  GtkWidgetUnref(Data^.FontDialog);
end;

procedure CalendarSelectFont(Button : PGtkWidget; Data : PCalendarData);
var
  Window : PGtkWidget;
begin
  if Data^.FontDialog = nil then
    begin
      Window := GtkFontSelectionDialogNew('Font Selection Dialog');
      if not GTK_IS_FONT_SELECTION_DIALOG(Window) then
        Exit;
      Data^.FontDialog := Window;

      GtkWindowSetPosition(PGtkWindow(Window), GTK_WIN_POS_MOUSE);

      GtkSignalConnect(PGtkObject(Window), 'destroy', @GtkWidgetDestroyed, @Data^.FontDialog);
      GtkSignalConnect(PGtkObject(PGtkFontSelectionDialog(Window)^.OkButton), 'clicked', @CalendarFontSelectionOk, Data);
      GtkSignalConnectObject(PGtkObject(PGtkFontSelectionDialog(Window)^.CancelButton), 'clicked', @GtkWidgetDestroy, PGtkObject(Data^.FontDialog));
    end;
  Window := Data^.FontDialog;
  if not GTK_WIDGET_VISIBLE(Window) then
    GtkWidgetShow(Window)
  else
    GtkWidgetDestroy(Window);
end;

procedure CreateCalendar;
var
  Window, VBox, VBox2, VBox3, HBox, HBBox, Calendar, Toggle, Button,
  Frame, Separator, LLabel, BBox : PGtkWidget;
  CalendarData : static TCalendarData;
  I : Integer;
  FlagNames : array[0..4] of TString = (
    'Show Heading',
    'Show Day Names',
    'No Month Change',
    'Show Week Numbers',
    'Week Start Monday');
begin
  CalendarData.Window := nil;
  CalendarData.Font := nil;
  CalendarData.FontDialog := nil;

  for I := 0 to 4 do
    CalendarData.Settings[I] := False;

  Window := GtkWindowNew(GTK_WINDOW_TOPLEVEL);
  GtkWindowSetTitle(PGtkWindow(Window), 'GtkCalendar Example');
  GtkContainerSetBorderWidth(PGtkContainer(Window), 5);
  GtkSignalConnect(PGtkObject(Window), 'destroy', @GtkMainQuit, nil);
  GtkSignalConnecT(PGtkObject(Window), 'delete_event', @GtkFalse, nil);
  GtkWindowSetPolicy(PGtkWindow(Window), G_FALSE, G_FALSE, G_TRUE);

  VBox := GtkVBoxNew(G_FALSE, DEF_PAD);
  GtkContainerAdd(PGtkContainer(Window), VBox);

  { The top part of the window, Calendar, flags and fontsel. }

  HBox := GtkHBoxNew(G_FALSE, DEF_PAD);
  GtkBoxPackStart(PGtkBox(VBox), HBox, G_TRUE, G_TRUE, DEF_PAD);
  HBBox := GtkHButtonBoxNew;
  GtkBoxPackStart(PGtkBox(HBox), HBBox, G_FALSE, G_FALSE, DEF_PAD);
  GtkButtonBoxSetLayout(PGtkButtonBox(HBBox), GTK_BUTTONBOX_SPREAD);
  GtkButtonBoxSetSpacing(PGtkButtonBox(HBBox), 5);

  { Calendar widget }
  Frame := GtkFrameNew('Calendar');
  GtkBoxPackStart(PGtkBox(HBBox), Frame, G_FALSE, G_TRUE, DEF_PAD);
  Calendar := GtkCalendarNew;
  CalendarData.Window := Calendar;
  CalendarSetFlags(@CalendarData);
  GtkCalendarMarkDay(PGtkCalendar(Calendar), 19);
  GtkContainerAdd(PGtkContainer(Frame), Calendar);
  GtkSignalConnect(PGtkObject(Calendar), 'month_changed', @CalendarMonthChanged, @CalendarData);
  GtkSignalConnect(PGtkObject(Calendar), 'day_selected', @CalendarDaySelected, @CalendarData);
  GtkSignalConnect(PGtkObject(Calendar), 'day_selected_double_click', @CalendarDaySelectedDoubleClick, @CalendarData);
  GtkSignalConnect(PGtkObject(Calendar), 'prev_month', @CalendarPrevMonth, @CalendarData);
  GtkSignalConnect(PGtkObject(Calendar), 'next_month', @CalendarNextMonth, @CalendarData);
  GtkSignalConnect(PGtkObject(Calendar), 'prev_year', @CalendarPrevYear, @CalendarData);
  GtkSignalConnect(PGtkObject(Calendar), 'next_year', @CalendarNextYear, @CalendarData);

  Separator := GtkVSeparatorNew;
  GtkBoxPackStart(PGtkBox(HBox), Separator, G_FALSE, G_TRUE, 0);

  VBox2 := GtkVBoxNew(G_FALSE, DEF_PAD);
  GtkBoxPackStart(PGtkBox(HBox), VBox2, G_FALSE, G_FALSE, DEF_PAD);

  { Build the Right frame with the flags in }

  Frame := GtkFrameNew('Flags');

  GtkBoxPackStart(PGtkBox(Vbox2), Frame, G_TRUE, G_TRUE, DEF_PAD);
  VBox3 := GtkVBoxNew(G_TRUE, DEF_PAD_SMALL);
  GtkContainerAdd(PGtkContainer(Frame), VBox3);

  for I := 0 to 4 do
    begin
      Toggle := GtkCheckButtonNewWithLabel(FlagNames[I]);
      GtkSignalConnect(PGtkObject(Toggle), 'toggled', @CalendarToggleFlag, @CalendarData);
      GtkBoxPackStart(PGtkBox(VBox3), Toggle, G_TRUE, G_TRUE, 0);
      CalendarData.FlagCheckboxes[I] := Toggle;
    end;

  { Build the right font-button }
  Button := GtkButtonNewWithLabel('Font...');
  GtkSignalConnect(PGtkObject(Button), 'clicked', @CalendarSelectFont, @CalendarData);
  GtkBoxPackStart(PGtkBox(VBox2), Button, G_FALSE, G_FALSE, 0);

  { Build the Signal-event part. }
  Frame := GtkFrameNew('Signal events');
  GtkBoxPackStart(PGtkBox(VBox), Frame, G_TRUE, G_TRUE, DEF_PAD);

  VBox2 := GtkVBoxNew(G_TRUE, DEF_PAD_SMALL);
  GtkContainerAdd(PGtkContainer(Frame), VBox2);

  HBox := GtkHBoxNew(G_FALSE, 3);
  GtkBoxPackStart(PGtkBox(VBox2), HBox, G_FALSE, G_TRUE, 0);
  LLabel := GtkLabelNew('Signal:');
  GtkBoxPackStart(PGtkBox(HBox), LLabel, G_FALSE, G_TRUE, 0);
  CalendarData.LastSig := GtkLabelNew('');
  GtkBoxPackStart(PGtkBox(HBox), CalendarData.LastSig, G_FALSE, G_TRUE, 0);

  HBox := GtkHBoxNew(G_FALSE, 3);
  GtkBoxPackStart(PGtkBox(VBox2), HBox, G_FALSE, G_TRUE, 0);
  LLabel := GtkLabelNew('Previous signal:');
  GtkBoxPackStart(PGtkBox(HBox), LLabel, G_FALSE, G_TRUE, 0);
  CalendarData.PrevSig := GtkLabelNew('');
  GtkBoxPackStart(PGtkBox(HBox), CalendarData.PrevSig, G_FALSE, G_TRUE, 0);

  HBox := GtkHBoxNew(G_FALSE, 3);
  GtkBoxPackStart(PGtkBox(VBox2), HBox, G_FALSE, G_TRUE, 0);
  LLabel := GtkLabelNew('Second previous signal:');
  GtkBoxPackStart(PGtkBox(HBox), LLabel, G_FALSE, G_TRUE, 0);
  CalendarData.Prev2Sig := GtkLabelNew('');
  GtkBoxPackStart(PGtkBox(HBox), CalendarData.Prev2Sig, G_FALSE, G_TRUE, 0);

  BBox := GtkHButtonBoxNew;
  GtkBoxPackStart(PGtkBox(VBox), BBox, G_FALSE, G_FALSE, 0);
  GtkButtonBoxSetLayout(PGtkButtonBox(BBox), GTK_BUTTONBOX_END);
  Button := GtkButtonNewWithLabel('Close');
  GtkSignalConnect(PGtkObject(Button), 'clicked', @GtkMainQuit, nil);
  GtkContainerAdd(PGtkContainer(BBox), Button);
  GTK_WIDGET_SET_FLAGS(Button, GTK_CAN_DEFAULT);
  GtkWidgetGrabDefault(Button);

  GtkWidgetShowAll(Window);
end;

begin
  GtkInit;
  CreateCalendar;
  GtkMain;
end.
