{ Report Bugs To Nicola Girardi <nikke@swlibero.org> }
program CList;

{$include "../units/gtk.inc"}
{$ignore-function-results}

{ User clicked the "Add List" button. }
procedure ButtonAddClicked(Button : PGtkButton; Data : GPointer);
var
  Indx : Integer;
  Drink : array[1..4, 1..2] of PGChar = ( ( 'Milk',    '3 oz' ),
                                          ( 'Water',   '6 l'  ),
                                          ( 'Carrots', '2'    ),
                                          ( 'Snakes',  '55'   ) );
begin
  { Here we do the actual adding of the text. It's done once for each row. }
  for Indx := 1 to 4 do
    GtkCListAppend(PGtkCList(Data), @Drink[Indx][1]);
end;

{ User clicked the "Clear List" button. }
procedure ButtonClearClicked(Button : PGtkButton; Data : GPointer);
begin
  { Clear the list using gtk_clist_clear. This is much faster than
    calling gtk_clist_remove once for each row. }
  GtkCListClear(PGtkCList(Data));
end;

{ The user clicked the "Hide/Show titles" button. }
procedure ButtonHideShowClicked(Button : PGtkButton; Data : GPointer);
var
  { Just a flag to remember the status. 0 = currently visible }
  Flag : static Integer = 0;
begin
  if Flag = 0 then
    begin
      { Hide the titles and set the flag to 1 }
      GtkCListColumnTitlesHide(PGtkCList(Data));
      Inc(Flag);
    end
  else
    begin
      { Show the titles and reset flag to 0 }
      GtkCListColumnTitlesShow(PGtkClist(Data));
      Dec(Flag);
    end;
end;

{ If we come here, then the user has selected a row in the list. }
procedure SelectionMade(CList : PGtkWidget; Row, Column : GInt; Event : PGdkEventButton; Data : GPointer);
var
  TText : PGChar;
begin
  { Get the text that is stored in the selected row and column
    which was clicked in. We will receive it as a pointer in the
    argument text. }
  GtkCListGetText(PGtkCList(CList), Row, Column, TText);

  { Just prints some information about the selected row }
  Writeln('You selected row ', Row, '. More specifically you clicked in '
          'column ', Column, ', and the text in this cell is ',
          CString2String(TText));
  Writeln;
end;

var
  Window, VBox, HBox, Scrolled, CList, ButtonAdd, ButtonClear, ButtonHideShow : PGtkWidget;
  Titles : array[1..2] of PGChar = ( 'Ingredients', 'Amount' );

begin
  GtkInit;

  Window := GtkWindowNew(GTK_WINDOW_TOPLEVEL);
  GtkWidgetSetUSize(PGtkWidget(Window), 300, 150);

  GtkWindowSetTitle(PGtkWindow(Window), 'GtkCList Example');
  GtkSignalConnect(PGtkObject(Window), 'destroy', @GtkMainQuit, nil);

  VBox := GtkVBoxNew(G_FALSE, 5);
  GtkContainerSetBorderWidth(PGtkContainer(VBox), 5);
  GtkContainerAdd(PGtkContainer(Window), VBox);
  GtkWidgetShow(VBox);

  { Create a scrolled window to pack the CList widget into }
  Scrolled := GtkScrolledWindowNew(nil, nil);
  GtkScrolledWindowSetPolicy(PGtkScrolledWindow(Scrolled), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
  GtkBoxPackStart(PGtkBox(VBox), Scrolled, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(Scrolled);

  { Create the CList. For this example we use 2 columns }
  CList := GtkCListNewWithTitles(2, @Titles[1]);

  { When a selection is made, we want to know about it. The callback
    used is selection_made, and its code can be found further down }
  GtkSignalConnect(PGtkObject(CList), 'select_row', @SelectionMade, nil);

  { It isn't necessary to shadow the border, but it looks nice :) }
  GtkCListSetShadowType(PGtkCList(CList), GTK_SHADOW_OUT);

  { What however is important, is that we set the column widths as
    they will never be right otherwise. Note that the columns are
    numbered from 0 and up (to 1 in this case). }
  GtkCListSetColumnWidth(PGtkClist(CList), 0, 150);

  { Add the CList widget to the vertical box and show it. }
  GtkContainerAdd(PGtkContainer(Scrolled), CList);
  GtkWidgetShow(CList);

  { Create the buttons and add them to the window. See the button
    tutorial for more examples and comments on this. }
  HBox := GtkHBoxNeW(G_FALSE, 0);
  GtkBoxPackStart(PGtkBox(VBox), HBox, G_FALSE, G_TRUE, 0);
  GtkWidgetShow(HBox);

  ButtonAdd := GtkButtonNewWithLabel('Add list');
  ButtonClear := GtkButtonNewWithLabel('Clear list');
  ButtonHideShow := GtkButtonNewWithLabel('Hide/Show titles');

  GtkBoxPackStart(PGtkBox(HBox), ButtonAdd, G_TRUE, G_TRUE, 0);
  GtkBoxPackStart(PGtkBox(HBox), ButtonClear, G_TRUE, G_TRUE, 0);
  GtkBoxPackStart(PGtkBox(HBox), ButtonHideShow, G_TRUE, G_TRUE, 0);

  { Connect our callbacks to the three buttons }
  GtkSignalConnect(PGtkObject(ButtonAdd), 'clicked', @ButtonAddClicked, CList);
  GtkSignalConnect(PGtkObject(ButtonClear), 'clicked', @ButtonClearClicked, CList);
  GtkSignalConnect(PGtkObject(ButtonHideShow), 'clicked', @ButtonHideShowClicked, CList);

  GtkWidgetShow(ButtonAdd);
  GtkWidgetShow(ButtonClear);
  GtkWidgetShow(ButtonHideShow);

  { The interface is completely set up so we show the window and
    enter the gtk_main loop. }
  GtkWidgetShow(Window);
  GtkMain;
end.
