{ Report bugs to Nicola Girardi <nikke@swlibero.org> }
program GetTargets;

{$include "../units/gtk.inc"}
{$ignore-function-results}
{$x+}

{ Signal handler invoked when user clicks on the "Get Targets" button }
procedure GetTargets(Widget : PGtkWidget; Data : GPointer);
var
  TargetsAtom : static GdkAtom = GDK_NONE;
begin
  { Get the atom corresponding to the string "TARGETS" }
  if TargetsAtom = GDK_NONE then
    TargetsAtom := GdkAtomIntern('TARGETS', G_FALSE);

  { And request the "TARGETS" target for the primary selection }
  GtkSelectionConvert(Widget, GDK_SELECTION_PRIMARY, TargetsAtom, GDK_CURRENT_TIME);
end;

{ Signal handler called when the selections owner returns the data }
procedure SelectionReceived(Widget : PGtkWidget; SelectionData : PGtkSelectionData; Data : GPointer);
var
  Atoms : PGdkAtom;
  ItemList : PGList;
  i : Integer;
begin
  { **** IMPORTANT **** Check to see if retrieval succeeded  }
  if SelectionData^.Length < 0 then
    begin
      Writeln('Selection retrieval failed');
      Exit;
    end;

  { Make sure we got the data in the expected form }
  if SelectionData^.TType <> GDK_SELECTION_TYPE_ATOM then
    begin
      Writeln('Selection "TARGETS" was not returned as atoms!');
      Exit;
    end;
  
  { Print out the atoms we received }
  Atoms := PGdkAtom(GPointer(SelectionData^.Data));

  ItemList := nil;
  for i := 0 to (SelectionData^.Length div SizeOf(GdkAtom) - 1) do
    begin
      var Name : PGChar = GdkAtomName(PGdkAtom(Atoms + i * SizeOf(GdkAtom))^);
      if Name <> nil then
        Writeln(CString2String(Name))
      else
        Writeln('(bad atom)');
    end;
end;

var
  Window, Button : PGtkWidget;

begin
  GtkInit;

  { Create the toplevel window }
  Window := GtkWindowNew(GTK_WINDOW_TOPLEVEL);
  GtkWindowSetTitle(PGtkWindow(Window), 'Event Box');
  GtkContainerSetBorderWidth(PGtkContainer(Window), 10);

  GtkSignalConnect(PGtkObject(Window), 'destroy', @GtkExit, nil);

  { Create a button the user can click to get targets }
  Button := GtkButtonNewWithLabel('Get Targets');
  GtkContainerAdd(PGtkContainer(Window), Button);

  GtkSignalConnect(PGtkObject(Button), 'clicked', @GetTargets, nil);
  GtkSignalConnect(PGtkObject(Button), 'selection_received', @SelectionReceived, nil);

  GtkWidgetShow(Button);
  GtkWidgetShow(Window);
  
  GtkMain;
end.
