{ Report bugs to Nicola Girardi <nikke@swlibero.org> }
program HelloWorld;

{$include "../units/gtk.inc"}
{$ignore-function-results}

procedure HelloCB (Widget : PGtkWidget; Data : GPointer);
begin
  Writeln('Hello, world!');
end;

function DeleteEventCB (Widget : PGtkWidget; Event : PGdkEvent; Data : GPointer) : GInt;
begin
  { If you return FALSE in the "delete_event" signal handler,
    GTK will emit the "destroy" signal. Returning TRUE means
    you don't want the window to be destroyed. This is useful
    for popping up 'are you sure you want to quit?' type
    dialogs. }
  Writeln('Delete event occurred.');

  { Change TRUE to FALSE and the main window will be destroyed
    with a "delete_event". }
  DeleteEventCB := 1;
end;

procedure DestroyCB (Widget : PGtkWidget; Data : GPointer);
begin
  GtkMainQuit;
end;

var
  { GtkWidget is the storage type for widgets. }
  Window, Button : PGtkWidget;

begin
  { This is called in all GTK applications. Arguments are parsed
    from the command line. }
  GtkInit;

  { Create a new window. }
  Window := GtkWindowNew(GTK_WINDOW_TOPLEVEL);

  { When the window is given the "delete_event" signal (this is given
    by the window manager, usually by the "close" option, or on the
    titlebar), we ask it to call the delete_event () function as
    defined above. The data passed to the callback function is NULL
    and is ignored in the callback function. }
  GtkSignalConnect(PGtkObject(Window), 'delete_event', @DeleteEventCB, nil);

  { Here we connect the "destroy" event to a signal handler.
    This event occurs when we call gtk_widget_destroy() on the window,
    or if we return FALSE in the "delete_event" callback. }
  GtkSignalConnect(PGtkObject(Window), 'destroy', @DestroyCB, nil);

  { Sets the border width of the window. }
  GtkContainerSetBorderWidth (PGtkContainer(Window), 10);

  { Creates a new button with the label "Hello World". }
  Button := GtkButtonNewWithLabel('Hello World');

  { When the button receives the "clicked" signal, it will call the
    function hello() passing it NULL as its argument.  The hello()
    function is defined above. }
  GtkSignalConnect(PGtkObject(Button), 'clicked', @HelloCB, nil);

  { This will cause the window to be destroyed by calling
    gtk_widget_destroy(window) when "clicked".  Again, the destroy
    signal could come from here, or the window manager. }
  GtkSignalConnectObject(PGtkObject(Button), 'clicked', @GtkWidgetDestroy, PGtkObject(Window));

  { This packs the button into the window (a gtk container). }
  GtkContainerAdd(PGtkContainer(Window), Button);

  { The final step is to display this newly created widget. }
  GtkWidgetShow(Button);

  { And the window. }
  GtkWidgetShow(Window);

  { All GTK applications must have a gtk_main(). Control ends here
    and waits for an event to occur (like a key press or mouse event). }
  GtkMain;
end.

