{ Report bugs to Nicola Girardi <nikke@swlibero.org> }
program HelloWorld2;

{$include "../units/gtk.inc"}
{$ignore-function-results}

uses GPC;

var
  Button1_Str : TString = "Button1";
  Button2_Str : TString = "Button2";

{ Our new improved callback.  The data passed to this function
  is printed to stdout. }
procedure Callback (Widget : PGtkWidget; Data : GPointer);
begin
  Writeln('Hello again - ', PString(Data)^, ' was pressed.');
end;

{ Another callback. }
function DeleteCB (Widget : PGtkWidget; Event : PGdkEvent; Data : GPointer) : GInt;
begin
  GtkMainQuit;
  DeleteCB := 0;
end;

var
  { GtkWidget is the storage type for widgets. }
  Window, Button, Box : PGtkWidget;

begin
  { This is called in all GTK applications. Arguments are parsed
    from the command line. }
  GtkInit;

  { Create a new window. }
  Window := GtkWindowNew(GTK_WINDOW_TOPLEVEL);

  { This is a new call, which just sets the title of our
    new window to "Hello Buttons!". }
  GtkWindowSetTitle(PGtkWindow(Window), 'Hello Buttons!');

  { Here we just set a handler for delete_event that immediately exits GTK. }
  GtkSignalConnect(PGtkObject(Window), 'delete_event', @DeleteCB, nil);

  { Sets the border width of the window. }
  GtkContainerSetBorderWidth(PGtkContainer(Window), 10);

  { We create a box to pack widgets into.  This is described in detail
    in the "packing" section. The box is not really visible, it
    is just used as a tool to arrange widgets. }
  Box := GtkHBoxNew(G_FALSE, 0);

  { Put the box into the main window. }
  GtkContainerAdd(PGtkContainer(Window), Box);

  { Creates a new button with the label "Button 1". }
  Button := GtkButtonNewWithLabel('Button 1');

  { Now when the button is clicked, we call the "callback" function
    with a pointer to "button 1" as its argument. }
  GtkSignalConnect(PGtkObject(Button), 'clicked', @Callback, @Button1_Str);

  { Instead of gtk_container_add, we pack this button into the invisible
    box, which has been packed into the window. }
  GtkBoxPackStart(PGtkBox(Box), Button, G_TRUE, G_TRUE, 0);

  { Always remember this step, this tells GTK that our preparation for
    this button is complete, and it can now be displayed. }
  GtkWidgetShow(Button);

  { Do these same steps again to create a second button. }
  Button := GtkButtonNewWithLabel('Button 2');

  { Call the same callback function with a different argument,
    passing a pointer to "button 2" instead. }
  GtkSignalConnect(PGtkObject(Button), 'clicked', @Callback, @Button2_Str);

  GtkBoxPackStart(PGtkBox(Box), Button, G_TRUE, G_TRUE, 0);

  { The order in which we show the buttons is not really important, but I
    recommend showing the window last, so it all pops up at once. }
  GtkWidgetShow(Button);
  GtkWidgetShow(Box);
  GtkWidgetShow(Window);

  { Rest in gtk_main and wait for the fun to begin! }
  GtkMain;
end.
