{ Report bugs to Nicola Girardi <nikke@swlibero.org> }
program ItemFactory;

{$include "../units/gtk.inc"}
{$ignore-function-results}

{ Obligatory basic callback }
procedure PrintHello(W : PGtkWidget; Data : GPointer);
begin
  Writeln('Hello, World!');
end;

{ This Is The GtkItemFactoryEntry Structure Used To Generate New Menus.
  Item 1: The Menu Path. The Letter After The Underscore Indicates An
          Accelerator Key Once The Menu Is Open.
  Item 2: The Accelerator Key For The Entry
  Item 3: The Callback Function.
  Item 4: The Callback Action.  This Changes The Parameters With
          Which The Function Is Called.  The Default Is 0.
  Item 5: The Item Type, Used To Define What Kind Of An Item It Is.
          Here Are The Possible Values:

          nil               ^. '<Item>'
          '"                 -> "<Item>'
          '<Title>'          ^. Create A Title Item
          '<Item>'           ^. Create A Simple Item
          '<CheckItem>'      ^. Create A Check Item
          '<ToggleItem>'     ^. Create A Toggle Item
          '<RadioItem>'      ^. Create A Radio Item
          <path>             ^. Path Of A Radio Item To Link Against
          '<Separator>'      ^. Create A Separator
          '<Branch>'         ^. Create An Item To Hold Sub Items(optional)
          '<LastBranch>'     ^. Create A Right Justified Branch }

var
  MenuItems : static array[0..10] of TGtkItemFactoryEntry = (
    ( '/_File',         nil,          nil,          0, '<Branch>' ),
    ( '/File/_New',     '<control>N', @PrintHello,  0, nil ),
    ( '/File/_Open',    '<control>O', @PrintHello,  0, nil ),
    ( '/File/_Save',    '<control>S', @PrintHello,  0, nil ),
    ( '/File/Save _As', nil,          nil,          0, nil ),
    ( '/File/sep1',     nil,          nil,          0, '<Separator>' ),
    ( '/File/Quit',     '<control>Q', @GtkMainQuit, 0, nil ),
    ( '/_Options',      nil,          nil,          0, '<Branch>' ),
    ( '/Options/Test',  nil,          nil,          0, nil ),
    ( '/_Help',         nil,          nil,          0, '<LastBranch>' ),
    ( '/_Help/About',   nil,          nil,          0, nil )
  );

procedure GetMainMenu(Window : PGtkWidget; var MenuBar : PGtkWidget);
var
  ItemFactory : PGtkItemFactory;
  AccelGroup : PGtkAccelGroup;
const
  NMenuItems = 11;
begin
  AccelGroup := GtkAccelGroupNew;

  { This Function Initializes The Item Factory.
    Param 1: The Type Of Menu - Can Be GTK_TYPE_MENU_BAR, GTK_TYPE_MENU,
             Or GTK_TYPE_OPTION_MENU.
    Param 2: The Path Of The Menu.
    Param 3: A Pointer To A GtkAccelGroup.  The Item Factory Sets Up
             The Accelerator Table While Generating Menus. }

  ItemFactory := GtkItemFactoryNew(GTK_TYPE_MENU_BAR, '<main>', AccelGroup);

  { This Function Generates The Menu Items. Pass The Item Factory,
    The Number Of Items In The Array, The Array Itself, And Any
    Callback Data For The The Menu Items. }
  GtkItemFactoryCreateItems(ItemFactory, NMenuItems, @MenuItems[0], nil);

  { Attach the new accelerator group to the window. }
  GtkWindowAddAccelGroup(PGtkWindow(window), AccelGroup);

  { Finally, return the actual menu bar created by the item factory. }
  MenuBar := GtkItemFactoryGetWidget(ItemFactory, '<main>');
end;

var
  Window, MainVBox, MenuBar : PGtkWidget;

begin
  GtkInit;
  
  Window := GtkWindowNew(GTK_WINDOW_TOPLEVEL);
  GtkSignalConnect(PGtkObject(Window), 'destroy', @GtkMainQuit, @'WM destroy');
  GtkWindowSetTitle(PGtkWindow(Window), 'Item Factory');
  GtkWidgetSetUsize(Window, 300, 200);
  
  MainVbox := GtkVboxNew(G_FALSE, 1);
  GtkContainerSetBorderWidth(PGtkContainer(MainVbox), 1);
  GtkContainerAdd(PGtkContainer(Window), MainVbox);
  GtkWidgetShow(MainVbox);
  
  GetMainMenu(Window, Menubar);
  GtkBoxPackStart(PGtkBox(MainVbox), Menubar, G_FALSE, G_TRUE, 0);
  GtkWidgetShow(Menubar);
  
  GtkWidgetShow(Window);
  GtkMain;
end.
