{ Report Bugs To Nicola Girardi <nikke@swlibero.org> }
program List;

{$include "../units/gtk.inc"}
{$ignore-function-results}

{ This is our data identification string to store data in list items }
var
  ListItemDataKey : PGChar = 'list_item_data';

{ This is the signal handler that gets called if List emits the "selection_changed" signal }
procedure SighPrintSelection(GtkList : PGtkWidget; FuncData : GPointer);
var
  DList : PGList;
begin
  { Fetch the doubly linked list of selected items
    of the List, remember to treat this as read-only! }
  DList := PGtkList(GtkList)^.Selection;

  { If there are no selected items there is nothing more
    to do than just telling the user so }
  if DList = nil then
    begin
      Writeln('Selection cleared');
      Exit;
    end;

  { Ok, we got a selection and so we print it }
  Write('The selection is a ');

  { Get the list item from the doubly linked list
    and then query the data associated with list_item_data_key.
    We then just print it }
  while DList <> nil do
    begin
      var ListItem : PGtkObject;
      var ItemDataString : PGChar;

      ListItem := PGtkObject(DList^.Data);
      ItemDataString := GtkObjectGetData(ListItem, ListItemDataKey);

      Write(CString2String(ItemDataString), ' ');

      DList := DList^.Next;
    end;

    Writeln;
end;

{ This is the signal handler that got connected to button press/release events of the List }
procedure SighButtonEvent(GtkList : PGtkWidget; Event : PGdkEventButton; Frame : PGtkWidget);
begin
  { We only do something if the third (rightmost mouse button was released) }
  if (Event^.TType = GDK_BUTTON_RELEASE) and (Event^.Button = 3) then
    begin
      var DList, FreeList : PGList;
      var NewPrisoner : PGtkWidget;

      { Fetch the currently selected list item which will be our next prisoner ;) }
      DList := PGtkList(GtkList)^.Selection;
      if Dlist <> nil then
        NewPrisoner := PGtkWidget(DList^.Data)
      else
        NewPrisoner := nil;

      { Look for already imprisoned list items, we
        will put them back into the list.
        Remember to free the doubly linked list that
        gtk_container_children() returns }
      DList := GtkContainerChildren(PGtkContainer(Frame));

      FreeList := DList;
      while DList <> nil do
        begin
          var ListItem : PGtkWidget;
          ListItem := PGtkWidget(DList^.Data);
          GtkWidgetReparent(ListItem, GtkList);
          DList := DList^.Next;
        end;
      GListFree(FreeList);

      { If we have a new prisoner, remove him from the
         List and put him into the frame "Prison".
         We need to unselect the item first. }
      if NewPrisoner <> nil then
        begin
          GtkListUnselectChild(PGtkList(GtkList), NewPrisoner);
          GtkWidgetReparent(NewPrisoner, Frame);
        end;
    end;
end;

var
  Separator, Window, VBox, Scrolled, Frame, GtkList, Button, ListItem : PGtkWidget;
  DList : PGList;
  i : Word;
  Buffer : String(64);

{ Main function to set up the user interface }
begin
  { Initialize GTK (and subsequently GDK) }
  GtkInit;

  { Create a window to put all the widgets in
    connect gtk_main_quit() to the "destroy" event of
    the window to handle window manager close-window-events }
  Window := GtkWindowNew(GTK_WINDOW_TOPLEVEL);
  GtkWindowSetTitle(PGtkWindow(Window), 'GtkList Example');
  GtkSignalConnect(PGtkObject(Window), 'destroy', @GtkMainQuit, nil);

  { Inside the window we need a box to arrange the widgets vertically }
  VBox := GtkVBoxNew(G_FALSE, 5);
  GtkContainerSetBorderWidth(PGtkContainer(Window), 5);
  GtkContainerAdd(PGtkContainer(Window), VBox);
  GtkWidgetShow(VBox);

  { This is the scrolled window to put the List widget inside }
  Scrolled := GtkScrolledWindowNew(nil, nil);
  GtkWidgetSetUSize(Scrolled, 250, 150);
  GtkContainerAdd(PGtkContainer(VBox), Scrolled);
  GtkWidgetShow(Scrolled);

  { Create the GtkList widget.
    Connect the sigh_print_selection() signal handler
    function to the "selection_changed" signal of the List
    to print out the selected items each time the selection
    has changed }
  GtkList := GtkListNew;
  GtkScrolledWindowAddWithViewport(PGtkScrolledWindow(Scrolled), GtkList);
  GtkWidgetShow(GtkList);
  GtkSignalConnect(PGtkObject(GtkList), 'selection_changed', @SighPrintSelection, nil);

  { We create a "Prison" to put a list item in ;) }
  Frame := GtkFrameNew('Prison');
  GtkWidgetSetUSize(Frame, 200, 50);
  GtkContainerSetBorderWidth(PGtkContainer(Frame), 5);
  GtkFrameSetShadowType(PGtkFrame(Frame), GTK_SHADOW_OUT);
  GtkBoxPackStart(PGtkBox(VBox), Frame, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(Frame);

  { Connect the sigh_button_event() signal handler to the List
    which will handle the "arresting" of list items }
  GtkSignalConnect(PGtkObject(GtkList), 'button_release_event', @SighButtonEvent, Frame);

  { Create a separator }
  Separator := GtkHSeparatorNew;
  GtkContainerAdd(PGtkContainer(VBox), Separator);
  GtkWidgetShow(Separator);

  { Finally create a button and connect its "clicked" signal
    to the destruction of the window }
  Button := GtkButtonNewWithLabel('Close');
  GtkBoxPackStart(PGtkBox(VBox), Button, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(Button);
  GtkSignalConnectObject(PGtkObject(Button), 'clicked', @GtkWidgetDestroy, PGtkObject(Window));

  { Now we create 5 list items, each having its own
    label and add them to the List using gtk_container_add()
    Also we query the text string from the label and
    associate it with the list_item_data_key for each list item }
  for i := 1 to 5 do
    begin
      var LLabel : PGtkWidget;
      var SString : PGChar;

      WriteStr(Buffer, 'ListItemContainer with Label #', i);
      LLabel := GtkLabelNew(Buffer);
      ListItem := GtkListItemNew;
      GtkContainerAdd(PGtkContainer(ListItem), LLabel);
      GtkWidgetShow(LLabel);
      GtkContainerAdd(PGtkContainer(GtkList), ListItem);
      GtkWidgetShow(ListItem);
      GtkLabelGet(PGtkLabel(LLabel), SString);
      GtkObjectSetData(PGtkObject(ListItem), ListItemDataKey, SString);
    end;

  { Here, we are creating another 5 labels, this time
    we use gtk_list_item_new_with_label() for the creation
    we can't query the text string from the label because
    we don't have the labels pointer and therefore
    we just associate the list_item_data_key of each
    list item with the same text string.
    For adding of the list items we put them all into a doubly
    linked list (GList), and then add them by a single call to
    gtk_list_append_items().
    Because we use g_list_prepend() to put the items into the
    doubly linked list, their order will be descending (instead
    of ascending when using g_list_append()) }
  DList := nil;
  for i := 4 to 9 do
    begin
      WriteStr(Buffer, 'List Item with Label ', i);
      ListItem := GtkListItemNewWithLabel(Buffer);
      DList := GListPrepend(DList, ListItem);
      GtkWidgetShow(ListItem);
      GtkObjectSetData(PGtkObject(ListItem), ListItemDataKey, @'ListItem with integrated Label');
    end;
  GtkListAppendItems(PGtkList(GtkList), DList);
    
  { Finally we want to see the window, don't we? ;) }
  GtkWidgetShow(Window);
    
  { Fire up the main event loop of gtk }
  GtkMain;

  { We get here after gtk_main_quit() has been called which
    happens if the main window gets destroyed }
end.
