{ Report bugs to Nicola Girardi <nikke@swlibero.org> }
program Notebook;

{$include "../units/gtk.inc"}
{$ignore-function-results}

uses GPC;

{ This function rotates the position of the tabs }
procedure RotateBook(Button : PGtkButton; Notebook : PGtkNotebook);
begin
  GtkNotebookSetTabPos(Notebook, (Notebook^.Frame1.TabPos+1) mod 4);
end;

{ Add/Remove the page tabs and the borders }
procedure TabsBorderBook(Button : PGtkButton; Notebook : PGtkNotebook);
var
  TVal, BVal : GBoolean := G_FALSE;
begin
  if Notebook^.Frame1.ShowTabs = G_FALSE then
    TVal := G_TRUE;
  if Notebook^.Frame1.ShowBorder = G_FALSE then
    BVal := G_TRUE;

  GtkNotebookSetShowTabs(Notebook, TVal);
  GtkNotebookSetShowBorder(Notebook, BVal);
end;

{ Remove a page from the notebook }
procedure RemoveBook(Button : PGtkButton; Notebook : PGtkNotebook);
var
  Page : GInt;
begin
  Page := GtkNotebookGetCurrentPage(Notebook);
  GtkNotebookRemovePage(Notebook, Page);
  { Need to refresh the widget --
    This forces the widget to redraw itself. }
  GtkWidgetDraw(PGtkWidget(Notebook), nil);
end;

function Delete(Widget, Event : PGtkWidget; Data : GPointer) : GInt;
begin
  GtkMainQuit;
  Delete := G_FALSE;
end;

var
  Window, Button, Table, Notebook, Frame, LLabel, CheckButton : PGtkWidget;
  I : Integer;
  BufferF, BufferL : TString;
begin
  GtkInit;

  Window := GtkWindowNew(GTK_WINDOW_TOPLEVEL);
  GtkSignalConnect(PGtkObject(Window), 'delete_event', @Delete, nil);
  GtkContainerSetBorderWidth(PGtkContainer(Window), 10);

  Table := GtkTableNew(3, 6, G_FALSE);
  GtkContainerAdd(PGtkContainer(Window), Table);

  { Create a new notebook, place the position of the tabs }
  Notebook := GtkNotebookNew;
  GtkNotebookSetTabPos(PGtkNotebook(Notebook), GTK_POS_TOP);
  GtkTableAttachDefaults(PGtkTable(Table), Notebook, 0, 6, 0, 1);
  GtkWidgetShow(Notebook);

  { Let's append a bunch of pages to the notebook }
  for I := 1 to 5 do
    begin
      WriteStr(BufferF, 'Append Frame ', I);
      WriteStr(BufferL, 'Page ', I);

      Frame := GtkFrameNew(BufferF);
      GtkContainerSetBorderWidth(PGtkContainer(Frame), 10);
      GtkWidgetSetUSize(Frame, 100, 75);
      GtkWidgetShow(Frame);

      LLabel := GtkLabelNew(BufferF);
      GtkContainerAdd(PGtkContainer(Frame), LLabel);
      GtkWidgetShow(LLabel);

      LLabel := GtkLabelNew(BufferL);
      GtkNotebookAppendPage(PGtkNotebook(Notebook), Frame, LLabel);
    end;

  { Now let's add a page to a specific spot }
  CheckButton := GtkCheckButtonNewWithLabel('Check me please!');
  GtkWidgetSetUSize(CheckButton, 100, 75);
  GtkWidgetShow(CheckButton);

  LLabel := GtkLabelNew('Add page');
  GtkNotebookInsertPage(PGtkNotebook(Notebook), CheckButton, LLabel, 2);

  { Now finally let's prepend pages to the notebook }
  for I := 1 to 5 do
    begin
      WriteStr(BufferF, 'Prepend Frame ', I);
      WriteStr(BufferL, 'PPage ', I);

      Frame := GtkFrameNew(BufferF);
      GtkContainerSetBorderWidth(PGtkContainer(Frame), 10);
      GtkWidgetSetUSize(Frame, 100, 75);
      GtkWidgetShow(Frame);

      LLabel := GtkLabelNew(BufferF);
      GtkContainerAdd(PGtkContainer(Frame), LLabel);
      GtkWidgetShow(LLabel);

      LLabel := GtkLabelNeW(BufferL);
      GtkNotebookPrependPage(PGtkNotebook(Notebook), Frame, LLabel);
    end;

  { Set what page to start at (page 4) }
  GtkNotebookSetPage(PGtkNotebook(Notebook), 3);

  { Create a bunch of buttons }
  Button := GtkButtonNewWithLabel('Close');
  GtkSignalConnectObject(PGtkObject(Button), 'clicked', @Delete, nil);
  GtkTableAttachDefaults(PGtkTable(Table), Button, 0, 1, 1, 2);
  GtkWidgetShow(Button);

  Button := GtkButtonNewWithLabel('Next page');
  GtkSignalConnectObject(PGtkObject(Button), 'clicked', @GtkNotebookNextPage, PGtkObject(Notebook));
  GtkTableAttachDefaults(PGtkTable(Table), Button, 1, 2, 1, 2);
  GtkWidgetShow(Button);

  Button := GtkButtonNewWithLabel('Prev page');
  GtkSignalConnectObject(PGtkObject(Button), 'clicked', @GtkNotebookPrevPage, PGtkObject(Notebook));
  GtkTableAttachDefaults(PGtkTable(Table), Button, 2, 3, 1, 2);
  GtkWidgetShow(Button);

  Button := GtkButtonNewWithLabel('Tab position');
  GtkSignalConnect(PGtkObject(Button), 'clicked', @RotateBook, Notebook);
  GtkTableAttachDefaults(PGtkTable(Table), Button, 3, 4, 1, 2);
  GtkWidgetShow(Button);

  Button := GtkButtonNewWithLabel('Tabs/Border On/Off');
  GtkSignalConnect(PGtkObject(Button), 'clicked', @TabsBorderBook, Notebook);
  GtkTableAttachDefaults(PGtkTable(Table), Button, 4, 5, 1, 2);
  GtkWidgetShow(Button);

  Button := GtkButtonNewWithLabel('Remove page');
  GtkSignalConnect(PGtkObject(Button), 'clicked', @RemoveBook, Notebook);
  GtkTableAttachDefaults(PGtkTable(Table), Button, 5, 6, 1, 2);
  GtkWidgetShow(Button);

  GtkWidgetShow(Table);
  GtkWidgetShow(Window);

  GtkMain;
end.
