{ Report bugs to Nicola Girardi <nikke@swlibero.org> }
program Packer;

{$include "../units/gtk.inc"}
{$ignore-function-results}

{$define IS_SET(flag, bit) ((flag and bit) <> 0)}

type
  PInfo = ^TInfo;
  TInfo = record
    Widgets : PGList;
    Packer, Current : PGtkWidget;
    PChild : PGtkPackerChild;

    ButtonTop, ButtonBottom, ButtonLeft, ButtonRight, ButtonN, ButtonNE,
    ButtonNW, ButtonE, ButtonW, ButtonS, ButtonSE, ButtonSW, ButtonCenter,
    ButtonFillX, ButtonFillY, ButtonExpand : PGtkWidget;
  end;

procedure Destroy(Widget : PGtkWidget; Data : GPointer);
begin
  GtkMainQuit;
end;

procedure ToggleOptions(Widget : PGtkWidget; Data : GPointer);
var
  Info : PInfo;
  Option : Integer;
  PChild : PGtkPackerChild;
  FillX, FillY, Expand : Integer;
begin
  Info := PInfo(Data);
  Option := Integer(GtkObjectGetData(PGtkObject(Widget), 'option'));
  PChild := Info^.PChild;

  if PChild = nil
    then Halt(1);

  FillX := 0;
  FillY := 0;
  Expand := 0;

  if GtkToggleButtonGetActive(PGtkToggleButton(Info^.ButtonFillX)) <> G_FALSE then
    FillX := GTK_FILL_X;
  if GtkToggleButtonGetActive(PGtkToggleButton(Info^.ButtonFillY)) <> G_FALSE then
    FillY := GTK_FILL_Y;
  if GtkToggleButtonGetActive(PGtkToggleButton(Info^.ButtonExpand)) <> G_FALSE then
    Expand := GTK_PACK_EXPAND;

  GtkPackerSetChildPacking(PGtkPacker(Info^.Packer), Info^.Current, PChild^.Side, PChild^.Anchor, Fillx or Filly or Expand, PChild^.BorderWidth, PChild^.PadX, PChild^.PadY, PChild^.IPadX, PChild^.IPadY);
end;

procedure ToggleAnchor(Widget : PGtkWidget; Data : GPointer);
var
  Info : PInfo;
  Anchor : Integer;
  PChild : PGtkPackerChild;
begin
  Info := PInfo(Data);

  if GtkToggleButtonGetActive(PGtkToggleButton(Widget)) <> G_FALSE then
    Anchor := Integer(GtkObjectGetData(PGtkObject(Widget), 'anchor'));

  PChild := Info^.PChild;
  if PChild = nil then
    Halt(1);

  GtkPackerSetChildPacking(PGtkPacker(Info^.Packer), Info^.Current, PChild^.Side, Anchor, PChild^.Options, PChild^.BorderWidth, PChild^.PadX, PChild^.PadY, PChild^.IPadX, PChild^.IPadY);

  if Info^.ButtonN <> Widget then
    begin
      GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonN), 0);
      GtkWidgetSetSensitive(Info^.ButtonN, 1);
    end;
  if Info^.ButtonNW <> Widget then
    begin
      GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonNW), 0);
      GtkWidgetSetSensitive(Info^.ButtonNW, 1);
    end;
  if Info^.ButtonNE <> Widget then
    begin
      GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonNE), 0);
      GtkWidgetSetSensitive(Info^.ButtonNE, 1);
    end;
  if Info^.ButtonS <> Widget then
    begin
      GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonS), 0);
      GtkWidgetSetSensitive(Info^.ButtonS, 1);
    end;
  if Info^.ButtonSW <> Widget then
    begin
      GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonSW), 0);
      GtkWidgetSetSensitive(Info^.ButtonSW, 1);
    end;
  if Info^.ButtonSE <> Widget then
    begin
      GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonSE), 0);
      GtkWidgetSetSensitive(Info^.ButtonSE, 1);
    end;
  if Info^.ButtonE <> Widget then
    begin
      GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonE), 0);
      GtkWidgetSetSensitive(Info^.ButtonE, 1);
    end;
  if Info^.ButtonW <> Widget then
    begin
      GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonW), 0);
      GtkWidgetSetSensitive(Info^.ButtonW, 1);
    end;
  if Info^.ButtonCenter <> Widget then
    begin
      GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonCenter), 0);
      GtkWidgetSetSensitive(Info^.ButtonCenter, 1);
    end;

  GtkWidgetSetSensitive(Widget, 0);
end;

procedure ToggleSide(Widget : PGtkWidget; Data : GPointer);
var
  Info : PInfo;
  Side : Integer;
  PChild : PGtkPackerChild;
begin
  Info := PInfo(Data);

  if GtkToggleButtonGetActive(PGtkToggleButton(Widget)) <> G_FALSE then
    Side := Integer(GtkObjectGetData(PGtkObject(Widget), 'side'));

  PChild := Info^.PChild;
  if PChild = nil then
    Halt(1);

  GtkPackerSetChildPacking(PGtkPacker(Info^.Packer), Info^.Current, Side, Pchild^.Anchor, Pchild^.Options, Pchild^.BorderWidth, Pchild^.PadX, Pchild^.PadY, Pchild^.IPadX, Pchild^.IPadY);

  if Info^.ButtonTop <> Widget then
    begin
      GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonTop), 0);
      GtkWidgetSetSensitive(Info^.ButtonTop, 1);
    end;

  if Info^.ButtonBottom <> Widget then
    begin
      GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonBottom), 0);
      GtkWidgetSetSensitive(Info^.ButtonBottom, 1);
    end;

  if Info^.ButtonLeft <> Widget then
    begin
      GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonLeft), 0);
      GtkWidgetSetSensitive(Info^.ButtonLeft, 1);
    end;

  if Info^.ButtonRight <> Widget then
    begin
      GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonRight), 0);
      GtkWidgetSetSensitive(Info^.ButtonRight, 1);
    end;

  GtkWidgetSetSensitive(Widget, 0);
end;

procedure SetWidget(W : PGtkWidget; Data : GPointer);
var
  Info : PInfo;
  List : PGList;
  Widget : PGtkWidget;
  PChild : PGtkPackerChild;
  Options : Integer;
begin
  if GtkToggleButtonGetActive(PGtkToggleButton(W)) <> G_FALSE then
    begin
      Info := PInfo(Data);
      Info^.Current := W;

      PChild := nil;
      List := GListFirst(PGtkPacker(Info^.Packer)^.Children);
      while List <> nil do
        begin
          if PGtkPackerChild(List^.Data)^.Widget = Info^.Current then
            begin
              PChild := PGtkPackerChild(List^.Data);
              Break;
            end;
           List := List^.Next;
        end;
      if PChild = nil then
        Halt(1);

      Info^.PChild := PChild;

      if PChild^.Side = GTK_SIDE_TOP then
        GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonTop), 1)
      else if PChild^.Side = GTK_SIDE_BOTTOM then
        GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonBottom), 1)
      else if PChild^.Side = GTK_SIDE_LEFT then
        GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonLeft), 1)
      else if PChild^.Side = GTK_SIDE_RIGHT then
        GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonRight), 1)
      else
        Writeln('foo... side = ', PChild^.side);

      if PChild^.Anchor = GTK_ANCHOR_N then
        GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonN), 1)
      else if PChild^.Anchor = GTK_ANCHOR_NW then
        GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonNW), 1)
      else if PChild^.Anchor = GTK_ANCHOR_NE then
        GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonNE), 1)
      else if PChild^.Anchor = GTK_ANCHOR_S then
        GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonS), 1)
      else if PChild^.Anchor = GTK_ANCHOR_SW then
        GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonSW), 1)
      else if PChild^.Anchor = GTK_ANCHOR_SE then
        GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonSE), 1)
      else if PChild^.Anchor = GTK_ANCHOR_W then
        GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonW), 1)
      else if PChild^.Anchor = GTK_ANCHOR_E then
        GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonE), 1)
      else if PChild^.Anchor = GTK_ANCHOR_CENTER then
        GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonCenter), 1);

      Options := PChild^.Options;
      if IS_SET(Options, GTK_PACK_EXPAND) then
        GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonExpand), 1)
      else
        GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonExpand), 0);
      if IS_SET(Options, GTK_FILL_X) then
        GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonFillX), 1)
      else
        GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonFillX), 0);
      if IS_SET(Options, GTK_FILL_Y) then
        GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonFillY), 1)
      else
        GtkToggleButtonSetActive(PGtkToggleButton(Info^.ButtonFillY), 0);
      GtkWidgetSetSensitive(W, 0);

      List := GListFirst(Info^.Widgets);
      while List <> nil do
        begin
          Widget := PGtkWidget(List^.Data);
          if Widget <> Info^.Current then
            begin
              GtkToggleButtonSetActive(PGtkToggleButton(Widget), 0);
              GtkWidgetSetSensitive(Widget, 1);
            end;
          List := List^.Next;
        end;
   end;
end;

procedure AddWidget(W : PGtkWidget; Data : GPointer);
var
  N : static Integer = 0;
  Packer : PGtkPacker;
  Widget : PGtkWidget;
  Str : String(255);
  Info : PInfo;
begin
  Info := PInfo(Data);
  Packer := PGtkPacker(Info^.Packer);

  WriteStr(Str, N);
  Widget := GtkToggleButtonNewWithLabel(Str);
  GtkWidgetSetUsize(Widget, 50, 50);
  GtkContainerAdd(PGtkContainer(Packer), Widget);
  GtkWidgetShow(Widget);

  GtkSignalConnect(PGtkObject(Widget), 'toggled', @SetWidget, Info);
  Info^.Widgets := GListAppend(Info^.Widgets, Widget);
  GtkToggleButtonSetActive(PGtkToggleButton(Widget), 1);
  SetWidget(Widget, Info);

  Inc(N);
end;

var
  Window, WindowPack, TopPack, Frame, Packer, ButtonPack, ButtonAdd,
  ButtonQuit, BottomPack, SideFrame, SidePack, ButtonTop, ButtonBottom,
  ButtonLeft, ButtonRight, AnchorTable, AnchorFrame, ButtonN, ButtonNE,
  ButtonNW, ButtonE, ButtonW, ButtonCenter, ButtonS, ButtonSE, ButtonSW,
  ButtonExpand, ButtonFillX, ButtonFillY, OptionsPack, OptionsFrame,
  AnchorPack : PGtkWidget;
  Info : PInfo;

begin
  GtkInit;

  New(Info);

  Window := GtkWindowNew(GTK_WINDOW_TOPLEVEL);
  GtkSignalConnect(PGtkObject(Window), 'destroy', @Destroy, nil);
  WindowPack := GtkPackerNew;
  GtkContainerAdd(PGtkContainer(Window), WindowPack);
  GtkContainerSetBorderWidth(PGtkContainer(Window), 4);

  TopPack := GtkPackerNew;
  GtkPackerAddDefaults(PGtkPacker(WindowPack), TopPack, GTK_SIDE_TOP, GTK_ANCHOR_CENTER, GTK_FILL_X or GTK_FILL_Y or GTK_EXPAND);

  Frame := GtkFrameNew('Packing Area');
  GtkWidgetSetUsize(Frame, 400, 400);
  GtkPackerAdd(PGtkPacker(TopPack), Frame, GTK_SIDE_LEFT, GTK_ANCHOR_CENTER, GTK_FILL_X or GTK_FILL_Y or GTK_EXPAND, 0, 8, 8, 0, 0);
  Packer := GtkPackerNew;
  GtkContainerAdd(PGtkContainer(Frame), Packer);

  ButtonPack := GtkPackerNew;
  GtkPackerAdd(PGtkPacker(TopPack), ButtonPack, GTK_SIDE_LEFT, GTK_ANCHOR_N, 0, 0, 0, 0, 0, 0);

  ButtonAdd := GtkButtonNewWithLabel('Add Button');
  GtkPackerAdd(PGtkPacker(TopPack), ButtonAdd, GTK_SIDE_TOP, GTK_ANCHOR_CENTER, GTK_FILL_X, 0, 8, 8, 8, 0);
  GtkSignalConnect(PGtkObject(ButtonAdd), "clicked", @AddWidget, Info);

  ButtonQuit := GtkButtonNewWithLabel('Quit');
  GtkPackerAdd(PGtkPacker(TopPack), ButtonQuit, GTK_SIDE_TOP, GTK_ANCHOR_CENTER, GTK_FILL_X, 0, 8, 8, 0, 0);
  GtkSignalConnectObject(PGtkObject(ButtonQuit), 'clicked', @GtkWidgetDestroy, PGtkObject(Window));

  BottomPack := GtkPackerNew;
  GtkPackerAddDefaults(PGtkPacker(WindowPack), BottomPack, GTK_SIDE_TOP, GTK_ANCHOR_CENTER, GTK_FILL_X);
  
  SideFrame := GtkFrameNew("Side");
  GtkPackerAdd(PGtkPacker(WindowPack), SideFrame, GTK_SIDE_LEFT, GTK_ANCHOR_W, GTK_FILL_Y, 0, 10, 10, 0, 0);

  SidePack := GtkPackerNew;
  GtkContainerAdd(PGtkContainer(SideFrame), SidePack);
    
  ButtonTop := GtkToggleButtonNewWithLabel('Top');
  ButtonBottom := GtkToggleButtonNewWithLabel('Bottom');
  ButtonLeft := GtkToggleButtonNewWithLabel('Left');
  ButtonRight := GtkToggleButtonNewWithLabel('Right');

  GtkObjectSetData(PGtkObject(ButtonTop), 'side', Pointer(GTK_SIDE_TOP));
  GtkObjectSetData(PGtkObject(ButtonBottom), 'side', Pointer(GTK_SIDE_BOTTOM));
  GtkObjectSetData(PGtkObject(ButtonLeft), 'side', Pointer(GTK_SIDE_LEFT));
  GtkObjectSetData(PGtkObject(ButtonRight), 'side', Pointer(GTK_SIDE_RIGHT));

  GtkWidgetSetUsize(ButtonTop, 50, -1);
  GtkWidgetSetUsize(ButtonBottom, 50, -1);
  GtkWidgetSetUsize(ButtonLeft, 50, -1);
  GtkWidgetSetUsize(ButtonRight, 50, -1);
  GtkPackerAdd(PGtkPacker(SidePack), ButtonTop, GTK_SIDE_TOP, GTK_ANCHOR_CENTER, 0, 0, 5, 5, 0, 0);
  GtkPackerAdd(PGtkPacker(SidePack), ButtonBottom, GTK_SIDE_BOTTOM, GTK_ANCHOR_CENTER, 0, 0, 5, 5, 0, 0);
  GtkPackerAdd(PGtkPacker(SidePack), ButtonLeft, GTK_SIDE_LEFT, GTK_ANCHOR_CENTER, 0, 0, 10, 5, 0, 0);
  GtkPackerAdd(PGtkPacker(SidePack), ButtonRight, GTK_SIDE_RIGHT, GTK_ANCHOR_CENTER, 0, 0, 10, 5, 0, 0);
  GtkSignalConnect(PGtkObject(ButtonTop), 'toggled', @ToggleSide, Info);
  GtkSignalConnect(PGtkObject(ButtonBottom), 'toggled', @ToggleSide, Info);
  GtkSignalConnect(PGtkObject(ButtonLeft), 'toggled', @ToggleSide, Info);
  GtkSignalConnect(PGtkObject(ButtonRight), 'toggled', @ToggleSide, Info);

  AnchorFrame := GtkFrameNew('Anchor');
  GtkPackerAdd(PGtkPacker(WindowPack), AnchorFrame, GTK_SIDE_LEFT, GTK_ANCHOR_W, GTK_FILL_Y, 0, 10, 10, 0, 0);

  AnchorPack := GtkPackerNew;
  GtkContainerAdd(PGtkContainer(AnchorFrame), AnchorPack);

  AnchorTable := GtkTableNew(3, 3, G_TRUE);
  GtkPackerAdd(PGtkPacker(AnchorPack), AnchorTable, GTK_SIDE_TOP, GTK_ANCHOR_CENTER, GTK_FILL_Y or GTK_FILL_X or GTK_PACK_EXPAND, 0, 10, 5, 0, 0);

  ButtonN := GtkToggleButtonNewWithLabel('N');
  ButtonS := GtkToggleButtonNewWithLabel('S');
  ButtonW := GtkToggleButtonNewWithLabel('W');
  ButtonE := GtkToggleButtonNewWithLabel('E');
  ButtonNE := GtkToggleButtonNewWithLabel('NE');
  ButtonNW := GtkToggleButtonNewWithLabel('NW');
  ButtonSE := GtkToggleButtonNewWithLabel('SE');
  ButtonSW := GtkToggleButtonNewWithLabel('SW');
  ButtonCenter := GtkToggleButtonNewWithLabel('');

  GtkObjectSetData(PGtkObject(ButtonN), 'anchor', Pointer(GTK_ANCHOR_N));
  GtkObjectSetData(PGtkObject(ButtonNW), 'anchor', Pointer(GTK_ANCHOR_NW));
  GtkObjectSetData(PGtkObject(ButtonNE), 'anchor', Pointer(GTK_ANCHOR_NE));
  GtkObjectSetData(PGtkObject(ButtonS), 'anchor', Pointer(GTK_ANCHOR_S));
  GtkObjectSetData(PGtkObject(ButtonSW), 'anchor', Pointer(GTK_ANCHOR_SW));
  GtkObjectSetData(PGtkObject(ButtonSE), 'anchor', Pointer(GTK_ANCHOR_SE));
  GtkObjectSetData(PGtkObject(ButtonW), 'anchor', Pointer(GTK_ANCHOR_W));
  GtkObjectSetData(PGtkObject(ButtonE), 'anchor', Pointer(GTK_ANCHOR_E));
  GtkObjectSetData(PGtkObject(ButtonCenter), 'anchor', Pointer(GTK_ANCHOR_CENTER));

  GtkSignalConnect(PGtkObject(ButtonN), 'toggled', @ToggleAnchor, Info);
  GtkSignalConnect(PGtkObject(ButtonNW), 'toggled', @ToggleAnchor, Info);
  GtkSignalConnect(PGtkObject(ButtonNE), 'toggled', @ToggleAnchor, Info);
  GtkSignalConnect(PGtkObject(ButtonS), 'toggled', @ToggleAnchor, Info);
  GtkSignalConnect(PGtkObject(ButtonSW), 'toggled', @ToggleAnchor, Info);
  GtkSignalConnect(PGtkObject(ButtonSE), 'toggled', @ToggleAnchor, Info);
  GtkSignalConnect(PGtkObject(ButtonW), 'toggled', @ToggleAnchor, Info);
  GtkSignalConnect(PGtkObject(ButtonE), 'toggled', @ToggleAnchor, Info);
  GtkSignalConnect(PGtkObject(ButtonCenter), 'toggled', @ToggleAnchor, Info);

  GtkTableAttachDefaults(PGtkTable(AnchorTable), ButtonNW, 0, 1, 0, 1);
  GtkTableAttachDefaults(PGtkTable(AnchorTable), ButtonN, 1, 2, 0, 1);
  GtkTableAttachDefaults(PGtkTable(AnchorTable), ButtonNE, 2, 3, 0, 1);
  GtkTableAttachDefaults(PGtkTable(AnchorTable), ButtonW, 0, 1, 1, 2);
  GtkTableAttachDefaults(PGtkTable(AnchorTable), ButtonCenter, 1, 2, 1, 2);
  GtkTableAttachDefaults(PGtkTable(AnchorTable), ButtonE, 2, 3, 1, 2);
  GtkTableAttachDefaults(PGtkTable(AnchorTable), ButtonSW, 0, 1, 2, 3);
  GtkTableAttachDefaults(PGtkTable(AnchorTable), ButtonS, 1, 2, 2, 3);
  GtkTableAttachDefaults(PGtkTable(AnchorTable), ButtonSE, 2, 3, 2, 3);

  OptionsFrame := GtkFrameNew('Options');
  GtkPackerAdd(PGtkPacker(WindowPack), OptionsFrame, GTK_SIDE_LEFT, GTK_ANCHOR_W, GTK_FILL_Y, 0, 10, 10, 0, 0);

  OptionsPack := GtkPackerNew;
  GtkContainerAdd(PGtkContainer(OptionsFrame), OptionsPack);

  ButtonFillX := GtkToggleButtonNewWithLabel('Fill X');
  ButtonFillY := GtkToggleButtonNewWithLabel('Fill Y');
  ButtonExpand := GtkToggleButtonNewWithLabel('Expand');

  GtkPackerAdd(PGtkPacker(OptionsPack), ButtonFillX, GTK_SIDE_TOP, GTK_ANCHOR_N, GTK_FILL_X or GTK_PACK_EXPAND, 0, 10, 5, 0, 0);
  GtkPackerAdd(PGtkPacker(OptionsPack), ButtonFillY, GTK_SIDE_TOP, GTK_ANCHOR_CENTER, GTK_FILL_X or GTK_PACK_EXPAND, 0, 10, 5, 0, 0);
  GtkPackerAdd(PGtkPacker(OptionsPack), ButtonExpand, GTK_SIDE_TOP, GTK_ANCHOR_S, GTK_FILL_X or GTK_PACK_EXPAND, 0, 10, 5, 0, 0);

  GtkObjectSetData(PGtkObject(ButtonFillX), 'option', Pointer(GTK_FILL_X));
  GtkObjectSetData(PGtkObject(ButtonFillY), 'option', Pointer(GTK_FILL_Y));
  GtkObjectSetData(PGtkObject(ButtonExpand), 'option', Pointer(GTK_PACK_EXPAND));

  GtkSignalConnect(PGtkObject(ButtonFillX), 'toggled', @ToggleOptions, Info);
  GtkSignalConnect(PGtkObject(ButtonFillY), 'toggled', @ToggleOptions, Info);
  GtkSignalConnect(PGtkObject(ButtonExpand), 'toggled', @ToggleOptions, Info);

  Info^.Widgets := nil;
  Info^.Packer := Packer;
  Info^.ButtonTop := ButtonTop;
  Info^.ButtonBottom := ButtonBottom;
  Info^.ButtonLeft := ButtonLeft;
  Info^.ButtonRight := ButtonRight;
  Info^.ButtonN := ButtonN;
  Info^.ButtonNW := ButtonNW;
  Info^.ButtonNE := ButtonNE;
  Info^.ButtonE := ButtonE;
  Info^.ButtonW := ButtonW;
  Info^.ButtonCenter := ButtonCenter;
  Info^.ButtonS := ButtonS;
  Info^.ButtonSW := ButtonSW;
  Info^.ButtonSE := ButtonSE;
  Info^.ButtonFillX := ButtonFillX;
  Info^.ButtonFillY := ButtonFillY;
  Info^.ButtonExpand := ButtonExpand;

  AddWidget(nil, Info);
  GtkWidgetShowAll(Window);

  GtkMain;
end.
