{ Report bugs to Nicola Girardi <nikke@swlibero.org> }
program PackBox;

{$include "../units/gtk.inc"}
{$ignore-function-results}

uses GPC;

function DeleteCB (Widget : PGtkWidget; Event : PGdkEvent; Data : GPointer) : GInt;
begin
  GtkMainQuit;
  DeleteCB := 0;
end;

{ Make a new hbox filled with button-labels. Arguments for the
  variables we're interested are passed in to this function.
  We do not show the box, but do show everything inside. }
function MakeBox (Homogeneous, Spacing, Expand, Fill, Padding : GInt) : PGtkWidget;
var
  Box, Button : PGtkWidget;
  PadStr : TString;
begin
  { Create a new hbox with the appropriate homogeneous and spacing settings. }
  Box := GtkHBoxNew(Homogeneous, Spacing);

  { Create a series of buttons with the appropriate settings. }
  Button := GtkButtonNewWithLabel ('GtkBoxPack');
  GtkBoxPackStart(PGtkBox(Box), Button, Expand, Fill, Padding);
  GtkWidgetShow(Button);

  Button := GtkButtonNewWithLabel('(Box,');
  GtkBoxPackStart(PGtkBox(Box), Button, Expand, Fill, Padding);
  GtkWidgetShow(Button);

  Button := GtkButtonNewWithLabel('button,');
  GtkBoxPackStart(PGtkBox(Box), Button, Expand, Fill, Padding);
  GtkWidgetShow(Button);

  { Create a button with the label depending on the value of Expand. }
  if Expand <> G_FALSE then
    Button := GtkButtonNewWithLabel('True,')
  else
    Button := GtkButtonNewWithLabel('False,');
  GtkBoxPackStart(PGtkBox(Box), Button, Expand, Fill, Padding);
  GtkWidgetShow(Button);

  { Create a button with the label depending on the value of Fill. }
  if Fill <> G_FALSE then
    Button := GtkButtonNewWithLabel('True,')
  else
    Button := GtkButtonNewWithLabel('False,');
  GtkBoxPackStart(PGtkBox(Box), Button, Expand, Fill, Padding);
  GtkWidgetShow(Button);

  WriteStr(PadStr, Padding, ');');
  Button := GtkButtonNewWithLabel(PadStr);
  GtkBoxPackStart(PGtkBox(Box), Button, Expand, Fill, Padding);
  GtkWidgetShow(Button);

  MakeBox := Box;
end;

var
  Window, Button, Box1, Box2, Separator, Lbl, QuitBox : PGtkWidget;
  Which, Error : Integer;

begin
  { Our init, don't forget this! :) }
  GtkInit;

  if ParamCount <> 1 then
    begin
      Writeln(StdErr, 'Usage: ', ParamStr(0), ' num, where num is 1, 2 or 3.');
      { This just does cleanup in GTK and exits with an exit status of 1. }
      GtkExit(1);
    end;

  { Doesn't check for errors. }
  Val(ParamStr(1), Which, Error);

  { Create our window. }
  Window := GtkWindowNew(GTK_WINDOW_TOPLEVEL);
  { You should always remember to connect the delete_event signal
    to the main window. This is very important for proper intuitive
    behavior. }
  GtkSignalConnect(PGtkObject(Window), 'delete_event', @DeleteCB, nil);
  GtkContainerSetBorderWidth(PGtkContainer(Window), 10);

  { We create a vertical box (vbox) to pack the horizontal boxes into.
    This allows us to stack the horizontal boxes filled with buttons one
    on top of the other in this vbox. }
  Box1 := GtkVBoxNew(G_FALSE, 0);

  { Which example to show. These correspond to the pictures above. }
  if Which = 1 then
    begin
      Lbl := GtkLabelNew('gtk_hbox_new (False, 0);');
      { Align the label to the left side.  We'll discuss this function and
        others in the section on Widget Attributes. }
      GtkMiscSetAlignment(PGtkMisc(Lbl), 0, 0);
      { Pack the label into the vertical box (vbox box1).  Remember that
        widgets added to a vbox will be packed one on top of the other in
        order. }
      GtkBoxPackStart(PGtkBox(Box1), Lbl, G_FALSE, G_FALSE, 0);

      { Show the label.}
      GtkWidgetShow(Lbl);

      { Call our make box function - homogeneous = False, spacing = 0,
        expand = False, fill = False, padding = 0 }
      Box2 := MakeBox(G_FALSE, 0, G_FALSE, G_FALSE, 0);
      GtkBoxPackStart(PGtkBox(Box1), Box2, G_FALSE, G_FALSE, 0);
      GtkWidgetShow(Box2);

      { Call our make box function - homogeneous = False, spacing = 0,
        expand = True, fill = False, padding = 0 }
      Box2 := MakeBox(G_FALSE, 0, G_TRUE, G_FALSE, 0);
      GtkBoxPackStart(PGtkBox(Box1), Box2, G_FALSE, G_FALSE, 0);
      GtkWidgetShow(Box2);

      { Args are: homogeneous, spacing, expand, fill, padding. }
      Box2 := MakeBox(G_FALSE, 0, G_TRUE, G_TRUE, 0);
      GtkBoxPackStart(PGtkBox(Box1), Box2, G_FALSE, G_FALSE, 0);
      GtkWidgetShow(Box2);

      { Creates a Separator, we'll learn more about these later,
        but they are quite simple. }
      Separator := GtkHSeparatorNew;

      { Pack the Separator into the vbox. Remember each of these
        widgets is being packed into a vbox, so they'll be stacked
        vertically. }
      GtkBoxPackStart(PGtkBox(Box1), Separator, G_FALSE, G_TRUE, 5);
      GtkWidgetShow(Separator);

      { Create another new label, and show it. }
      Lbl := GtkLabelNew('gtk_hBox_new (True, 0);');
      GtkMiscSetAlignment(PGtkMisc(Lbl), 0, 0);
      GtkBoxPackStart(PGtkBox(Box1), Lbl, G_FALSE, G_FALSE, 0);
      GtkWidgetShow(Lbl);

      { Args are: homogeneous, spacing, expand, fill, padding. }
      Box2 := MakeBox(G_TRUE, 0, G_TRUE, G_FALSE, 0);
      GtkBoxPackStart(PGtkBox(Box1), Box2, G_FALSE, G_FALSE, 0);
      GtkWidgetShow(Box2);

      { Args are: homogeneous, spacing, expand, fill, padding. }
      Box2 := MakeBox(G_TRUE, 0, G_TRUE, G_FALSE, 0);
      GtkBoxPackStart(PGtkBox(Box1), Box2, G_FALSE, G_FALSE, 0);
      GtkWidgetShow(Box2);

      { Args are: homogeneous, spacing, expand, fill, padding. }
      Box2 := MakeBox(G_TRUE, 0, G_TRUE, G_TRUE, 0);
      GtkBoxPackStart(PGtkBox(Box1), Box2, G_FALSE, G_FALSE, 0);
      GtkWidgetShow(Box2);

      { Another new Separator. }
      Separator := GtkHSeparatorNew;
      { The last 3 arguments to GtkBoxPackStart are:
        expand, fill, padding. }
      GtkBoxPackStart(PGtkBox(Box1), Separator, G_FALSE, G_TRUE, 5);
      GtkWidgetShow(Separator);
    end
  else if Which = 2 then
    begin
      { Create a new label, remember Box1 is a vBox as created
        near the beginning of main(). }
      Lbl := GtkLabelNew('gtk_hbox_new(False, 10);');
      GtkMiscSetAlignment(PGtkMisc(Lbl), 0, 0);
      GtkBoxPackStart(PGtkBox(Box1), Lbl, G_FALSE, G_FALSE, 0);
      GtkWidgetShow(Lbl);

      { Args are: homogeneous, spacing, expand, fill, padding. }
      Box2 := MakeBox(G_FALSE, 10, G_TRUE, G_FALSE, 0);
      GtkBoxPackStart(PGtkBox(Box1), Box2, G_FALSE, G_FALSE, 0);
      GtkWidgetShow(Box2);

      { Args are: homogeneous, spacing, expand, fill, padding }
      Box2 := MakeBox(G_FALSE, 10, G_TRUE, G_TRUE, 0);
      GtkBoxPackStart(PGtkBox(Box1), Box2, G_FALSE, G_FALSE, 0);
      GtkWidgetShow(Box2);

      Separator := GtkHSeparatorNew;
      { The last 3 arguments to GtkBoxPackStart are:
        expand, fill, padding. }
      GtkBoxPackStart(PGtkBox(Box1), Separator, G_FALSE, G_TRUE, 5);
      GtkWidgetShow(Separator);

      Lbl := GtkLabelNew('gtk_hBox_new(False, 0);');
      GtkMiscSetAlignment(PGtkMisc(Lbl), 0, 0);
      GtkBoxPackStart(PGtkBox(Box1), Lbl, G_FALSE, G_FALSE, 0);
      GtkWidgetShow(Lbl);

      { Args are: homogeneous, spacing, expand, fill, padding. }
      Box2 := MakeBox(G_FALSE, 0, G_TRUE, G_FALSE, 10);
      GtkBoxPackStart(PGtkBox(Box1), Box2, G_FALSE, G_FALSE, 0);
      GtkWidgetShow(Box2);

      { Args are: homogeneous, spacing, expand, fill, padding. }
      Box2 := MakeBox(G_FALSE, 0, G_TRUE, G_TRUE, 10);
      GtkBoxPackStart(PGtkBox(Box1), Box2, G_FALSE, G_FALSE, 0);
      GtkWidgetShow(Box2);

      Separator := GtkHSeparatorNew;
      { The last 3 arguments to GtkBoxPackStart are: expand, fill, padding. }
      GtkBoxPackStart(PGtkBox(Box1), Separator, G_FALSE, G_TRUE, 5);
      GtkWidgetShow(Separator);
    end
  else if Which = 3 then
    begin
      { Create the label that will be put at the end. }
      Lbl := GtkLabelNew('end');

      { This demonstrates the ability to use GtkBoxPackEnd() to
        right justify widgets. First, we create a new Box as before. }
      Box2 := MakeBox(G_FALSE, 0, G_FALSE, G_FALSE, 0);

      { Pack it using GtkBoxPackEnd(), so it is put on the right
        side of the hBox created in the MakeBox() call. }
      GtkBoxPackEnd(PGtkBox(Box2), Lbl, G_FALSE, G_FALSE, 0);

      { Show the label. }
      GtkWidgetShow(Lbl);

      { Pack Box2 into Box1 (the vbox remember ? :) }
      GtkBoxPackStart(PGtkBox(Box1), Box2, G_FALSE, G_FALSE, 0);
      GtkWidgetShow(Box2);

      { A Separator for the bottom. }
      Separator := GtkHSeparatorNew;
      { This explicitly sets the Separator to 400 pixels wide by 5 pixels
        high. This is so the hbox we created will also be 400 pixels wide,
        and the 'end' label will be separated from the other labels in the
        hbox. Otherwise, all the widgets in the hbox would be packed as
        close together as possible. }
      GtkWidgetSetUSize(Separator, 400, 5);
      { pack the Separator into the vbox (Box1) created near the start of main. }
      GtkBoxPackStart(PGtkBox(Box1), Separator, G_FALSE, G_TRUE, 5);
      GtkWidgetShow(Separator);
    end;

  { Create another new hbox.. remember we can use as many as we need! }
  QuitBox := GtkHBoxNew(G_FALSE, 0);

  { Our quit button. }
  Button := GtkButtonNewWithLabel('Quit');

  { Setup the signal to terminate the program when the button is clicked. }
  GtkSignalConnectObject(PGtkObject(Button), 'clicked', @GtkMainQuit, PGtkObject(Window));

  { Pack the button into the quitBox.
    The last 3 arguments to GtkBoxPackStart are:
    expand, fill, padding. }
  GtkBoxPackStart(PGtkBox(QuitBox), button, G_TRUE, G_FALSE, 0);
  { Pack the QuitBox into the vbox (Box1) }
  GtkBoxPackStart(PGtkBox(Box1), QuitBox, G_FALSE, G_FALSE, 0);

  { Pack the vBox(Box1) which now contains all our widgets, into the main window. }
  GtkContainerAdd(PGtkContainer(Window), Box1);

  { And show everything left. }
  GtkWidgetShow(Button);
  GtkWidgetShow(QuitBox);
  GtkWidgetShow(Box1);
  { Showing the window last so everything pops up at once. }
  GtkWidgetShow(Window);

  { And of course, our main function. }
  GtkMain;
  { Control returns here when gtk_main_quit() is called,
    but not when gtk_exit is used. }
end.
