{ Report bugs to Nicola Girardi <nikke@swlibero.org> }
program Paned;

{$include "../units/gtk.inc"}
{$ignore-function-results}

uses GPC;

{ Create the list of "messages" }
function CreateList : PGtkWidget;
var
  Scrolled, List, ListItem : PGtkWidget;
  I : Integer;
  Buffer : TString;
begin
  { Create a new scrolled window, with scrollbars only if needed }
  Scrolled := GtkScrolledWindowNew(nil, nil);
  GtkScrolledWindowSetPolicy(PGtkScrolledWindow(Scrolled), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   
  { Create a new list and put it in the scrolled window }
  List := GtkListNew;
  GtkScrolledWindowAddWithViewPort(PGtkScrolledWindow(Scrolled), List);
  GtkWidgetShow(List);

  { Add some messages to the window }
  for I := 0 to 10 do
    begin
      WriteStr(Buffer, 'Message ', I);
      ListItem := GtkListItemNewWithLabel(Buffer);
      GtkContainerAdd(PGtkContainer(List), ListItem);
      GtkWidgetShow(ListItem);
    end;

  CreateList := Scrolled;
end;

   
{ Add some text to our text widget - this is a callback that is invoked
  when our window is realized. We could also force our window to be
  realized with gtk_widget_realize, but it would have to be part of
  a hierarchy first }
procedure RealizeText(TText : PGtkWidget; Data : GPointer);
begin
  GtkTextFreeze(PGtkText(TText));
  GtkTextInsert(PGtkText(TText), nil, @TText^.Style^.Black, nil,
    "From: pathfinder@nasa.gov\n"
    "To: mom@nasa.gov\n"
    "Subject: Made it!\n"
    "\n"
    "We just got in this morning. The weather has been\n"
    "great - clear but cold, and there are lots of fun sights.\n"
    "Sojourner says hi. See you soon.\n"
    " -Path\n", -1);
  GtkTextThaw(PGtkText(TText));
end;
   
{ Create a scrolled text area that displays a "message" }
function CreateText : PGtkWidget;
var
  Table, TText, HScroll, VScroll : PGtkWidget;
begin
  { Create a table to hold the text widget and scrollbars }
  Table := GtkTableNew(2, 2, G_FALSE);
   
  { Put a text widget in the upper left hand corner. Note the use of
    GTK_SHRINK in the y direction }
  TText := GtkTextNew(nil, nil);
  GtkTableAttach(PGtkTable(Table), TText, 0, 1, 0, 1, GTK_FILL or GTK_EXPAND, GTK_FILL or GTK_EXPAND or GTK_SHRINK, 0, 0);
  GtkWidgetShow(TText);

  { Put a HScrollbar in the lower left hand corner }
  HScroll := GtkHScrollbarNew(PGtkText(TText)^.HAdj);
  GtkTableAttach(PGtkTable(Table), HScroll, 0, 1, 1, 2, GTK_EXPAND or GTK_FILL, GTK_FILL, 0, 0);
  GtkWidgetShow(HScroll);
   
  { And a VScrollbar in the upper right }
  VScroll := GtkVScrollbarNew(PGtkText(TText)^.VAdj);
  GtkTableAttach(PGtkTable(Table), VScroll, 1, 2, 0, 1, GTK_FILL, GTK_EXPAND or GTK_FILL or GTK_SHRINK, 0, 0);

  { Add a handler to put a message in the text widget when it is realized }
  GtkSignalConnect(PGtkObject(TText), 'realize', @RealizeText, nil);

  CreateText := Table;
end;

var
  Window, VPaned, List, TText : PGtkWidget;

begin
    GtkInit;

    Window := GtkWindowNew(GTK_WINDOW_TOPLEVEL);
    GtkWindowSetTitle(PGtkWindow(Window), 'Paned Windows');
    GtkSignalConnect(PGtkObject(Window), 'destroy', @GtkMainQuit, nil);
    GtkContainerSetBorderWidth(PGtkContainer(Window), 10);
    GtkWidgetSetUSize(Window, 450, 400);

    { Create a vpaned widget and add it to our toplevel window }
    VPaned := GtkVPanedNew;
    GtkContainerAdd(PGtkContainer(Window), VPaned);
    GtkPanedSetHandleSize(PGtkPaned(VPaned), 10);
    GtkPanedSetGutterSize(PGtkPaned(VPaned), 15);
    GtkWidgetShow(VPaned);

    { Now create the contents of the two halves of the window }
    List := CreateList;
    GtkPanedAdd1(PGtkPaned(VPaned), List);
    GtkWidgetShow(List);

    TText := CreateText;
    GtkPanedAdd2(PGtkPaned(VPaned), TText);
    GtkWidgetShow(TText);
    GtkWidgetShow(Window);
    GtkMain;
end.
