{ Report bugs to Nicola Girardi <nikke@swlibero.org> }
program ProgressBar;

{$include "../units/gtk.inc"}
{$ignore-function-results}

type
  PProgressData = ^ProgressData;
  ProgressData  = record
    Window, PBar : PGtkWidget;
    Timer : Integer;
  end;

{ Update the value of the progress bar so that we get some movement. }
function ProgressTimeout (Data : GPointer) = rv : GInt;
var
  NewVal : GFloat;
  Adj : PGtkAdjustment;
begin
  { Calculate the value of the progress bar using the
    value range set in the adjustment object. }
  NewVal := GtkProgressGetValue(PGtkProgress(Data)) + 1;
  Adj := PGtkProgress(Data)^.Adjustment;

  if NewVal > Adj^.Upper then
    NewVal := Adj^.Lower;

  { Set the new value. }
  GtkProgressSetValue(PGtkProgress(Data), NewVal);

  { As this is a timeout function, return TRUE so that it
    continues to get called. }
  rv := G_TRUE;
end;

{ Callback that toggles the text display within the progress bar trough. }
procedure ToggleShowText (Widget : PGtkWidget; PData : PProgressData);
begin
  GtkProgressSetShowText(PGtkProgress(PData^.PBar),
    GtkToggleButtonGetActive(PGtkToggleButton(Widget)));
end;

{ Callback that toggles the activity mode of the progress bar. }
procedure ToggleActivityMode (Widget : PGtkWidget; PData : PProgressData);
begin
  GtkProgressSetActivityMode(PGtkProgress(PData^.PBar), 
    GtkToggleButtonGetActive(PGtkToggleButton(Widget)));
end;

{ Callback that toggles the continuous mode of the progress bar. }
procedure SetContinuousMode (Widget : PGtkWidget; PData : PProgressData);
begin
  GtkProgressBarSetBarStyle(PGtkProgressBar(PData^.PBar), GTK_PROGRESS_CONTINUOUS);
end;

{ Callback that toggles the discrete mode of the progress bar. }
procedure SetDiscreteMode (Widget : PGtkWidget; PData : PProgressData);
begin
  GtkProgressBarSetBarStyle(PGtkProgressBar(PData^.PBar), GTK_PROGRESS_DISCRETE);
end;

{ Clean up allocated memory and remove the timer. }
procedure DestroyProgress (Widget : PGtkWidget; PData : PProgressData);
begin
  GtkTimeoutRemove(PData^.Timer);
  PData^.Timer := 0;
  PData^.Window := nil;
  Dispose(PData);
  GtkMainQuit;
end;

var
  PData : PProgressData;
  Align, Separator, Table, Button, Check, VBox : PGtkWidget;
  Adj : PGtkAdjustment;

begin
  GtkInit;

  { Allocate memory for the data that is passed to the callbacks. }
  New(PData);

  PData^.Window := GtkWindowNew(GTK_WINDOW_TOPLEVEL);
  GtkWindowSetPolicy(PGtkWindow(PData^.Window), G_FALSE, G_FALSE, G_TRUE);
  GtkSignalConnect(PGtkObject(PData^.Window), 'destroy', @DestroyProgress, PData);
  GtkWindowSetTitle(PGtkWindow(PData^.Window), 'GtkProgressBar');
  GtkContainerSetBorderWidth(PGtkContainer(PData^.Window), 0);

  VBox := GtkVBoxNew(G_FALSE, 5);
  GtkContainerSetBorderWidth(PGtkContainer(VBox), 10);
  GtkContainerAdd(PGtkContainer(PData^.Window), VBox);
  GtkWidgetShow(VBox);

  { Create a centering alignment object. }
  Align := GtkAlignmentNew(0.5, 0.5, 0, 0);
  GtkBoxPackStart(PGtkBox(VBox), Align, G_FALSE, G_FALSE, 5);
  GtkWidgetShow(Align);

  { Create a Adjusment object to hold the range of the progress bar. }
  Adj := PGtkAdjustMent(GtkAdjustmentNew(0, 1, 150, 0, 0, 0));

  { Create the GtkProgressBar using the adjustment. }
  PData^.PBar := GtkProgressBarNewWithAdjustment(Adj);
  { Set the format of the string that can be displayed in the trough
    of the progress bar:
    %p - percentage
    %v - value
    %l - lower range value
    %u - upper range value }
  GtkProgressSetFormatString(PGtkProgress(PData^.PBar),'%v from [%l-%u] (=%p%%)');
  GtkContainerAdd(PGtkContainer(Align), PData^.PBar);
  GtkWidgetShow(PData^.PBar);

  { Add a timer callback to update the value of the progress bar. }
  PData^.Timer := GtkTimeoutAdd(100, @ProgressTimeout, PData^.PBar);

  Separator := GtkHSeparatorNew;
  GtkBoxPackStart(PGtkBox(VBox), Separator, G_FALSE, G_FALSE, 0);
  GtkWidgetShow(Separator);

  { Rows, columns, homogeneous. }
  Table := GtkTableNew(2, 3, G_FALSE);
  GtkBoxPackStart(PGtkBox(VBox), Table, G_FALSE, G_TRUE, 0);
  GtkWidgetShow(Table);

  { Add a check button to select displaying of the trough text. }
  Check := GtkCheckButtonNewWithLabel('Show text');
  GtkTableAttach(PGtkTable(Table), Check, 0, 1, 0, 1, GTK_EXPAND or GTK_FILL or GTK_EXPAND, GTK_FILL, 5, 5);
  GtkSignalConnect(PGtkObject(Check), 'clicked', @ToggleShowText, PData);
  GtkWidgetShow(Check);

  { Add a check button to toggle activity mode. }
  Check := GtkCheckButtonNewWithLabel('Activity mode');
  GtkTableAttach(PGtkTable(Table), Check, 0, 1, 1, 2, GTK_EXPAND or GTK_FILL or GTK_EXPAND, GTK_FILL, 5, 5);
  GtkSignalConnect(PGtkObject(Check), 'clicked', @ToggleActivityMode, PData);
  GtkWidgetShow(Check);

  Separator := GtkVSeparatorNew;
  GtkTableAttach(PGtkTable(Table), Separator, 1, 2, 0, 2, GTK_EXPAND or GTK_FILL or GTK_EXPAND, GTK_FILL, 5, 5);
  GtkWidgetShow(Separator);

  { Add a radio button to select continuous display mode. }
  Button := GtkRadioButtonNewWithLabel(nil, 'Continuous');
  GtkTableAttach(PGtkTable(Table), Button, 2, 3, 0, 1, GTK_EXPAND or GTK_FILL or GTK_EXPAND, GTK_FILL, 5, 5);
  GtkSignalConnect(PGtkObject(Button), 'clicked', @SetContinuousMode, PData);
  GtkWidgetShow(Button);
  { Add a radio button to select discrete display mode. }
  Button := GtkRadioButtonNewWithLabel(GtkRadioButtonGroup(PGtkRadioButton(Button)), 'discrete');
  GtkTableAttach(PGtkTable(Table), Button, 2, 3, 1, 2, GTK_EXPAND or GTK_FILL or GTK_EXPAND, GTK_FILL, 5, 5);
  GtkSignalConnect(PGtkObject(Button), 'clicked', @SetDiscreteMode, PData);
  GtkWidgetShow(Button);

  Separator := GtkHSeparatorNew;
  GtkBoxPackStart(PGtkBox(VBox), Separator, G_FALSE, G_FALSE, 0);
  GtkWidgetShow(Separator);

  { Add a button to exit the program. }
  Button := GtkButtonNewWithLabel('close');
  GtkSignalConnectObject(PGtkObject(Button), 'clicked', @GtkWidgetDestroy, PGtkObject(PData^.Window));
  GtkBoxPackStart(PGtkBox(VBox), Button, G_FALSE, G_FALSE, 0);

  { This makes it so the button is the default. }
  GTK_WIDGET_SET_FLAGS(Button, GTK_CAN_DEFAULT);
  { This grabs this button to be the default button. Simply hitting
    the "Enter" key will cause this button to activate. }
  GtkWidgetGrabDefault(Button);
  GtkWidgetShow(Button);
  GtkWidgetShow(PData^.Window);

  GtkMain;
end.
