{ Report bugs to Nicola Girardi <nikke@swlibero.org> }
program RangeWidgets;

{$include "../units/gtk.inc"}
{$ignore-function-results}

var
  HScale, VScale : PGtkWidget;

procedure PosMenuSelectCB (Item : PGtkWidget; Pos : TGtkPositionType);
begin
  { Set the value position on both scale widgets. }
  GtkScaleSetValuePos(PGtkScale(HScale), Pos);
  GtkScaleSetValuePos(PGtkScale(VScale), Pos);
end;

procedure UpdateMenuSelectCB (Item : PGtkWidget; Policy : TGtkUpdateType);
begin
  { Set the update policy for both scale widgets. }
  GtkRangeSetUpdatePolicy(PGtkRange(HScale), Policy);
  GtkRangeSetUpdatePolicy(PGtkRange(VScale), Policy);
end;

procedure DigitsScaleCB (Adj : PGtkAdjustment);
begin
  { Set the number of decimal places to which adj->value is rounded. }
  GtkScaleSetDigits(PGtkScale(HScale), Round(Adj^.VValue));
  GtkScaleSetDigits(PGtkScale(VScale), Round(Adj^.VValue));
end;

procedure PageSizeCB (AGet, ASet : PGtkAdjustment);
begin
  { Set the page size and page increment size of the sample
    adjustment to the value specified by the "Page Size" scale. }
  ASet^.PageSize := AGet^.VValue;
  ASet^.PageIncrement := AGet^.VValue;
  { Now emit the "changed" signal to reconfigure all the widgets that
    are attached to this adjustment. }
  GtkSignalEmitByName(PGtkObject(ASet), 'changed');
end;

procedure DrawValueCB (Button : PGtkToggleButton);
Var
  gactive : GBoolean;
  
begin
  gactive := GtkTogglebuttonGetActive(PGtkToggleButton(Button));
  { Turn the value display on the scale widgets off or on depending
    on the state of the checkbutton. }
  GtkScaleSetDrawValue(PGtkScale(HScale), gactive);
  GtkScaleSetDrawValue(PGtkScale(VScale), gactive);
end;

{ Convenience functions. }
function MakeMenuItem (Name : PGChar; Callback : TGtkSignalFunc; Data : GPointer) = Item : PGtkWidget;
begin
  Item := GtkMenuItemNewWithLabel(Name);
  GtkSignalConnect(PGtkObject(Item), 'activate', Callback, Data);
  GtkWidgetShow(Item);
end;

procedure ScaleSetDefaultValues (Scale : PGtkScale);
begin
  GtkRangeSetUpdatePolicy(PGtkRange(Scale), GTK_UPDATE_CONTINUOUS);
  GtkScaleSetDigits(Scale, 1);
  GtkScaleSetValuePos(Scale, GTK_POS_TOP);
  GtkScaleSetDrawValue(Scale, G_TRUE);
end;

{ Makes the sample window. }
procedure CreateRangeControls;
var
  Window, Box1, Box2, Box3, Button, Scrollbar, Separator, Opt, Menu, Item, Lbl, Scale : PGtkWidget;
  Adj1, Adj2 : PGtkObject;
begin
  { Standard window-creating stuff. }
  Window := GtkWindowNew(GTK_WINDOW_TOPLEVEL);
  GtkSignalConnect(PGtkObject(Window), 'destroy', @GtkMainQuit, nil);
  GtkWindowSetTitle(PGtkWindow(Window), 'range controls');

  Box1 := GtkVBoxNew(G_FALSE, 0);
  GtkContainerAdd(PGtkContainer(Window), Box1);
  GtkWidgetShow(Box1);

  Box2 := GtkHBoxNew(G_FALSE, 10);
  GtkContainerSetBorderWidth(PGtkContainer(Box2), 10);
  GtkBoxPackStart(PGtkBox(Box1), Box2, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(Box2);

  { Value, lower, upper, step_increment, page_increment, page_size. }
  { Note that the page_size value only makes a difference for
    scrollbar widgets, and the highest value you'll get is actually
    (upper - page_size). }
  Adj1 := GtkAdjustmentNew(0.0, 0.0, 101.0, 0.1, 1.0, 1.0);

  VScale := GtkVScaleNew(PGtkAdjustment(Adj1));
  ScaleSetDefaultValues(PGtkScale(VScale));
  GtkBoxPackStart(PGtkBox(Box2), VScale, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(VScale);

  Box3 := GtkVBoxNew(G_FALSE, 10);
  GtkBoxPackStart(PGtkBox(Box2), Box3, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(Box3);

  { Reuse the same adjustment. }
  HScale := GtkHScaleNew(PGtkAdjustment(Adj1));
  GtkWidgetSetUSize(PGtkWidget(HScale), 200, 30);
  ScaleSetDefaultValues(PGtkScale(HScale));
  GtkBoxPackStart(PGtkBox(Box3), HScale, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(HScale);

  { Reuse the same adjustment again. }
  ScrollBar := GtkHScrollbarNew(PGtkAdjustment(Adj1));
  { Notice how this causes the scales to always be updated
    continuously when the scrollbar is moved. }
  GtkRangeSetUpdatePolicy(PGtkRange(Scrollbar), GTK_UPDATE_CONTINUOUS);
  GtkBoxPackStart(PGtkBox(Box3), Scrollbar, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(Scrollbar);

  Box2 := GtkHBoxNew(G_FALSE, 10);
  GtkContainerSetBorderWidth(PGtkContainer(Box2), 10);
  GtkBoxPackStart(PGtkBox(Box1), Box2, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(Box2);

  { A checkbutton to control whether the value is displayed or not. }
  Button := GtkCheckButtonNewWithLabel('Display value on scale widgets');
  GtkToggleButtonSetActive(PGtkToggleButton(Button), G_TRUE);
  GtkSignalConnect(PGtkObject(Button), 'toggled', @DrawValueCB, nil);
  GtkBoxPackStart(PGtkBox(Box2), Button, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(Button);

  Box2 := GtkHBoxNew(G_FALSE, 10);
  GtkContainerSetBorderWidth(PGtkContainer(Box2), 10);

  { An option menu to change the position of the value. }
  Lbl := GtkLabelNew('Scale Value Position:');
  GtkBoxPackStart(PGtkBox(Box2), Lbl, G_FALSE, G_FALSE, 0);
  GtkWidgetShow(Lbl);\

  Opt := GtkOptionMenuNew;
  Menu := GtkMenuNew;

  Item := MakeMenuItem('Top', @PosMenuSelectCB, GPointer(GTK_POS_TOP));
  GtkMenuAppend(PGtkMenu(Menu), Item);

  Item := MakeMenuItem('Bottom', @PosMenuSelectCB, GPointer(GTK_POS_BOTTOM));
  GtkMenuAppend(PGtkMenu(Menu), Item);

  Item := MakeMenuItem('Left', @PosMenuSelectCB, GPointer(GTK_POS_LEFT));
  GtkMenuAppend(PGtkMenu(Menu), Item);

  Item := MakeMenuItem('Right', @PosMenuSelectCB, GPointer(GTK_POS_RIGHT));
  GtkMenuAppend(PGtkMenu(Menu), Item);

  GtkOptionMenuSetMenu(PGtkOptionMenu(Opt), Menu);
  GtkBoxPackStart(PGtkBox(Box2), Opt, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(Opt);
  GtkBoxPackStart(PGtkBox(Box1), Box2, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(Box2);

  Box2 := GtkHBoxNew(G_FALSE, 10);
  GtkContainerSetBorderWidth(PGtkContainer(Box2), 10);

  { Yet another option menu, this time for the update policy of the
    scale widgets. }
  Lbl := GtkLabelNew('Scale Update Policy:');
  GtkBoxPackStart(PGtkBox(Box2), Lbl, G_FALSE, G_FALSE, 0);
  GtkWidgetShow(Lbl);

  Opt := GtkOptionMenuNew;
  Menu := GtkMenuNew;

  Item := MakeMenuItem('Continuous', @UpdateMenuSelectCB, GPointer(GTK_UPDATE_CONTINUOUS));
  GtkMenuAppend(PGtkMenu(Menu), Item);

  Item := MakeMenuItem('Discontinuous', @UpdateMenuSelectCB, GPointer(GTK_UPDATE_DISCONTINUOUS));
  GtkMenuAppend(PGtkMenu(Menu), Item);

  Item := MakeMenuItem('Delayed', @UpdateMenuSelectCB, GPointer(GTK_UPDATE_DELAYED));
  GtkMenuAppend(PGtkMenu(Menu), Item);

  GtkOptionMenuSetMenu(PGtkOptionMenu(Opt), Menu);
  GtkBoxPackStart(PGtkBox(Box2), Opt, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(Opt);
  GtkBoxPackStart(PGtkBox(Box1), Box2, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(Box2);

  Box2 := GtkHBoxNew(G_FALSE, 10);
  GtkContainerSetBorderWidth(PGtkContainer(Box2), 10);


  { An HScale widget for adjusting the number of digits on the
    sample scales. }
  Lbl := GtkLabelNew('Scale Digits:');
  GtkBoxPackStart(PGtkBox(Box2), Lbl, G_FALSE, G_FALSE, 0);
  GtkWidgetShow(Lbl);

  Adj2 := GtkAdjustmentNew(1.0, 0.0, 5.0, 1.0, 1.0, 0.0);
  GtkSignalConnect(Adj2, 'value_changed', @DigitsScaleCB, nil);

  Scale := GtkHScaleNew(PGtkAdjustment(Adj2));
  GtkScaleSetDigits(PGtkScale(Scale), 0);
  GtkBoxPackStart(PGtkBox(Box2), Scale, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(Scale);
  GtkBoxPackStart(PGtkBox(Box1), Box2, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(Box2);

  Box2 := GtkHBoxNew(G_FALSE, 10);
  GtkContainerSetBorderWidth(PGtkContainer(Box2), 10);

  { And one last HScale widget for adjusting the page size of the scrollbar. }
  Lbl := GtkLabelNeW('Scrollbar Page Size:');
  GtkBoxPackStart(PGtkBox(Box2), Lbl, G_FALSE, G_FALSE, 0);
  GtkWidgetShow(Lbl);

  Adj2 := GtkAdjustmentNew(1.0, 1.0, 101.0, 1.0, 1.0, 0.0);
  GtkSignalConnect(Adj2, 'value_changed', @PageSizeCB, Adj1);

  Scale := GtkHScaleNew(PGtkAdjustment(Adj2));
  GtkScaleSetDigits(PGtkScale(Scale), 0);
  GtkBoxPackStart(PGtkBox(Box2), Scale, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(Scale);
  GtkBoxPackStart(PGtkBox(Box1), Box2, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(Box2);

  Separator := GtkHSeparatorNew;
  GtkBoxPackStart(PGtkBox(Box1), Separator, G_FALSE, G_TRUE, 0);
  GtkWidgetShow(Separator);

  Box2 := GtkVBoxNew(G_FALSE, 10);
  GtkContainerSetBorderWidth(PGtkContainer(Box2), 10);
  GtkBoxPackStart(PGtkBox(Box1), Box2, G_FALSE, G_TRUE, 0);
  GtkWidgetShow(Box2);

  Button := GtkButtonNewWithLabel('Quit');
  GtkSignalConnectObject(PGtkObject(Button), 'clicked', @GtkMainQuit, nil);
  GtkBoxPackStart(PGtkBox(Box2), Button, G_TRUE, G_TRUE, 0);
  GTK_WIDGET_SET_FLAGS(Button, GTK_CAN_DEFAULT);
  GtkWidgetGrabDefault(Button);
  GtkWidgetShow(Button);

  GtkWidgetShow(Window);
end;

begin
  GtkInit;
  CreateRangeControls;
  GtkMain;
end.
