{ Report bugs to Nicola Girardi <nikke@swlibero.org> }
program ScribbleSimple;

{$include "../units/gtk.inc"}
{$ignore-function-results}

{ Backing pixmap for drawing area }
var
  Pixmap : static PGdkPixmap = nil;

{ Create a new backing pixmap of the appropriate size }
function ConfigureEvent(Widget : PGtkWidget; Event : PGdkEventConfigure) : GInt;
begin
  if Pixmap <> nil then
    GdkPixmapUnref(Pixmap);

  with Widget^ do
    begin
      Pixmap := GdkPixmapNew(Window, Allocation.Width, Allocation.Height, -1);
      GdkDrawRectangle(Pixmap, Style^.WhiteGc, G_TRUE, 0, 0, Allocation.Width, Allocation.Height);
    end;

  ConfigureEvent := G_TRUE;
end;

{ Redraw the screen from the backing pixmap }
function ExposeEvent(Widget : PGtkWidget; Event : PGdkEventExpose) : GInt;
begin
  with Event^ do
    GdkDrawPixmap(Widget^.Window, Widget^.Style^.FgGc[Widget^.State], Pixmap, Area.x, Area.y, Area.x, Area.y, Area.Width, Area.Height);
  ExposeEvent := G_FALSE;
end;

{ Draw a rectangle on the screen }
procedure DrawBrush(Widget : PGtkWidget; x, y : GDouble);
var
  UpdateRect : GdkRectangle;
begin
  UpdateRect.x := Round(x - 5);
  UpdateRect.y := Round(y - 5);
  UpdateRect.Width := 10;
  UpdateRect.Height := 10;
  GdkDrawRectangle(Pixmap, Widget^.Style^.BlackGc, G_TRUE, UpdateRect.x, UpdateRect.y, UpdateRect.Width, UpdateRect.Height);
  GtkWidgetDraw(Widget, @UpdateRect);
end;

function ButtonPressEvent(Widget : PGtkWidget; Event : PGdkEventButton) : GInt;
begin
  if (Event^.Button = 1) and (Pixmap <> nil) then
    DrawBrush(Widget, Event^.x, Event^.y);
  ButtonPressEvent := G_TRUE;
end;

function MotionNotifyEvent(Widget : PGtkWidget; Event : PGdkEventMotion) : GInt;
var
  x, y : Integer;
  State : GdkModifierType;
begin
  if Event^.IsHint <> G_FALSE then
    GdkWindowGetPointer(Event^.Window, x, y, State)
  else
    begin
      x := Round(Event^.x);
      y := Round(Event^.y);
      State := Event^.State;
    end;

  if ((State and GDK_BUTTON1_MASK) <> 0) and (Pixmap <> nil) then
    DrawBrush(Widget, x, y);
  
  MotionNotifyEvent := G_TRUE;
end;

procedure Quit;
begin
  GtkExit(0);
end;

var
  Window, DrawingArea, VBox, Button : PGtkWidget;

begin
  GtkInit;

  Window := GtkWindowNew(GTK_WINDOW_TOPLEVEL);
  GtkWidgetSetName(Window, 'Test Input');

  VBox := GtkVBoxNew(G_FALSE, 0);
  GtkContainerAdd(PGtkContainer(Window), VBox);
  GtkWidgetShow(VBox);

  GtkSignalConnect(PGtkObject(Window), 'destroy', @Quit, nil);

  { Create the drawing area }

  DrawingArea := GtkDrawingAreaNew;
  GtkDrawingAreaSize(PGtkDrawingArea(DrawingArea), 200, 200);
  GtkBoxPackStart(PGtkBox(VBox), DrawingArea, G_TRUE, G_TRUE, 0);

  GtkWidgetShow(DrawingArea);

  { Signals used to handle backing pixmap }

  GtkSignalConnect(PGtkObject(drawingArea), 'expose_event', @ExposeEvent, nil);
  GtkSignalConnect(PGtkObject(DrawingArea),'configure_event', @ConfigureEvent, nil);

  { Event signals }

  GtkSignalConnect(PGtkObject(DrawingArea), 'motion_notify_event', @MotionNotifyEvent, nil);
  GtkSignalConnect(PGtkObject(DrawingArea), 'button_press_event', @ButtonPressEvent, nil);

  GtkWidgetSetEvents(DrawingArea,
                     GDK_EXPOSURE_MASK
                     or GDK_LEAVE_NOTIFY_MASK
                     or GDK_BUTTON_PRESS_MASK
                     or GDK_POINTER_MOTION_MASK
                     or GDK_POINTER_MOTION_HINT_MASK);

  { .. And a quit button }
  Button := GtkButtonNewWithLabel('Quit');
  GtkBoxPackStart(PGtkBox(VBox), Button, G_FALSE, G_FALSE, 0);

  GtkSignalConnectObject(PGtkObject(Button), 'clicked', @GtkWidgetDestroy, PGtkObject(Window));
  GtkWidgetShow(Button);
  GtkWidgetShow(Window);
  GtkMain;
end.
