{ Report bugs to Nicola Girardi <nikke@swlibero.org> }
program ScribbleXInput;

{$include "../units/gtk.inc"}
{$ignore-function-results}

{ Backing pixmap for drawing area }
var
  Pixmap : static PGdkPixmap = nil;

{ Create a new backing pixmap of the appropriate size }
function ConfigureEvent(Widget : PGtkWidget; Event : PGdkEventConfigure) : GInt;
begin
  if Pixmap <> nil then
    GdkPixmapUnref(Pixmap);

  with Widget^ do
    begin
      Pixmap := GdkPixmapNew(Window, Allocation.Width, Allocation.Height, -1);
      GdkDrawRectangle(Pixmap, Style^.WhiteGc, G_TRUE, 0, 0, Allocation.Width, Allocation.Height);
    end;

  ConfigureEvent := G_TRUE;
end;

{ Redraw the screen from the backing pixmap }
function ExposeEvent(Widget : PGtkWidget; Event : PGdkEventExpose) : GInt;
begin
  with Event^ do
    GdkDrawPixmap(Widget^.Window, Widget^.Style^.FgGc[Widget^.State], Pixmap, Area.x, Area.y, Area.x, Area.y, Area.Width, Area.Height);
  ExposeEvent := G_FALSE;
end;

{ Draw A Rectangle On The Screen, Size Depending On Pressure, And Color On The Type Of Device }
procedure DrawBrush(Widget : PGtkWidget; Source : GdkInputSource; x, y, Pressure : GDouble);
var
  UpdateRect : GdkRectangle;
  GC : PGdkGC;
begin
  if Source = GDK_SOURCE_MOUSE then
    GC := Widget^.Style^.DarkGc[Widget^.State]
  else if Source = GDK_SOURCE_PEN then
    GC := Widget^.Style^.BlackGc
  else if Source = GDK_SOURCE_ERASER then
    GC := Widget^.Style^.WhiteGc
  else
    GC := Widget^.Style^.LightGc[Widget^.State];

  UpdateRect.x := Round(X - 10 * Pressure);
  UpdateRect.y := Round(Y - 10 * Pressure);
  UpdateRect.Width := Round(20 * Pressure);
  UpdateRect.Height := Round(20 * Pressure);
  with UpdateRect do
    GdkDrawRectangle(Pixmap, Gc, G_TRUE, x, y, Width, Height);
  GtkWidgetDraw(Widget, @UpdateRect);
end;

procedure PrintButtonPress(DeviceID : GUint32);
var
  Tmp : PGList;
begin
  { GdkInputListDevices Returns An Internal List, So We Shouldn't Free It Afterwards }
  Tmp := GdkInputListDevices;

  while Tmp <> nil do
    begin
      var Info : PGdkDeviceInfo = PGdkDeviceInfo(Tmp^.Data);
      if Info^.DeviceID = DeviceID then
        begin
          Writeln('Button press on device ', CString2String(Info^.Name));
          Exit;
        end;
      Tmp := Tmp^.Next;
    end;
end;

function ButtonPressEvent(Widget : PGtkWidget; Event : PGdkEventButton) : GInt;
begin
  PrintButtonPress(Event^.DeviceID);
  if (Event^.Button = 1) and (Pixmap <> nil) then
    DrawBrush(Widget, Event^.Source, Event^.x, Event^.y, Event^.Pressure);
  ButtonPressEvent := G_TRUE;
end;

function MotionNotifyEvent(Widget : PGtkWidget; Event : PGdkEventMotion) : GInt;
var
  x, y, Pressure, DummyX, DummyY : GDouble;
  State : GdkModifierType;
begin
  if Event^.IsHint <> G_FALSE then
    GdkInputWindowGetPointer(Event^.Window, Event^.DeviceID, x, y, Pressure, DummyX, DummyY, State)
  else
    begin
      x := Round(Event^.x);
      y := Round(Event^.y);
      Pressure := Event^.Pressure;
      State := Event^.State;
    end;

  if ((State and GDK_BUTTON1_MASK) <> 0) and (Pixmap <> nil) then
    DrawBrush(Widget, Event^.Source, x, y, Pressure);
  
  MotionNotifyEvent := G_TRUE;
end;

procedure InputDialogDestroy(W : PGtkWidget; Data : GPointer);
begin
  PPGtkWidget(Data)^ := nil;
end;

procedure CreateInputDialog;
var
  InputD : static PGtkWidget = nil;
begin
  if InputD = nil then
    begin
      InputD := GtkInputDialogNew;
      GtkSignalConnect(PGtkObject(Inputd), 'destroy', @InputDialogDestroy, @InputD);
      GtkSignalConnectObject(PGtkObject(PGtkInputDialog(InputD)^.CloseButton), 'clicked', @GtkWidgetHide, PGtkObject(InputD));
      GtkWidgetHide(PGtkInputDialog(InputD)^.SaveButton);
      GtkWidgetShow(InputD);
    end
  else
    begin
      if not GTK_WIDGET_MAPPED(InputD) then
        GtkWidgetShow(InputD)
      else
        GdkWindowRaise(InputD^.Window);
    end;
end;

procedure Quit;
begin
  GtkExit(0);
end;

var
  Window, DrawingArea, VBox, Button : PGtkWidget;

begin
  GtkInit;

  Window := GtkWindowNew(GTK_WINDOW_TOPLEVEL);
  GtkWidgetSetName(Window, 'Test Input');

  VBox := GtkVBoxNew(G_FALSE, 0);
  GtkContainerAdd(PGtkContainer(Window), VBox);
  GtkWidgetShow(VBox);

  GtkSignalConnect(PGtkObject(window), 'destroy', @Quit, nil);

  { Create the drawing area }

  DrawingArea := GtkDrawingAreaNew;
  GtkDrawingAreaSize(PGtkDrawingArea(DrawingArea), 200, 200);
  GtkBoxPackStart(PGtkBox(VBox), DrawingArea, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(DrawingArea);

  { Signals used to handle backing pixmap }

  GtkSignalConnect(PGtkObject(DrawingArea), 'expose_event', @ExposeEvent, nil);
  GtkSignalConnect(PGtkObject(DrawingArea),'configure_event', @ConfigureEvent, nil);

  { Event signals }

  GtkSignalConnect(PGtkObject(DrawingArea), 'motion_notify_event', @MotionNotifyEvent, nil);
  GtkSignalConnect(PGtkObject(DrawingArea), 'button_press_event', @ButtonPressEvent, nil);

  GtkWidgetSetEvents(DrawingArea,
                     GDK_EXPOSURE_MASK
                     or GDK_LEAVE_NOTIFY_MASK
                     or GDK_BUTTON_PRESS_MASK
                     or GDK_POINTER_MOTION_MASK
                     or GDK_POINTER_MOTION_HINT_MASK);

  { The Following Call Enables Tracking And Processing Of Extension Events For The Drawing Area }
  GtkWidgetSetExtensionEvents(DrawingArea, GDK_EXTENSION_EVENTS_CURSOR);

  { .. And some buttons }
  Button := GtkButtonNewWithLabel('Input Dialog');
  GtkBoxPackStart(PGtkBox(VBox), Button, G_FALSE, G_FALSE, 0);

  GtkSignalConnect(PGtkObject(Button), 'clicked', @CreateInputDialog, nil);
  GtkWidgetShow(Button);

  Button := GtkButtonNewWithLabel('Quit');
  GtkBoxPackStart(PGtkBox(VBox), Button, G_FALSE, G_FALSE, 0);

  GtkSignalConnectObject(PGtkObject(Button), 'clicked', @GtkWidgetDestroy, PGtkObject(Window));
  GtkWidgetShow(Button);

  GtkWidgetShow(Window);

  GtkMain;
end.
