{ Report bugs to Nicola Girardi <nikke@swlibero.org> }
program ScrolledWin;

{$include "../units/gtk.inc"}
{$ignore-function-results}

procedure Destroy(Widget : PGtkWidget; Data : GPointer);
begin
  GtkMainQuit;
end;

var
  Window : static PGtkWidget;
  ScrolledWindow, Table, Button : PGtkWidget;
  Buffer : String(32);
  i, j : Integer;

begin
  GtkInit;

  { Create a new dialog window for the scrolled window to be packed into. }
  Window := GtkDialogNew;
  GtkSignalConnect(PGtkObject(Window), 'destroy', @Destroy, nil);
  GtkWindowSetTitle(PGtkWindow(Window), 'GtkScrolledWindow example');
  GtkContainerSetBorderWidth(PGtkContainer(Window), 0);
  GtkWidgetSetUSize(Window, 300, 300);

  { Create a new scrolled window. }
  ScrolledWindow := GtkScrolledWindowNew(nil, nil);
  GtkContainerSetBorderWidth(PGtkContainer(ScrolledWindow), 10);

  { The policy is one of GTK_POLICY AUTOMATIC, or GTK_POLICY_ALWAYS.
    GTK_POLICY_AUTOMATIC will automatically decide whether you need
    scrollbars, whereas GTK_POLICY_ALWAYS will always leave the scrollbars
    there.  The first one is the horizontal scrollbar, the second,
    the vertical. }
  GtkScrolledWindowSetPolicy(PGtkScrolledWindow(ScrolledWindow), GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);

  { The dialog window is created with a vbox packed into it. }
  GtkBoxPackStart(PGtkBox(PGtkDialog(Window)^.VBox), ScrolledWindow, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(ScrolledWindow);

  { Create a table of 10 by 10 squares. }
  Table := GtkTableNew(10, 10, G_FALSE);

  { Set the spacing to 10 on x and 10 on y }
  GtkTableSetRowSpacings(PGtkTable(Table), 10);
  GtkTableSetColSpacings(PGtkTable(Table), 10);

  { Pack the table into the scrolled window }
  GtkScrolledWindowAddWithViewport(PGtkScrolledWindow(ScrolledWindow), Table);
  GtkWidgetShow(Table);

  { This simply creates a grid of toggle buttons on the table
    to demonstrate the scrolled window. }
  for i := 1 to 10 do
    for j := 1 to 10 do
      begin
        WriteStr(Buffer, 'Button (', i, ',', j, ')');
        Button := GtkToggleButtonNewWithLabel(Buffer);
        GtkTableAttachDefaults(PGtkTable(Table), Button, i, i+1, j, j+1);
        GtkWidgetShow(Button);
      end;
    
  { Add a "close" button to the bottom of the dialog }
  Button := GtkButtonNewWithLabel('Close');
  GtkSignalConnectObject(PGtkObject(Button), 'clicked', @GtkWidgetDestroy, PGtkObject(Window));

  { This makes it so the button is the default. }
  GTK_WIDGET_SET_FLAGS(Button, GTK_CAN_DEFAULT);
  GtkBoxPackStart(PGtkBox(PGtkDialog(Window)^.ActionArea), Button, G_TRUE, G_TRUE, 0);

  { This grabs this button to be the default button. Simply hitting
    the "Enter" key will cause this button to activate. }
  GtkWidgetGrabDefault(Button);
  GtkWidgetShow(Button);

  GtkWidgetShow(Window);

  GtkMain;
end.
