{ Report bugs to Nicola Girardi <nikke@swlibero.org> }
program SetSelection;

{$include "../units/gtk.inc"}
{$ignore-function-results}
uses GPC;

{ Callback when the user toggles the selection }
procedure SelectionToggled(Widget : PGtkWidget; var HaveSelection : Boolean);
begin
  if GtkToggleButtonGetActive(PGtkToggleButton(Widget)) <> G_FALSE then
    begin
      HaveSelection := GtkSelectionOwnerSet(Widget, GDK_SELECTION_PRIMARY, GDK_CURRENT_TIME) <> G_FALSE;
      { If Claiming The Selection Failed, We Return The Button To The Out State }
      if not HaveSelection then
        GtkToggleButtonSetActive(PGtkToggleButton(Widget), G_FALSE);
    end
  else
    begin
      if HaveSelection then
        begin
          { Before Clearing The Selection By Setting The Owner To nil,
            We Check If We Are The Actual Owner }
          if GdkSelectionOwnerGet(GDK_SELECTION_PRIMARY) = Widget^.Window then
            GtkSelectionOwnerSet(nil, GDK_SELECTION_PRIMARY, GDK_CURRENT_TIME);
          HaveSelection := False;
        end;
    end;
end;

{ Called when another application claims the selection }
function SelectionClear(Widget : PGtkWidget; Event : PGdkEventSelection; var HaveSelection : Boolean) : GInt;
begin
  HaveSelection := False;
  GtkToggleButtonSetActive(PGtkToggleButton(Widget), G_FALSE);
  SelectionClear := G_TRUE
end;

{ Supplies the current time as the selection. }
procedure SelectionHandle(Widget : PGtkWidget; SelectionData : PGtkSelectionData; Info, TStamp : GUint; Data : GPointer);
function GetTimeStr(var Len : Integer) : PGChar; asmname '_p_time_str';
var
  Stamp : TimeStamp;
  TimeStr : TString;
begin
  GetTimeStamp(Stamp);
  TimeStr := FormatTime(Stamp, '%x %X');
  { When We Return A Single String, It Should Not Be Null Terminated. That Will Be Done For Us }
  GtkSelectionDataSet(SelectionData, GDK_SELECTION_TYPE_STRING, 8, PGUChar(String2CString(TimeStr)), Length(TimeStr))
end;

var
  Window, SelectionButton : PGtkWidget;
  HaveSelection : static Boolean = False;

begin
  GtkInit;

  { Create the toplevel window }

  Window := GtkWindowNew(GTK_WINDOW_TOPLEVEL);
  GtkWindowSetTitle(PGtkWindow(Window), 'Event Box');
  GtkContainerSetBorderWidth(PGtkContainer(Window), 10);

  GtkSignalConnect(PGtkObject(Window), 'destroy', @GtkExit, nil);

  { Create a toggle button to act as the selection }

  SelectionButton := GtkToggleButtonNewWithLabel('Claim Selection');
  GtkContainerAdd(PGtkContainer(Window), SelectionButton);
  GtkWidgetShow(SelectionButton);

  GtkSignalConnect(PGtkObject(SelectionButton), 'toggled', @SelectionToggled, @HaveSelection);
  GtkSignalConnect(PGtkObject(SelectionButton), 'selection_clear_event', @SelectionClear, @HaveSelection);

  GtkSelectionAddTarget(SelectionButton, GDK_SELECTION_PRIMARY, GDK_SELECTION_TYPE_STRING, 1);
  GtkSignalConnect(PGtkObject(SelectionButton), 'selection_get', @SelectionHandle, @HaveSelection);

  GtkWidgetShow(SelectionButton);
  GtkWidgetShow(Window);
  
  GtkMain;
end.
