{ Report bugs to Nicola Girardi <nikke@swlibero.org> }
program TText;

{$include "../units/gtk.inc"}
{$ignore-function-results}

uses GPC;

{ !! This function is deprecated, so it's not in gdk.pas. }
function GdkColorAlloc(Colormap : PGdkColormap; Color : PGdkColor) : GBoolean;
asmname 'gdk_color_alloc';

procedure TextToggleEditable(CheckButton : PGtkWidget; TText : PGtkWidget);
begin
  GtkTextSetEditable(PGtkText(TText), GtkToggleButtonGetActive(PGtkToggleButton(Checkbutton)));
end;

procedure TextToggleWordWrap(CheckButton : PGtkWidget; TText : PGtkWidget);
begin
  GtkTextSetWordWrap(PGtkText(TText), GtkToggleButtonGetActive(PGtkToggleButton(Checkbutton)));
end;

procedure CloseApplication(Widget : PGtkWidget; Data : GPointer);
begin
  GtkMainQuit;
end;

var
  Window, Box1, Box2, HBox, Button, Check, Separator, Table, VScrollbar, TText : PGtkWidget;
  CMap : PGdkColormap;
  Color : GdkColor;
  FixedFont : PGdkFont;
  InFile : Text;

begin
  GtkInit;

  Window := GtkWindowNew(GTK_WINDOW_TOPLEVEL);
  GtkWidgetSetUsize(Window, 600, 500);
  GtkWindowSetPolicy(PGtkWindow(Window), G_TRUE, G_TRUE, G_FALSE);
  GtkSignalConnect(PGtkObject(Window), 'destroy', @CloseApplication, nil);
  GtkWindowSetTitle(PGtkWindow(Window), 'Text Widget Example');
  GtkContainerSetBorderWidth(PGtkContainer(Window), 0);

  Box1 := GtkVboxNew(G_FALSE, 0);
  GtkContainerAdd(PGtkContainer(Window), Box1);
  GtkWidgetShow(Box1);
  
  Box2 := GtkVboxNew(G_FALSE, 10);
  GtkContainerSetBorderWidth(PGtkContainer(Box2), 10);
  GtkBoxPackStart(PGtkBox(Box1), Box2, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(Box2);
  
  Table := GtkTableNew(2, 2, G_FALSE);
  GtkTableSetRowSpacing(PGtkTable(Table), 0, 2);
  GtkTableSetColSpacing(PGtkTable(Table), 0, 2);
  GtkBoxPackStart(PGtkBox(Box2), Table, G_TRUE, G_TRUE, 0);
  GtkWidgetShow(Table);
  
  { Create the GtkText widget }
  TText := GtkTextNew(nil, nil);
  GtkTextSetEditable(PGtkText(TText), G_TRUE);
  GtkTableAttach(PGtkTable(Table), TText, 0, 1, 0, 1, GTK_EXPAND or GTK_SHRINK or GTK_FILL, GTK_EXPAND or GTK_SHRINK or GTK_FILL, 0, 0);
  GtkWidgetShow(TText);

  { Add a vertical scrollbar to the GtkText widget }
  Vscrollbar := GtkVscrollbarNew(PGtkText(TText)^.VAdj);
  GtkTableAttach(PGtkTable(Table), VScrollbar, 1, 2, 0, 1, GTK_FILL, GTK_EXPAND or GTK_SHRINK or GTK_FILL, 0, 0);
  GtkWidgetShow(VScrollbar);

  { Get the system color map and allocate the color red }
  CMap := GdkColormapGetSystem;
  Color.Red := 16#ffff;
  Color.Green := 0;
  Color.Blue := 0;
  if GdkColorAlloc(CMap, @Color) = G_FALSE then
    Writeln('couldn''t allocate color');

  { Load a fixed font }
  FixedFont := GdkFontLoad('-misc-fixed-medium-r-*-*-*-140-*-*-*-*-*-*');

  { Realizing A Widget Creates A Window For It,
    Ready For Us To Insert Some Text }
  GtkWidgetRealize(TText);

  { Freeze the text widget, ready for multiple updates }
  GtkTextFreeze(PGtkText(TText));
  
  { Insert some colored text }
  GtkTextInsert(PGtkText(TText), nil, @TText^.Style^.Black, nil, 'Supports ', -1);
  GtkTextInsert(PGtkText(TText), nil, @Color, nil, 'colored ', -1);
  GtkTextInsert(PGtkText(TText), nil, @TText^.Style^.Black, nil, 'text and different ', -1);
  GtkTextInsert(PGtkText(TText), FixedFont, @TText^.Style^.Black, nil, "fonts\n\n", -1);
  
  { Load the file text.pas into the text window }
  Reset(InFile, 'text.pas');

  while not EOF(InFile) do
    begin
      var Str : TString;
      Readln(InFile, Str);
      GtkTextInsert(PGtkText(TText), FixedFont, nil, nil, Str, Length(Str));
      GtkTextInsert(PGtkText(TText), FixedFont, nil, nil, "\n", 1);
    end;

  Close(InFile);

  { Thaw the text widget, allowing the updates to become visible }  
  GtkTextThaw(PGtkText(TText));

  HBox := GtkHButtonBoxNew;
  GtkBoxPackStart(PGtkBox(Box2), HBox, G_FALSE, G_FALSE, 0);
  GtkWidgetShow(HBox);

  Check := GtkCheckButtonNewWithLabel('Editable');
  GtkBoxPackStart(PGtkBox(HBox), Check, G_FALSE, G_FALSE, 0);
  GtkSignalConnect(PGtkObject(Check), 'toggled', @TextToggleEditable, TText);
  GtkToggleButtonSetActive(PGtkToggleButton(Check), G_TRUE);
  GtkWidgetShow(Check);
  Check := GtkCheckButtonNewWithLabel('Wrap Words');
  GtkBoxPackStart(PGtkBox(HBox), Check, G_FALSE, G_TRUE, 0);
  GtkSignalConnect(PGtkObject(Check), 'toggled', @TextToggleWordWrap, TText);
  GtkToggleButtonSetActive(PGtkToggleButton(Check), G_FALSE);
  GtkWidgetShow(Check);

  Separator := GtkHseparatorNew;
  GtkBoxPackStart(PGtkBox(Box1), Separator, G_FALSE, G_TRUE, 0);
  GtkWidgetShow(Separator);

  Box2 := GtkVboxNew(G_FALSE, 10);
  GtkContainerSetBorderWidth(PGtkContainer(Box2), 10);
  GtkBoxPackStart(PGtkBox(Box1), Box2, G_FALSE, G_TRUE, 0);
  GtkWidgetShow(Box2);
  
  Button := GtkButtonNewWithLabel('close');
  GtkSignalConnect(PGtkObject(Button), 'clicked', @CloseApplication, nil);
  GtkBoxPackStart(PGtkBox(Box2), Button, G_TRUE, G_TRUE, 0);
  GTK_WIDGET_SET_FLAGS(Button, GTK_CAN_DEFAULT);
  GtkWidgetGrabDefault(Button);
  GtkWidgetShow(Button);

  GtkWidgetShow(Window);

  GtkMain;
end.
