{ Report bugs to Nicola Girardi <nikke@swlibero.org> }
program Tree;

{$include "../units/gtk.inc"}
{$ignore-function-results}

uses GPC;

{ For all the GtkItem:: and GtkTreeItem:: signals }
procedure CbItemSignal(Item : PGtkWidget; SigName : PGChar);
var
  Name : PGChar;
  LLabel : PGtkLabel;
begin
  { It's A Bin, So It Has One Child, Which We Know To Be A Label, So Get That }
  LLabel := PGtkLabel(PGtkBin(Item)^.Child);
  { Get the text of the label }
  GtkLabelGet(LLabel, Name);
  { Get the level of the tree which the item is in }
  Writeln(CString2String(SigName), ' called for item ', CString2String(Name), '->', Word(Item), ', level ', PGtkTree(Item^.Parent)^.Level);
end;

{ Note that this is never called }
procedure CbUnselectChild(RootTree, Child, SubTree : PGtkWidget);
begin
  Writeln('unselect_child called for root tree ', Word(RootTree), ', SubTree ', WorD(SubTree), ', child ', Word(Child), '.');
end;

{ Note That This Is Called Every Time The User Clicks On An Item,
  Whether It Is Already Selected Or Not. }
procedure CbSelectChild(RootTree, Child, SubTree : PGtkWidget);
begin
  Writeln('select_child called for root tree ', Word(RootTree), ', SubTree ', WorD(SubTree), ', child ', Word(Child), '.');
end;

procedure CbSelectionChanged(Tree : PGtkWidget);
var
  i : PGList;
begin
  Writeln('selection_change called for tree ', Word(Tree));
  Writeln('selected objects are:');

  i := GTK_TREE_SELECTION(Tree);

  while i <> nil do
    begin
      var Name : PGChar;
      var LLabel : PGtkLabel;
      var Item : PGtkWidget;

      { Get a GtkWidget pointer from the list node }
      Item := PGtkWidget(i^.Data);
      LLabel := PGtkLabel(PGtkBin(Item)^.Child);
      GtkLabelGet(LLabel, Name);
      Writeln('    ', CString2String(Name), ' on level ', PGtkTree(Item^.Parent)^.Level);
      i := i^.Next;
    end;
end;

var
  Window, Scrolled, Tree : PGtkWidget;
  ItemNames : static array[0..4] of String(12) = ( 'Foo', 'Bar', 'Baz', 'Quux', 'Maurice' );
  i : Integer;

begin
  GtkInit;

  { A generic toplevel window }
  Window := GtkWindowNew(GTK_WINDOW_TOPLEVEL);
  GtkSignalConnect(PGtkObject(Window), 'delete_event', @GtkMainQuit, nil);
  GtkContainerSetBorderWidth(PGtkContainer(Window), 5);

  { A generic scrolled window }
  Scrolled := GtkScrolledWindowNew(nil, nil);
  GtkScrolledWindowSetPolicy(PGtkScrolledWindow(Scrolled), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  GtkWidgetSetUsize(Scrolled, 150, 200);
  GtkContainerAdd(PGtkContainer(Window), Scrolled);
  GtkWidgetShow(Scrolled);
  
  { Create the root tree }
  Tree := GtkTreeNew;
  Writeln('root tree is ', WorD(Tree));
  { Connect all GtkTree:: signals }
  GtkSignalConnect(PGtkObject(Tree), 'select_child', @CbSelectChild, Tree);
  GtkSignalConnect(PGtkObject(Tree), 'unselect_child', @CbUnselectChild, Tree);
  GtkSignalConnect(PGtkObject(Tree), 'selection_changed', @CbSelectionChanged, Tree);
  { Add it to the scrolled window }
  GtkScrolledWindowAddWithViewport(PGtkScrolledWindow(Scrolled), Tree);
  { Set the selection mode }
  GtkTreeSetSelectionMode(PGtkTree(Tree), GTK_SELECTION_MULTIPLE);
  { Show it }
  GtkWidgetShow(tree);

  for i := 0 to 4 do
    begin
      var SubTree, Item : PGtkWidget;
      var j : Integer;

      { Create a tree item }
      Item := GtkTreeItemNewWithLabel(ItemNames[i]);
      { Connect all GtkItem:: and GtkTreeItem:: signals }
      GtkSignalConnect(PGtkObject(Item), 'select', @CbItemSignal, @'select');
      GtkSignalConnect(PGtkObject(Item), 'deselect', @CbItemSignal, @'deselect');
      GtkSignalConnect(PGtkObject(Item), 'toggle', @CbItemSignal, @'toggle');
      GtkSignalConnect(PGtkObject(Item), 'expand', @CbItemSignal, @'expand');
      GtkSignalConnect(PGtkObject(Item), 'collapse', @CbItemSignal, @'collapse');
      { Add it to the parent tree }
      GtkTreeAppend(PGtkTree(Tree), Item);
      { Show it - this can be done at any time }
      GtkWidgetShow(item);
      { Create this item's SubTree }
      SubTree := GtkTreeNew;
      Writeln('-> item ', ItemNames[i], '->', Word(Item), ', SubTree ', Word(SubTree));

      { This Is Still Necessary If You Want These Signals To Be Called
        For The SubTree's Children.  Note That SelectionChange Will Be
       Signalled For The Root Tree Regardless. }
      GtkSignalConnect(PGtkObject(SubTree), 'select_child', @CbSelectChild, SubTree);
      GtkSignalConnect(PGtkObject(SubTree), 'unselect_child', @CbUnselectChild, SubTree);
      { This Has Absolutely No Effect, Because It Is Completely Ignored In SubTrees }
      GtkTreeSetSelectionMode(PGtkTree(SubTree), GTK_SELECTION_SINGLE);
      { Neither Does This, But For A Rather Different Reason - The
        ViewMode And ViewLine Values Of A Tree Are Propagated To
        SubTrees When They Are Mapped.  So, Setting It Later On Would
        Actually Have A(somewhat Unpredictable) Effect }
      GtkTreeSetViewMode(PGtkTree(SubTree), GTK_TREE_VIEW_ITEM);
      { Set This Item's SubTree - Note That You Cannot Do This Until
        AFTER The Item Has Been Added To Its Parent Tree! }
      GtkTreeItemSetSubTree(PGtkTreeItem(Item), SubTree);

      for j := 0 to 4 do
        begin
          var SubItem : PGtkWidget;

          { Create a SubTree item, in much the same way }
          SubItem := GtkTreeItemNewWithLabel(ItemNames[j]);
          { Connect all GtkItem:: and GtkTreeItem:: signals }
          GtkSignalConnect(PGtkObject(SubItem), 'select', @CbItemSignal, @'select');
          GtkSignalConnect(PGtkObject(SubItem), 'deselect', @CbItemSignal, @'deselect');
          GtkSignalConnect(PGtkObject(SubItem), 'toggle', @CbItemSignal, @'toggle');
          GtkSignalConnect(PGtkObject(SubItem), 'expand', @CbItemSignal, @'expand');
          GtkSignalConnect(PGtkObject(SubItem), 'collapse', @CbItemSignal, @'collapse');
          Writeln('-> -> item ', ItemNames[j], '->', Word(SubItem));
          { Add it to its parent tree }
          GtkTreeAppend(PGtkTree(SubTree), SubItem);
          { Show it }
          GtkWidgetShow(SubItem);
        end;
    end;

  { Show the window and loop endlessly }
  GtkWidgetShow(Window);
  GtkMain;
end.
