{ Report bugs to Nicola Girardi <nikke@swlibero.org> }
unit GtkDial;

interface

{$include "../../units/gtk.inc"}
{$ignore-function-results}

function GTK_TYPE_DIAL : TGtkType;
function GTK_IS_DIAL (Obj : Pointer) : Boolean;
function GTK_IS_DIAL_CLASS (Klass : Pointer) : Boolean;

type
  PGtkDialClass = ^TGtkDialClass;
  PGtkDial      = ^TGtkDial;

  TGtkDialClass = record
    ParentClass : TGtkWidgetClass;
  end;

  TGtkDial = record
    Widget : TGtkWidget;

    { update policy (GTK_UPDATE_[CONTINUOUS/DELAYED/DISCONTINUOUS]) }
    Policy : GUint2;

    { Button currently pressed or 0 if none }
    Button : GUint8;

    { Dimensions of dial components }
    Radius, PointerWidth : GInt;

    { ID of update timer, or 0 if none }
    Timer : Guint32;

    { Current angle }
    Angle, LastAngle : GFloat;

    { Old values from adjustment stored so we know when something changes }
    OldValue, OldLower, OldUpper : GFloat;

    { The adjustment object that stores the data for this dial }
    Adjustment : PGtkAdjustment;
  end;

function GtkDialNew(Adjustment : PGtkAdjustment) : PGtkWidget;
function GtkDialGetType : TGtkType;
function GtkDialGetAdjustment(Dial : PGtkDial) : PGtkAdjustment;
procedure GtkDialSetUpdatePolicy(Dial : PGtkDial; Policy : TGtkUpdateType);
procedure GtkDialSetAdjustment(Dial : PGtkDial; Adjustment : PGtkAdjustment);

{ !! Private }
procedure GtkDialClassInit(Klass : PGtkDialClass);
procedure GtkDialInit(Dial : PGtkDial);
procedure GtkDialDestroy(OObject : PGtkObject);
procedure GtkDialRealize(Widget : PGtkWidget);
procedure GtkDialSizeRequest(Widget : PGtkWidget; Requisition : PGtkRequisition);
procedure GtkDialSizeAllocate(Widget : PGtkWidget; Allocation : PGtkAllocation);
function GtkDialExpose(Widget : PGtkWidget; Event : PGdkEventExpose) : GInt;
function GtkDialButtonPress(Widget : PGtkWidget; Event : PGdkEventButton) : GInt;
function GtkDialButtonRelease(Widget : PGtkWidget; Event : PGdkEventButton) : GInt;
function GtkDialMotionNotify(Widget : PGtkWidget; Event : PGdkEventMotion) : GInt;
function GtkDialTimer(Dial : PGtkDial) : GInt;
procedure GtkDialUpdateMouse(Dial : PGtkDial; x, y : GInt);
procedure GtkDialUpdate(Dial : PGtkDial);
procedure GtkDialAdjustmentChanged(Adjustment : PGtkAdjustment; Data : GPointer);
procedure GtkDialAdjustmentValueChanged(Adjustment : PGtkAdjustment; Data : GPointer);

implementation

(* @@ double atan2(double y, double x);
      double fabs(double x); *)
function c_atan2(y, x : Double) : Double; asmname 'atan2';
function c_fabs(x : Double) : Double; asmname 'fabs';

{$define SCROLL_DELAY_LENGTH 300}
{$define DIAL_DEFAULT_SIZE 100}

{ Local data }
var
  ParentClass : static PGtkWidgetClass = nil;

procedure GtkDialClassInit(Klass : PGtkDialClass);
var
  ObjectClass : PGtkObjectClass;
  WidgetClass : PGtkWidgetClass;
begin
  ObjectClass := PGtkObjectClass(Klass);
  WidgetClass := PGtkWidgetClass(Klass);

  ParentClass := GtkTypeClass(GTK_TYPE_WIDGET);

  ObjectClass^.Destroy := @GtkDialDestroy;

  WidgetClass^.Realize := @GtkDialRealize;
  WidgetClass^.ExposeEvent := @GtkDialExpose;
  WidgetClass^.SizeRequest := @GtkDialSizeRequest;
  WidgetClass^.SizeAllocate := @GtkDialSizeAllocate;
  WidgetClass^.ButtonPressEvent := @GtkDialButtonPress;
  WidgetClass^.ButtonReleaseEvent := @GtkDialButtonRelease;
  WidgetClass^.MotionNotifyEvent := @GtkDialMotionNotify;
end;

procedure GtkDialInit(Dial : PGtkDial);
begin
  Dial^.Button := 0;
  Dial^.Policy := GTK_UPDATE_CONTINUOUS;
  Dial^.Timer := 0;
  Dial^.Radius := 0;
  Dial^.PointerWidth := 0;
  Dial^.Angle := 0.0;
  Dial^.OldValue := 0.0;
  Dial^.OldLower := 0.0;
  Dial^.OldUpper := 0.0;
  Dial^.Adjustment := nil;
end;

function GtkDialGetType : TGtkType;
var
  DialType : static TGtkType = 0;
begin
  if DialType = 0 then
    begin
      var DialInfo : TGtkTypeInfo =
      (
        'GtkDial',
        SizeOf(TGtkDial),
        SizeOf(TGtkDialClass),
        @GtkDialClassInit,
        @GtkDialInit,
        nil,
        nil,
        nil
      );
      DialType := GtkTypeUnique(GTK_TYPE_WIDGET, @DialInfo);
    end;

  GtkDialGetType := DialType;
end;

function GtkDialNew(Adjustment : PGtkAdjustment) : PGtkWidget;
var
  Dial : PGtkDial;
begin
  Dial := GtkTypeNew(GtkDialGetType);

  if Adjustment = nil then
    Adjustment := PGtkAdjustment(GtkAdjustmentNew(0.0, 0.0, 0.0, 0.0, 0.0, 0.0));

  GtkDialSetAdjustment(Dial, Adjustment);

  GtkDialNew := PGtkWidget(Dial);
end;

procedure GtkDialDestroy(OObject : PGtkObject);
var
  Dial : PGtkDial;
begin
  if OObject = nil then Exit;
  if not GTK_IS_DIAL(OObject) then Exit;

  Dial := PGtkDial(OObject);

  if Dial^.Adjustment <> nil then
    GtkObjectUnref(PGtkObject(Dial^.Adjustment));

  if PGtkObjectClass(ParentClass)^.Destroy <> nil then
    PGtkObjectClass(ParentClass)^.Destroy^(OObject);
end;

function GtkDialGetAdjustment(Dial : PGtkDial) : PGtkAdjustment;
begin
  GtkDialGetAdjustment := nil;
  if Dial = nil then Exit;
  if not GTK_IS_DIAL(Dial) then Exit;
  GtkDialGetAdjustment := Dial^.adjustment;
end;

procedure GtkDialSetUpdatePolicy(Dial : PGtkDial; Policy : TGtkUpdateType);
begin
  if Dial = nil then Exit;
  if not GTK_IS_DIAL(Dial) then Exit;

  Dial^.Policy := Policy;
end;

procedure GtkDialSetAdjustment(Dial : PGtkDial; Adjustment : PGtkAdjustment);
begin
  if Dial = nil then Exit;
  if not GTK_IS_DIAL(Dial) then Exit;

  if Dial^.Adjustment <> nil then
    begin
      GtkSignalDisconnectByData(PGtkObject(Dial^.Adjustment), Dial);
      GtkObjectUnref(PGtkObject(Dial^.Adjustment));
    end;

  Dial^.Adjustment := Adjustment;
  GtkObjectRef(PGtkObject(Dial^.Adjustment));

  GtkSignalConnect(PGtkObject(Adjustment), 'changed', @GtkDialAdjustmentChanged, Dial);
  GtkSignalConnect(PGtkObject(Adjustment), 'value_changed', @GtkDialAdjustmentValueChanged, Dial);

  Dial^.OldValue := Adjustment^.VValue;
  Dial^.OldLower := Adjustment^.Lower;
  Dial^.OldUpper := Adjustment^.Upper;

  GtkDialUpdate(Dial);
end;

procedure GtkDialRealize(Widget : PGtkWidget);
var
  Dial : PGtkDial;
  Attributes : GdkWindowAttr;
  AttributesMask : GInt;
begin
  if Widget = nil then Exit;
  if not GTK_IS_DIAL(Widget) then Exit;

  GTK_WIDGET_SET_FLAGS(Widget, GTK_REALIZED);
  Dial := PGtkDial(Widget);

  Attributes.x := Widget^.Allocation.x;
  Attributes.y := Widget^.Allocation.y;
  Attributes.Width := Widget^.Allocation.Width;
  Attributes.Height := Widget^.Allocation.Height;
  Attributes.WClass := GDK_INPUT_OUTPUT;
  Attributes.WindowType := GDK_WINDOW_CHILD;
  Attributes.EventMask := GtkWidgetGetEvents(Widget) or GDK_EXPOSURE_MASK or GDK_BUTTON_PRESS_MASK or GDK_BUTTON_RELEASE_MASK or GDK_POINTER_MOTION_MASK or GDK_POINTER_MOTION_HINT_MASK;
  Attributes.visual := GtkWidgetGetVisual(Widget);
  Attributes.colormap := GtkWidgetGetColormap(Widget);

  AttributesMask := GDK_WA_X or GDK_WA_Y or GDK_WA_VISUAL or GDK_WA_COLORMAP;
  Widget^.Window := GdkWindowNew(Widget^.Parent^.Window, @Attributes, AttributesMask);
  Widget^.Style := GtkStyleAttach(Widget^.Style, Widget^.Window);

  GdkWindowSetUserData(Widget^.Window, Widget);

  GtkStyleSetBackground(Widget^.Style, Widget^.Window, GTK_STATE_ACTIVE);
end;

procedure GtkDialSizeRequest(Widget : PGtkWidget; Requisition : PGtkRequisition);
begin
  Requisition^.Width := DIAL_DEFAULT_SIZE;
  Requisition^.Height := DIAL_DEFAULT_SIZE;
end;

procedure GtkDialSizeAllocate(Widget : PGtkWidget; Allocation : PGtkAllocation);
var
  Dial : PGtkDial;
begin
  if Widget = nil then Exit;
  if not GTK_IS_DIAL(Widget) then Exit;
  if Allocation = nil then Exit;

  Widget^.Allocation := Allocation^;
  Dial := PGtkDial(Widget);

  if GTK_WIDGET_REALIZED(Widget) then
    GdkWindowMoveResize(Widget^.Window, Allocation^.x, Allocation^.y, Allocation^.Width, Allocation^.Height);

  Dial^.Radius := Round(Min(Allocation^.Width, Allocation^.Height) * 0.45);
  Dial^.PointerWidth := Round(Dial^.Radius / 5);
end;

function GtkDialExpose(Widget : PGtkWidget; Event : PGdkEventExpose) : GInt;
var
  Dial : PGtkDial;
  Points : array[0..4] of GdkPoint;
  s, c, Theta, Last, Increment : GDouble;
  BlankStyle : PGtkStyle;
  xc, yc, Upper, Lower, TickLength, i, IInc : GInt;
begin
  GtkDialExpose := G_FALSE;
  if (Widget = nil) or (not GTK_IS_DIAL(Widget)) or (Event = nil) then
    Exit;
  if Event^.Count > 0 then
    Exit;
  
  Dial := PGtkDial(Widget);

(*  GdkWindowClearArea(widget^.window,
                         0, 0,
                         Widget^.allocation.width,
                         Widget^.allocation.height);
*)
  xc := Widget^.Allocation.Width div 2;
  yc := Widget^.Allocation.Height div 2;

  Upper := Round(Dial^.Adjustment^.Upper);
  Lower := Round(Dial^.Adjustment^.Lower);

  { Erase old pointer }

  s := Sin(Dial^.LastAngle);
  c := Cos(Dial^.LastAngle);
  Dial^.LastAngle := Dial^.Angle;

  Points[0].x := Round(xc + s*Dial^.PointerWidth/2);
  Points[0].y := Round(yc + c*Dial^.PointerWidth/2);
  Points[1].x := Round(xc + c*Dial^.Radius);
  Points[1].y := Round(yc - s*Dial^.Radius);
  Points[2].x := Round(xc - s*Dial^.PointerWidth/2);
  Points[2].y := Round(yc - c*Dial^.PointerWidth/2);
  Points[3].x := Round(xc - c*Dial^.Radius/10);
  Points[3].y := Round(yc + s*Dial^.Radius/10);
  Points[4].x := Points[0].x;
  Points[4].y := Points[0].y;

  Blankstyle := GtkStyleNew;
  Blankstyle^.BgGc[GTK_STATE_NORMAL] := Widget^.Style^.BgGc[GTK_STATE_NORMAL];
  Blankstyle^.DarkGc[GTK_STATE_NORMAL] := Widget^.Style^.BgGc[GTK_STATE_NORMAL];
  Blankstyle^.LightGc[GTK_STATE_NORMAL] := Widget^.Style^.BgGc[GTK_STATE_NORMAL];
  Blankstyle^.BlackGc := Widget^.Style^.BgGc[GTK_STATE_NORMAL];

  GtkDrawPolygon(BlankStyle, Widget^.Window, GTK_STATE_NORMAL, GTK_SHADOW_OUT, @Points[0], 5, G_FALSE);
  GtkStyleUnref(Blankstyle);

  { Draw ticks }
  if Upper - Lower = 0 then
    Exit;

  Increment := (100*Pi)/(Dial^.Radius*Dial^.Radius);

  IInc := Upper - Lower;

  while IInc < 100 do
    IInc := IInc * 10;
  while IInc >= 1000 do
    IInc := IInc div 10;
  Last := -1;

  for i := 0 to IInc do
    begin
      Theta := i*Pi/(18*IInc/24.0) - Pi/6.0;
      if Theta - Last < Increment then
        Continue;
      Last := Theta;
      s := Sin(Theta);
      c := Cos(Theta);
      if (i mod Round(IInc/10)) = 0 then
        TickLength := Dial^.PointerWidth
      else
        TickLength := Dial^.PointerWidth div 2;

      GdkDrawLine(Widget^.Window, Widget^.Style^.FgGc[Widget^.State], Round(xc + c*(Dial^.Radius - TickLength)), Round(yc - s*(Dial^.Radius - TickLength)), Round(xc + c*Dial^.Radius), Round(yc - s*Dial^.Radius));
    end;

  { Draw pointer }

  s := Sin(Dial^.Angle);
  c := Cos(Dial^.Angle);
  Dial^.LastAngle := Dial^.Angle;

  Points[0].x := Round(xc + s*Dial^.PointerWidth/2);
  Points[0].y := Round(yc + c*Dial^.PointerWidth/2);
  Points[1].x := Round(xc + c*Dial^.Radius);
  Points[1].y := Round(yc - s*Dial^.Radius);
  Points[2].x := Round(xc - s*Dial^.PointerWidth/2);
  Points[2].y := Round(yc - c*Dial^.PointerWidth/2);
  Points[3].x := Round(xc - c*Dial^.Radius/10);
  Points[3].y := Round(yc + s*Dial^.Radius/10);
  Points[4].x := Points[0].x;
  Points[4].y := Points[0].y;

  GtkDrawPolygon(Widget^.Style, Widget^.Window, GTK_STATE_NORMAL, GTK_SHADOW_OUT, @Points[0], 5, G_TRUE);
end;

function GtkDialButtonPress(Widget : PGtkWidget; Event : PGdkEventButton) : GInt;
var
  Dial : PGtkDial;
  dx, dy : GInt;
  s, c, DParallel, DPerpendicular : Double;
begin
  GtkDialButtonPress := G_FALSE;
  if Widget = nil then Exit;
  if not GTK_IS_DIAL(Widget) then Exit;
  if Event = nil then Exit;

  Dial := PGtkDial(Widget);

  { Determine If Button Press Was Within Pointer Region - We
    Do This By Computing The Parallel And Perpendicular Distance Of
    The Point Where The Mouse Was Pressed From The Line Passing Through
    The Pointer }
  
  dx := Round(Event^.x - Widget^.Allocation.Width/2);
  dy := Round(Widget^.Allocation.Height/2 - Event^.y);
  
  s := Sin(Dial^.Angle);
  c := Cos(Dial^.Angle);
  
  DParallel := s*dy + c*dx;
  DPerpendicular := c_fabs(s*dx - c*dy);

  if (Dial^.Button = 0) and (DPerpendicular < Dial^.PointerWidth/2) and (DParallel > -Dial^.PointerWidth) then
    begin
      GtkGrabAdd(widget);
      Dial^.Button := Event^.Button;
      GtkDialUpdateMouse(Dial, Round(Event^.x), Round(Event^.y));
    end;
end;

function GtkDialButtonRelease(Widget : PGtkWidget; Event : PGdkEventButton) : GInt;
var
  Dial : PGtkDial;
begin
  GtkDialButtonRelease := G_FALSE;
  if Widget = nil then Exit;
  if not GTK_IS_DIAL(Widget) then Exit;
  if Event = nil then Exit;

  Dial := PGtkDial(Widget);

  if Dial^.Button = Event^.Button then
    begin
      GtkGrabRemove(Widget);
      Dial^.Button := 0;

      if Dial^.Policy = GTK_UPDATE_DELAYED then
        GtkTimeoutRemove(Dial^.Timer);
      
      if (Dial^.Policy <> GTK_UPDATE_CONTINUOUS) and (Dial^.OldValue <> Dial^.Adjustment^.VValue) then
        GtkSignalEmitByName(PGtkObject(Dial^.Adjustment), 'value_changed');
    end;
end;

function GtkDialMotionNotify(Widget : PGtkWidget; Event : PGdkEventMotion) : GInt;
var
  Dial : PGtkDial;
  Mods : GdkModifierType;
  x, y, Mask : GInt;
begin
  GtkDialMotionNotify := G_FALSE;
  if Widget = nil then Exit;
  if not GTK_IS_DIAL(Widget) then Exit;
  if Event = nil then Exit;

  Dial := PGtkDial(Widget);

  if Dial^.Button <> 0 then
    begin
      x := Round(Event^.x);
      y := Round(Event^.y);

      if (Event^.IsHint <> G_FALSE) or (Event^.Window <> Widget^.Window) then
        GdkWindowGetPointer(Widget^.Window, x, y, Mods);

      if Dial^.Button = 1 then
        Mask := GDK_BUTTON1_MASK
      else if Dial^.Button = 2 then
        Mask := GDK_BUTTON2_MASK
      else if Dial^.Button = 3 then
        Mask := GDK_BUTTON3_MASK
      else
        Mask := 0;

      if (Mods and Mask) <> 0 then
        GtkDialUpdateMouse(Dial, x, y);
    end;
end;

function GtkDialTimer(Dial : PGtkDial) : GInt;
begin
  GtkDialTimer := G_FALSE;
  if Dial = nil then Exit;
  if not GTK_IS_DIAL(Dial) then Exit;

  if Dial^.Policy = GTK_UPDATE_DELAYED then
    GtkSignalEmitByName(PGtkObject(Dial^.Adjustment), 'value_changed');
end;

procedure GtkDialUpdateMouse(Dial : PGtkDial; x, y : GInt);
var
  Xc, Yc : GInt;
  OldValue : GFloat;
begin
  if Dial = nil then Exit;
  if not GTK_IS_DIAL(Dial) then Exit;

  Xc := PGtkWidget(Dial)^.Allocation.Width div 2;
  Yc := PGtkWidget(Dial)^.Allocation.Height div 2;

  OldValue := Dial^.Adjustment^.VValue;
  Dial^.Angle := c_atan2(Yc-y, x-Xc);

  if Dial^.Angle < -Pi/2.0 then
    Dial^.Angle := Dial^.Angle + 2*Pi;

  if Dial^.Angle < -Pi/6 then
    Dial^.Angle := -Pi/6;

  if Dial^.Angle > 7.0*Pi/6.0 then
    Dial^.Angle := 7.0*Pi/6.0;

  Dial^.Adjustment^.VValue := Dial^.Adjustment^.Lower +(7.0*Pi/6 - Dial^.Angle) * (Dial^.Adjustment^.Upper - Dial^.Adjustment^.Lower) /(4.0*Pi/3.0);

  if Dial^.Adjustment^.VValue <> OldValue then
    begin
      if Dial^.Policy = GTK_UPDATE_CONTINUOUS then
        GtkSignalEmitByName(PGtkObject(Dial^.Adjustment), 'value_changed')
      else
        begin
          GtkWidgetDraw(PGtkWidget(Dial), nil);
          if Dial^.Policy = GTK_UPDATE_DELAYED then
            begin
              if Dial^.Timer <> 0 then
                GtkTimeoutRemove(Dial^.Timer);
              Dial^.Timer := GtkTimeoutAdd(SCROLL_DELAY_LENGTH, @GtkDialTimer, Dial);
            end;
        end;
    end;
end;

procedure GtkDialUpdate(Dial : PGtkDial);
var
  NewValue : GFloat;
begin
  if Dial = nil then Exit;
  if not GTK_IS_DIAL(Dial) then Exit;

  NewValue := Dial^.Adjustment^.VValue;

  if NewValue < Dial^.Adjustment^.Lower then
    NewValue := Dial^.Adjustment^.Lower;

  if NewValue > Dial^.Adjustment^.Upper then
    NewValue := Dial^.Adjustment^.Upper;

  if NewValue <> Dial^.Adjustment^.VValue then
    begin
      Dial^.Adjustment^.VValue := NewValue;
      GtkSignalEmitByName(PGtkObject(Dial^.Adjustment), 'value_changed');
    end;

  Dial^.Angle := 7.0*Pi/6.0 - (NewValue - Dial^.Adjustment^.Lower) * 4.0*Pi/3.0 / (Dial^.Adjustment^.Upper - Dial^.Adjustment^.Lower);
  GtkWidgetDraw(PGtkWidget(Dial), nil);
end;

procedure GtkDialAdjustmentChanged(Adjustment : PGtkAdjustment; Data : GPointer);
var
  Dial : PGtkDial;
begin
  if Adjustment = nil then Exit;
  if Data = nil then Exit;

  Dial := PGtkDial(Data);

  if (Dial^.OldValue <> Adjustment^.VValue) or (Dial^.OldLower <> Adjustment^.Lower) or (Dial^.OldUpper <> Adjustment^.Upper) then
    begin
      GtkDialUpdate(Dial);

      Dial^.OldValue := Adjustment^.VValue;
      Dial^.OldLower := Adjustment^.Lower;
      Dial^.OldUpper := Adjustment^.Upper;
    end;
end;

procedure GtkDialAdjustmentValueChanged(Adjustment : PGtkAdjustment; Data : GPointer);
var
  Dial : PGtkDial;
begin
  if Adjustment = nil then Exit;
  if Data = nil then Exit;

  Dial := PGtkDial(Data);

  if Dial^.OldValue <> Adjustment^.VValue then
    begin
      GtkDialUpdate(Dial);
      Dial^.OldValue := Adjustment^.VValue;
    end;
end;

function GTK_TYPE_DIAL : TGtkType;
begin
  GTK_TYPE_DIAL := GtkDialGetType;
end;

function GTK_IS_DIAL (Obj : Pointer) : Boolean;
begin
  GTK_IS_DIAL := (Obj <> nil) and GTK_IS_DIAL_CLASS(PGtkTypeObject(Obj)^.Klass);
end;

function GTK_IS_DIAL_CLASS (Klass : Pointer) : Boolean;
begin
  GTK_IS_DIAL_CLASS := (Klass <> nil) and (GtkTypeIsA(PGtkTypeClass(Klass)^.TType, GTK_TYPE_DIAL) = G_TRUE);
end;

end.
