{ Report bugs to Nicola Girardi <nikke@swlibero.org> }
unit Tictactoe;

interface

{$include "../../units/gtk.inc"}
{$ignore-function-results}

function IS_TICTACTOE (Obj : Pointer) : Boolean;
function IS_TICTACTOE_CLASS (Klass : Pointer) : Boolean;

type
  PTictactoeClass = ^TTictactoeClass;
  PTictactoe      = ^TTictactoe;

  TTictactoeClass = record
    ParentClass : TGtkVBoxClass;
    Tictactoe : ^procedure(TTT : PTictactoe);
  end;

  TTictactoe = record
    VBox : TGtkVBox;
    Buttons : array[0..2, 0..2] of PGtkWidget;
  end;

function TictactoeGetType : TGtkType;
function TictactoeNew : PGtkWidget;
procedure TictactoeClear(TTT : PTictactoe);

implementation

const
  TICTACTOE_SIGNAL = 0;
  LAST_SIGNAL      = 1;

var
  TictactoeSignals : array[0..LAST_SIGNAL] of GUint;

procedure TictactoeClassInit(Klass : PTictactoeClass);
var
  ObjectClass : PGtkObjectClass;
begin
  ObjectClass := PGtkObjectClass(Klass);

  TictactoeSignals[TICTACTOE_SIGNAL] :=
    GtkSignalNew('tictactoe', GTK_RUN_FIRST, ObjectClass^.TType,
                 GTK_SIGNAL_OFFSET(PTictactoeClass, Tictactoe),
                 @GtkSignalDefaultMarshaller, GTK_TYPE_NONE, 0);

  GtkObjectClassAddSignals(ObjectClass, @TictactoeSignals[0], LAST_SIGNAL);

  Klass^.Tictactoe := nil;
end;

procedure TictactoeToggle(Widget : PGtkWidget; TTT : PTictactoe);
var
  i, k : Integer;
  RWins : static array[0..7, 0..2] of Integer = ( ( 0, 0, 0 ), ( 1, 1, 1 ), ( 2, 2, 2 ),
                                                  ( 0, 1, 2 ), ( 0, 1, 2 ), ( 0, 1, 2 ),
                                                  ( 0, 1, 2 ), ( 0, 1, 2 ) );
  CWins : static array[0..7, 0..2] of Integer = ( ( 0, 1, 2 ), ( 0, 1, 2 ), ( 0, 1, 2 ),
                                                  ( 0, 0, 0 ), ( 1, 1, 1 ), ( 2, 2, 2 ),
                                                  ( 0, 1, 2 ), ( 2, 1, 0 ) );
  Success, Found : Boolean;
begin
  for k := 0 to 7 do
    begin
      Success := True;
      Found := False;

      for i := 0 to 2 do
        begin
          Success := Success and (PGtkToggleButton(TTT^.Buttons[RWins[k][i]][CWins[k][i]])^.Active <> G_FALSE);
          Found := Found or (TTT^.Buttons[RWins[k][i]][CWins[k][i]] = Widget);
        end;

      if Success and Found then
        begin
          GtkSignalEmit(PGtkObject(TTT), TictactoeSignals[TICTACTOE_SIGNAL]);
          Break;
        end;
    end;
end;

procedure TictactoeInit(TTT : PTictactoe);
var
  Table : PGtkWidget;
  i, j : Integer;
begin
  Table := GtkTableNew(3, 3, G_TRUE);
  GtkContainerAdd(PGtkContainer(TTT), Table);
  GtkWidgetShow(Table);

  for i := 0 to 2 do
    for j := 0 to 2 do
      with TTT^ do
        begin
          Buttons[i][j] := GtkToggleButtonNew;
          GtkTableAttachDefaults(PGtkTable(Table), Buttons[i][j], i, i+1, j, j+1);
          GtkSignalConnect(PGtkObject(Buttons[i][j]), 'toggled', @TictactoeToggle, TTT);
          GtkWidgetSetUsize(Buttons[i][j], 20, 20);
          GtkWidgetShow(Buttons[i][j]);
        end;
end;

function TictactoeGetType : TGtkType;
var
  TTTType : static TGtkType = 0;
begin
  if TTTType = 0 then
    begin
      var TTTInfo : TGtkTypeInfo;

      with TTTInfo do
        begin
          TypeName := String2CString('Tictactoe');
          ObjectSize := SizeOf(TTictactoe);
          ClassSize := SizeOf(TTictactoeClass);
          ClassInitFunc := @TictactoeClassInit;
          ObjectInitFunc := @TictactoeInit;
          Reserved1 := nil;
          Reserved2 := nil;
          BaseClassInitFunc := nil;
        end;

      TTTType := GtkTypeUnique(GtkVboxGetType, @TTTInfo);
    end;
  TictactoeGetType := TTTType;
end;

function TictactoeNew : PGtkWidget;
begin
  TictactoeNew := PGtkWidget(GtkTypeNew(TictactoeGetType));
end;

procedure TictactoeClear(TTT : PTictactoe);
var
  i, j : Integer;
begin
  for i := 0 to 2 do
    for j := 0 to 2 do
      with TTT^ do
        begin
          GtkSignalHandlerBlockByData(PGtkObject(Buttons[i][j]), TTT);
          GtkToggleButtonSetActive(PGtkToggleButton(Buttons[i][j]), G_FALSE);
          GtkSignalHandlerUnblockByData(PGtkObject(Buttons[i][j]), TTT);
        end;
end;

function IS_TICTACTOE (Obj : Pointer) : Boolean;
begin
  IS_TICTACTOE := (Obj <> nil) and IS_TICTACTOE_CLASS(PGtkTypeObject(Obj)^.Klass);
end;

function IS_TICTACTOE_CLASS (Klass : Pointer) : Boolean;
begin
  IS_TICTACTOE_CLASS := (Klass <> nil) and (GtkTypeIsA(PGtkTypeClass(Klass)^.TType, TictactoeGetType) = G_TRUE);
end;

begin
  var Count : Integer;
  for Count := 0 to LAST_SIGNAL do
    TictactoeSignals[Count] := 0;
end.
