{
This unit is part of the Gtk+ units package.
Copyright (C) 2001  Nicola Girardi <nikke@swlibero.org>
                    Eike Lange <eike.lange@uni-essen.de>

This unit is free software; you can redistribute it and/or
modify it under the terms of the GNU unit General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This unit is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
unit General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this unit; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

Original GTK copyright and license information follows.
}

{
GTK - The GIMP Toolkit
Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
}

{
Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
file for a list of people on the GTK+ Team.  See the ChangeLog
files for a list of changes.  These files are distributed with
GTK+ at ftp://ftp.gtk.org/pub/gtk/.
}

Unit GtkCList;

Interface

Uses
  Glib, GdkTypes, GtkSignal, GtkAdjustment, GtkAlignment, GtkContainer,
  GtkEnums, GtkLabel, GtkButton, GtkHScrollbar, GtkVScrollbar,
  GtkObject, GtkTypeUtils, GtkWidget;


Const
  { clist flags }
  GTK_CLIST_IN_DRAG             = 1 shl  0;
  GTK_CLIST_ROW_HEIGHT_SET      = 1 shl  1;
  GTK_CLIST_SHOW_TITLES         = 1 shl  2;
  GTK_CLIST_CHILD_HAS_FOCUS     = 1 shl  3;
  GTK_CLIST_ADD_MODE            = 1 shl  4;
  GTK_CLIST_AUTO_SORT           = 1 shl  5;
  GTK_CLIST_AUTO_RESIZE_BLOCKED = 1 shl  6;
  GTK_CLIST_REORDERABLE         = 1 shl  7;
  GTK_CLIST_USE_DRAG_ICONS      = 1 shl  8;
  GTK_CLIST_DRAW_DRAG_LINE      = 1 shl  9;
  GTK_CLIST_DRAW_DRAG_RECT      = 1 shl 10;

  { cell types, GtkCellType }
  GTK_CELL_EMPTY=0;
  GTK_CELL_TEXT=1;
  GTK_CELL_PIXMAP=2;
  GTK_CELL_PIXTEXT=3;
  GTK_CELL_WIDGET=4;

  { GtkCListDragPos }
  GTK_CLIST_DRAG_NONE = 0;
  GTK_CLIST_DRAG_BEFORE=1;
  GTK_CLIST_DRAG_INTO=2;
  GTK_CLIST_DRAG_AFTER=3;

  { GtkButtonAction }
  GTK_BUTTON_IGNORED = 0;
  GTK_BUTTON_SELECTS = 1;
  GTK_BUTTON_DRAGS   = 1 shl 1;
  GTK_BUTTON_EXPANDS = 1 shl 2;

Type
  TGtkCellType = Integer;
  TGtkCListDragPos = Integer;
  TGtkButtonAction = Integer;

  PGtkCList = ^TGtkCList;
  PGtkCListClass = ^TGtkCListClass;
  PGtkCListColumn = ^TGtkCListColumn;
  PGtkCListRow = ^TGtkCListRow;
  PGtkCell =  ^TGtkCell;
  PGtkCellText = ^TGtkCellText;
  PGtkCellPixmap = ^TGtkCellPixmap;
  PGtkCellPixText = ^TGtkCellPixText;
  PGtkCellWidget = ^TGtkCellWidget;
  PGtkCListCellInfo = ^TGtkCListCellInfo;
  PGtkCListDestInfo = ^TGtkCListDestInfo;
  
  TGtkCListCompareFunc = ^function (CList : PGtkCList; Ptr1, Ptr2 : GConstPointer) : GInt;

  TGtkCListCellInfo = Record
    Row, Column : gint;
  End;
  
  TGtkCListDestInfo = Record
    Cell : TGtkCListCellInfo;
    InsertPos : TGtkCListDragPos;
  End;

  TGtkCList = Record
    Container : TGtkContainer;
    Flags : guint16;
    
    { Mem chunks }
    RowMemChunk, CellMemChunk : PGMemChunk;
    FreezeCount : guint;
    
    { Allocation rectangle after the conatiner_border_width and the width of the shadow border. }
    InternalAllocation : GdkRectangle;
  
    { Rows }
    Rows, RowCenterOffset, RowHeight : gint;
    RowList, RowListEnd : PGList;
    
    { Columns }
    Columns : gint;
    ColumnTitleArea : GdkRectangle;
    TitleWindow : PGdkWindow;
    
    { Dynamicly allocated array of column structures }
    Column : PGtkCListColumn;
  
    { The scrolling window and it's height and width to make things a little speedier. }
    CListWindow : GdkWindow;
    CListWindowWidth, CListWindowHeight : gint;
    
    { Offsets for scrolling }
    HOffset, VOffset : gint;
    
    { Border shadow style }
    ShadowType : TGtkShadowType;
    
    { The list's selection mode (gtkenums.h) }
    SelectionMode : TGtkSelectionMode;
  
    { List of selected rows }
    Selection, SelectionEnd : PGList;
    UndoSelection, UndoUnselection : PGList;
    UndoAnchor : gint;
    
    { Mouse buttons }
    ButtonActions : array [0..4] of guint8;
    DragButton : guint8;
    
    { DND }
    ClickCell : TGtkCListCellInfo;

    { scroll adjustments }  
    HAdjustment, VAdjustment : PGtkAdjustment;
    
    { xor GC for the vertical drag line }  
    XorGX : PGdkGC;
    
    { GC for drawing unselected cells }
    FgGC, BgGC : PGdkGC;
    
    { Cursor used to indicate dragging }
    CursorDrag : PGdkCursor;
    
    { The current x-pixel location of the xor-drag line }
    XDrag : gint;
    
    { Focus handling }
    FocusRow : gint;
    
    { Dragging the selection }
    Anchor : gint;
    AnchorState : TGtkStateType;
    DragPos, HTimer, VTimer : gint;
    SortType : TGtkSortType;
    Compare : TGtkCListCompareFunc;
    SortColumn : gint;
  End;

  TGtkCListClass = Record
    ParentClass : TGtkContainerClass;
    SetScrollAdjustments : ^Procedure (CList : PGtkCList; HAdj, Vadj : PGtkAdjustment);
    Refresh : ^Procedure (CList : PGtkCList);
    SelectRow : ^Procedure (CList : PGtkCList; Row, Col : gint; Event :PGdkEvent);
    UnselectRow : ^Procedure (CList : PGtkCList; Row, Col : gint; Event:PGdkEvent);
    RowMove : ^Procedure (CList : PGtkCList; Source, Dest : gint);
    ClickColumn : ^Procedure (CList : PGtkCList; Col : gint);
    ResizeColumn : ^Procedure (CList : PGtkCList; Col, Width : gint);
    ToggleFocusRow : ^Procedure (CList : PGtkCList);
    SelectAll : ^Procedure (CList : PGtkCList);
    UnselectAll : ^Procedure (CList : PGtkCList);
    UndoSelection      : ^Procedure (CList : PGtkCList);
    StartSelection     : ^Procedure (CList : PGtkCList);
    EndSelection       : ^Procedure (CList : PGtkCList);
    ExtendSelection    : ^Procedure (CList : PGtkCList; ScrollType : TGtkScrollType; Position : gfloat; AutoStartSel : gboolean);
    ScrollHorizontal   : ^Procedure (CList : PGtkCList; ScrollType:TGtkScrollType; Position : gfloat);
    ScrollVertical     : ^Procedure (CList : PGtkCList; stype:TGtkScrollType; Position : gfloat);
    ToggleAddMode     : ^Procedure (CList : PGtkCList);
    AbortColumnResize : ^Procedure (CList : PGtkCList);
    ResyncSelection    : ^Procedure (CList : PGtkCList; Event : PGdkEvent);
    SelectionFind : ^Function(CList : PGtkCList; Row : gint; RowElem :PGList) : PGlist;
    DrawRow : ^Procedure (CList : PGtkCList; Area : PGdkRectangle; Row : GInt; ListRow:PGtkCListRow);
    DrawDragHighlight : ^Procedure (CList : PGtkCList; TargetRow : PGtkCListRow; TargetRowNR : gint; DragPos : TGtkClistDragPos);
    Clear               : ^Procedure (CList : PGtkCList);
    FakeUnselectAll   : ^procedure (CList : PGtkCList; Row : gint);
    SortList           : ^Procedure (CList : PGtkCList);
    InsertRow          : ^Function (CList : PGtkCList; Row : gint; TText : ppgchar) : gint;
    RemoveRow : ^Procedure (CList : PGtkCList; Row : gint);
    SetCellContents : ^Procedure (CList : PGtkCList; CListRow : PGtkCListRow; Col : gint; TType : TGtkCellType; TText : PGChar; Spacing : guint8; Pixmap : PGdkPixmap; Mask : PGdkBitmap);
    CellSizeRequest : ^Procedure (CList : PGtkCList; CListRow : PGtkCListRow; Col : gint; Requisition : PGtkRequisition);
  End;

  TGtkCListColumnFrame1 = packed record
    Visible, WidthSet, Resizeable, AutoResize, ButtonPassive : guint1;
  end;

  TGtkCListColumn = Record
    Title : PGChar;
    Area : GdkRectangle;
    Button, Window : PGdkWindow;
    Width, MinWidth, MaxWidth : gint;
    Justification : TGtkJustification;

    Frame1 : TGtkCListColumnFrame1;
  End;

  TGtkCListRow = Record
    Cell : PGtkCell;
    State : TGtkStateType;
    Foreground, Background : GdkColor;
    Style : PGtkStyle;
    Data : gpointer;
    Destroy : TGtkDestroyNotify;
    FgSet, BgSet, Selectable : guint1;
  End;

  { Cell Structures }  
  TGtkCellText = Record
    TType : TGtkCellType;
    Vertical, Horizontal : gint16;
    Style : PGtkStyle;
    TText : PGChar;
  End;

  TGtkCellPixmap = Record
    TType : TGtkCellType;
    Vertical, Horizontal : gint16;
    Style : PGtkStyle;
    Pixmap : PGdkPixmap;
    Mask : PGdkBitmap;
  End;

  TGtkCellPixText = Record
    TType : TGtkCellType;
    Vertical, Horizontal : gint16;
    Style : PGtkStyle;
    TText : PGChar;
    Spacing : guint8;
    Pixmap : PGdkPixmap;
    Mask : PGdkBitmap;
  End;

  TGtkCellWidget = Record
    TType : TGtkCellType;
    Vertical, Horizontal : gint16;
    Style : PGtkStyle;
    Widget : PGtkWidget;
  End;

  pmStruct = Record
    Pixmap : PGdkPixmap;
    Mask : PGdkBitmap;
  End;
  
  ptStruct = Record
    TText : PGChar;
    Spacing : guint8;
    Pixmap : PGdkPixmap;
    Mask : PGdkBitmap;
  End;
  
  TGtkCell = Record
    TType : TGtkCellType;
    Vertical, Horizontal : gint16;
    Style : PGtkStyle;
    case Longint of
      0: (TText: PGChar);
      1: (pm : pmStruct);
      2: (pt : ptStruct);
      3: (Widget : PGtkWidget);
  End;

{ =======  Functions ======= }

function GTK_TYPE_CLIST : TGtkType;
asmname 'gtk_clist_get_type';

function GTK_IS_CLIST (Obj : Pointer) : Boolean;
function GTK_IS_CLIST_CLASS (Klass : Pointer) : Boolean;

{ !! Not implemented. FPC-Gtk doesn't implement these, either.
#define GTK_CLIST_FLAGS(clist)             (GTK_CLIST (clist)->flags)
#define GTK_CLIST_SET_FLAG(clist,flag)     (GTK_CLIST_FLAGS (clist) |= (GTK_ ## flag))
#define GTK_CLIST_UNSET_FLAG(clist,flag)   (GTK_CLIST_FLAGS (clist) &= ~(GTK_ ## flag))
#define GTK_CLIST_IN_DRAG(clist)           (GTK_CLIST_FLAGS (clist) & GTK_CLIST_IN_DRAG)
#define GTK_CLIST_ROW_HEIGHT_SET(clist)    (GTK_CLIST_FLAGS (clist) & GTK_CLIST_ROW_HEIGHT_SET)
#define GTK_CLIST_SHOW_TITLES(clist)       (GTK_CLIST_FLAGS (clist) & GTK_CLIST_SHOW_TITLES)
#define GTK_CLIST_CHILD_HAS_FOCUS(clist)   (GTK_CLIST_FLAGS (clist) & GTK_CLIST_CHILD_HAS_FOCUS)
#define GTK_CLIST_ADD_MODE(clist)          (GTK_CLIST_FLAGS (clist) & GTK_CLIST_ADD_MODE)
#define GTK_CLIST_AUTO_SORT(clist)         (GTK_CLIST_FLAGS (clist) & GTK_CLIST_AUTO_SORT)
#define GTK_CLIST_AUTO_RESIZE_BLOCKED(clist) (GTK_CLIST_FLAGS (clist) & GTK_CLIST_AUTO_RESIZE_BLOCKED)
#define GTK_CLIST_REORDERABLE(clist)       (GTK_CLIST_FLAGS (clist) & GTK_CLIST_REORDERABLE)
#define GTK_CLIST_USE_DRAG_ICONS(clist)    (GTK_CLIST_FLAGS (clist) & GTK_CLIST_USE_DRAG_ICONS)
#define GTK_CLIST_DRAW_DRAG_LINE(clist)    (GTK_CLIST_FLAGS (clist) & GTK_CLIST_DRAW_DRAG_LINE)
#define GTK_CLIST_DRAW_DRAG_RECT(clist)    (GTK_CLIST_FLAGS (clist) & GTK_CLIST_DRAW_DRAG_RECT)
#define GTK_CLIST_ROW(_glist_) ((GtkCListRow * )((_glist_)->data))
}

{ !! "Pointer casting for cells" not implemented. }

Function GtkClistGetType : TGtkType;
  asmname 'gtk_clist_get_type';

{ Constructors useful for gtk-- wrappers }
Procedure GtkClistConstruct(CList : PGtkCList; Col : gint; Titles : ppgchar);
  asmname 'gtk_clist_construct';

{ Create a new GtkCList }
Function GtkClistNew(Col : Gint) : PGtkWidget;
  asmname 'gtk_clist_new';

Function GtkClistNewWithTitles(Col : gint; Titles : ppgchar) : PGtkWidget;
  asmname 'gtk_clist_new_with_titles';

{ Set adjustments of clist }
Procedure GtkClistSetHAdjustment(CList : PGtkCList; adj:PGtkAdjustment);
  asmname 'gtk_clist_set_hadjustment';

Procedure GtkClistSetVAdjustment(CList : PGtkCList; adj:PGtkAdjustment);
  asmname 'gtk_clist_set_vadjustment';

{ Get adjustments of clist }
Function GtkClistGetHAdjustment(CList : PGtkCList) : PGtkAdjustment;
  asmname 'gtk_clist_get_hadjustment';

Function GtkClistGetVAdjustment(CList : PGtkCList) : PGtkAdjustment;
  asmname 'gtk_clist_get_vadjustment';

{ Set the border style of the clist }
Procedure GtkCListSetShadowType(CList : PGtkCList; TType : TGtkShadowType);
  asmname 'gtk_clist_set_shadow_type';

{ Set the clist's selection mode }
Procedure GtkClistSetSelectionMode(CList : PGtkCList; Mode : TGtkSelectionMode);
  asmname 'gtk_clist_set_selection_mode';

{ Enable clists reorder ability }
Procedure GtkClistSetReorderable(CList : PGtkCList; Reorderable : gboolean);
  asmname 'gtk_clist_set_reorderable';

Procedure GtkClistSetUseDragIcons(CList : PGtkCList; UseIcons : gboolean);
  asmname 'gtk_clist_set_use_drag_icons';

Procedure GtkClistSetButtonActions(CList : PGtkCList; Button : guint; Action : guint8);
  asmname 'gtk_clist_set_button_actions';

{ Freeze all visual updates of the list, and then thaw the list after
  you have made a number of changes and the updates wil occure in a
  more efficent mannor than if you made them on a unfrozen list }
Procedure GtkClistFreeze(CList : PgtkCList);
  asmname 'gtk_clist_freeze';

Procedure GtkClistThaw(CList : PGtkCList);
  asmname 'gtk_clist_thaw';

 { show and hide the column title buttons }  
Procedure GtkClistColumnTitlesShow(CList : PGtkCList);
  asmname 'gtk_clist_column_titles_show';
Procedure GtkClistColumnTitlesHide(CList : PGtkCList);
  asmname 'gtk_clist_column_titles_hide';

{ Set the column title to be a active title (responds to button presses,
  prelights, and grabs keyboard focus), or passive where it acts as just
  a title }
Procedure GtkClistColumnTitleActive(CList : PGtkCList);
  asmname 'gtk_clist_column_title_active';

Procedure GtkClistColumnTitlePassive(CList : PGtkCList; Col:gint);
  asmname 'gtk_clist_column_title_passive';

Procedure GtkCListColumnTitlesActive(CList : PGtkCList);
  asmname 'gtk_clist_column_titles_active';

Procedure GtkClistColumnTitlesPassive(CList : PGtkCList);
  asmname 'gtk_clist_column_titles_passive';

{ Set the title in the column title button }
Procedure GtkClistSetColumnTitles(CList : PGtkCList; Col : gint; Title : PGChar);
  asmname 'gtk_clist_set_column_title';

{ Returns the title of column. Returns NULL if title is not set }
Function GtkClistGetColumnTitle(CList : PGtkCList; Col : gint) : PGChar;
  asmname 'gtk_clist_get_column_title';

{ Set a widget instead of a title for the column title button }
Procedure GtkClistSetColumnWidget(CList : PGtkCList; Col : gint; Widget : PGtkWidget);
  asmname 'gtk_clist_set_column_widget';

{ Returns the column widget }
Function GtkClistGetColumnWidget(CList : PGtkCList; Col : gint) : PGtkWidget;
  asmname 'gtk_clist_get_column_widget';

{ Set the justification on a column }
Procedure GtkClistSetColumnJustification(CList : PGtkCList; Col : gint; Justification : TGtkJustification);
  asmname 'gtk_clist_set_column_justification';

{ Set visibility of a column }
Procedure GtkClistSetColumnVisibility(CList : PGtkCList; Col : gint; Visible : gboolean);
  asmname 'gtk_clist_set_column_visibility';

{ Enable/disable column resize operations by mouse }
Procedure GtkClistSetColumnResizeable(CList : PGtkCList; Col : gint; Resizeable : gboolean);
  asmname 'gtk_clist_set_column_resizeable';

{ Resize column automatically to its optimal width }
Procedure GtkCListSetColumnAutoResize(CList : PGtkCList; Col : gint; AutoResize : gboolean);
  asmname 'gtk_clist_set_column_auto_resize';

Function GtkClistColumnsAutosize(CList : PGtkCList) : gint;
  asmname 'gtk_clist_columns_autosize';

{ Return the optimal column width, i.e. maximum of all cell widths }
Function GtkClistOptimalColumnWidth(CList : PGtkCList; Col : gint) : gint;
  asmname 'gtk_clist_optimal_column_width';

{ Set the pixel width of a column; this is a necessary step in
  creating a CList because otherwise the column width is chozen from
  the width of the column title, which will never be right }
Procedure GtkClistSetColumnWidth(CList : PGtkCList; Col, Width : gint);
  asmname 'gtk_clist_set_column_width';

 { set column minimum/maximum width. min/max_width < 0 => no restriction }  
Procedure GtkClistSetColumnMinWidth(CList : PGtkCList; Col, MinWidth : gint);
  asmname 'gtk_clist_set_column_min_width';
Procedure GtkCListSetColumnMaxWidth(CList : PGtkCList; Col, MaxWidth : gint);
  asmname 'gtk_clist_set_column_max_width';

{ Change the height of the rows, the default (height=0) is
  the hight of the current font. }
Procedure GtkClistSetRowHeight(CList : PGtkCList; Height : guint);
  asmname 'gtk_clist_set_row_height';

{ Scroll the viewing area of the list to the given column and row;
  row_align and col_align are between 0-1 representing the location the
  row should appear on the screnn, 0.0 being top or left, 1.0 being
  bottom or right; if row or column is -1 then then there is no change }
Procedure GkClistMoveto(CList : PGtkCList; Row, Col : gint; RowAlign, ColAlign : gfloat);
  asmname 'gtk_clist_moveto';

 { returns whether the row is visible }  
Function GtkClistRowIsVisible(CList : PGtkCList; Row : gint) : TGtkVisibility;
  asmname 'gtk_clist_row_is_visible';

 { returns the cell type }  
Function GtkClistGetCellType(CList : PGtkCList; Row, Col : gint) : TGtkCellType;
  asmname 'gtk_clist_get_cell_type';

 { sets a given cell's text, replacing it's current contents }  
Procedure GtkClistSetText(CList : PGtkCList; Row, Col : gint; TText : PGChar);
  asmname 'gtk_clist_set_text';

{ For the "get" functions, any of the return pointer can be
  NULL if you are not interested }

Function GtkClistGetText(CList : PGtkCList; Row, Col : gint; var TText : pgchar) : gint;
  asmname 'gtk_clist_get_text';

 { sets a given cell's pixmap, replacing it's current contents }  
Procedure GtkClistSetPixmap(CList : PGtkCList; Row, Col : gint; Pixmap : PGdkPixmap; Mask : PGdkBitmap);
  asmname 'gtk_clist_set_pixmap';

Function GtkClistGetPixmap(CList : PGtkCList; Row, Col : gint; var Pixmap : PGdkPixmap; var Mask : PGdkBitmap) : gint;
  asmname 'gtk_clist_get_pixmap';

{ Sets a given cell's pixmap and text, replacing it's current contents }
Procedure GtkClistSetPixtext(CList : PGtkCList; Row, Col : gint; TText : PGChar; Spacing : guint8; Pixmap : PGdkPixmap; Mask : PGdkPixmap);
  asmname 'gtk_clist_set_pixtext';

Function GtkClistGetPixtext(CList : PGtkCList; Row, Col : gint; var TText : pgchar; Spacing : pguint8; var Pixmap : PGdkPixmap; var Mask : PGdkBitmap) : gint;
  asmname 'gtk_clist_get_pixtext';

{ Sets the foreground color of a row, the color must already be allocated }
Procedure GtkClistSetForeground(CList : PGtkCList; Row : gint; Color : PGdkColor);
  asmname 'gtk_clist_set_foreground';

{ Sets the background color of a row, the color must already be allocated }
Procedure GtkClistSetBackground(CList : PGtkCList; Row : gint; Color : PGdkColor);
  asmname 'gtk_clist_set_background';

{ set / get cell styles }
Procedure GtkClistSetCellStyle(CList : PGtkCList; Row, Col : gint; Style : PGtkStyle);
  asmname 'gtk_clist_set_cell_style';

Function GtkClistGetCellStyle(CList : PGtkCList; Row, Col : gint) : PGtkStyle;
  asmname 'gtk_clist_get_cell_style';

Procedure GtkClistSetRowStyle(CList : PGtkCList; Row : gint; Style : PGtkStyle);
  asmname 'gtk_clist_set_row_style';

Function GtkClistGetRowStyle(CList : PGtkCList; Row : gint):PGtkStyle;
  asmname 'gtk_clist_get_row_style';

{ This sets a horizontal and vertical shift for drawing
  the contents of a cell; it can be positive or negitive;
  this is particulary useful for indenting items in a column }
Procedure GtkClistSetShift(CList : PGtkCList; Row, Col, Vertical, Horizontal : gint);
  asmname 'gtk_clist_set_shift';

{ set/get selectable flag of a single row }
Procedure GtkClistSetSelectable(CList : PGtkCList; Row : gint; Selectable : gboolean);
  asmname 'gtk_clist_set_selectable';

Function GtkClistGetSelectable(CList : PGtkCList; Row : gint) : gboolean;
  asmname 'gtk_clist_get_selectable';

{ prepend/append returns the index of the row you just added,
  making it easier to append and modify a row }
Function GtkClistPrepend(CList : PGtkCList; TText : ppgchar) : gint;
  asmname 'gtk_clist_prepend';
Function GtkClistAppend(CList : PGtkCList; TText : ppgchar) : gint;
  asmname 'gtk_clist_append';

{ Inserts a row at index row and returns the row where it was
  actually inserted (may be different from "row" in auto_sort mode) }
Function GtkClistInsert(CList : PGtkCList; Row : gint; TText : ppgchar) : gint;
  asmname 'gtk_clist_insert';

 { removes row at index row }  
Procedure GtkClistRemove(CList : PGtkCList; Row : gint);
  asmname 'gtk_clist_remove';

 { sets a arbitrary data pointer for a given row }  
Procedure GtkClistSetRowData(CList : PGtkCList; Row : gint; Data : gpointer);
  asmname 'gtk_clist_set_row_data';

{ Sets a data pointer for a given row with destroy notification }
Procedure GtkClistSetRowDataFull(CList : PGtkCList; Row : gint; Data : gpointer; Destroy : TGtkDestroyNotify);
  asmname 'gtk_clist_set_row_data_full';

{ Returns the data set for a row }
Function GtkClistGetRowData(CList : PGtkCList; Row : gint) : gpointer;
  asmname 'gtk_clist_get_row_data';

{ Giving a data pointer, find the first (and hopefully only!)
  row that points to that data, or -1 if none do }
Function GtkClistFindRowFromData(CList : PGtkCList; Data : gpointer) : gint;
  asmname 'gtk_clist_find_row_from_data';

 { force selection of a row }  
Procedure GtkClistSelectRow(CList : PGtkCList; Row, Col : gint);
  asmname 'gtk_clist_select_row';

 { force unselection of a row }  
Procedure GtkClistUnselectRow(CList : PGtkCList; Row, Col : gint);
  asmname 'gtk_clist_unselect_row';

{ Undo the last select/unselect operation }
Procedure GtkClistUndoSelection(CList : PGtkCList);
  asmname 'gtk_clist_undo_selection';

{ Clear the entire list -- this is much faster than removing
  each item with gtk_clist_remove }
Procedure GtkClistClear(CList : PGtkCList);
  asmname 'gtk_clist_clear';

{ Return the row column corresponding to the x and y coordinates,
  the returned values are only valid if the x and y coordinates
  are respectively to a window == clist->clist_window }
Function GtkClistGetSelectionInfo(CList : PGtkCList; x, y : gint; Row, Col : pgint) : gint;
  asmname 'gtk_clist_get_selection_info';

 { in multiple or extended mode, select all rows }  
Procedure GtkClistSelectAll(CList : PGtkCList);
  asmname 'gtk_clist_select_all';

 { in all modes except browse mode, deselect all rows }  
Procedure GtkClistUnselectAll(CList : PGtkCList);
  asmname 'gtk_clist_unselect_all';

 { swap the position of two rows }  
Procedure GtkClistSwapRows(CList : PGtkCList; Row1, Row2 : gint);
  asmname 'gtk_clist_swap_rows';

 { move row from source_row position to dest_row position }  
Procedure GtkClistRowMove(CList : PGtkCList; Source, Dest : gint);
  asmname 'gtk_clist_row_move';

 { sets a compare function different to the default }  
Procedure GtkClistSetCompareFunc(CList : PGtkCList; Func : TGtkCListCompareFunc);
  asmname 'gtk_clist_set_compare_func';

 { the column to sort by }  
Procedure GtkClistSetSortColumn(CList : PGtkCList; Col : gint);
  asmname 'gtk_clist_set_sort_column';

 { how to sort : ascending or descending }  
Procedure GkClistSetSortType(CList : PGtkCList; Sort : TGtkSortType);
  asmname 'gtk_clist_set_sort_type';

 { sort the list with the current compare function }  
Procedure GtkClistSort(CList : PGtkCList);
  asmname 'gtk_clist_sort';

 { Automatically sort upon insertion }  
Procedure GtkClistSetAutoSort(CList : PGtkCList; AutoSort : gboolean);
  asmname 'gtk_clist_set_auto_sort';

Implementation

function GTK_IS_CLIST (Obj : Pointer) : Boolean;
begin
  GTK_IS_CLIST := (Obj <> nil) and GTK_IS_CLIST_CLASS(PGtkTypeObject(Obj)^.Klass);
end;

function GTK_IS_CLIST_CLASS (Klass : Pointer) : Boolean;
begin
  GTK_IS_CLIST_CLASS := (Klass <> nil) and (GtkTypeIsA(PGtkTypeClass(Klass)^.TType, GTK_TYPE_CLIST) = G_TRUE);
end;

end.
