{
This unit is part of the Gtk+ units package.
Copyright (C) 2001  Nicola Girardi <nikke@swlibero.org>
                    Eike Lange <eike.lange@uni-essen.de>

This unit is free software; you can redistribute it and/or
modify it under the terms of the GNU unit General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This unit is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
unit General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this unit; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

Original GTK copyright and license information follows.
}

{
GTK - The GIMP Toolkit
Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
}

{
Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
file for a list of people on the GTK+ Team.  See the ChangeLog
files for a list of changes.  These files are distributed with
GTK+ at ftp://ftp.gtk.org/pub/gtk/.
}

unit GtkContainer;

interface

uses Glib, GtkAdjustment, GtkEnums, GtkObject, GtkTypeUtils, GtkWidget;

function GTK_TYPE_CONTAINER : TGtkType;
asmname 'gtk_container_get_type';

function GTK_IS_CONTAINER (Obj : Pointer) : Boolean;
function GTK_IS_CONTAINER_CLASS (Klass : Pointer) : Boolean;

type
  PGtkContainerClass = ^TGtkContainerClass;
  PGtkContainer      = ^TGtkContainer;

  TGtkContainerFrame1 = packed record
    BorderWidget : GUint16;
    NeedResize : GUint1;
    ResizeMode : GUint2;
    ReallocateRedraws : GUint1;
  end;

  TGtkContainer = record
    Widget     : TGtkWidget;
    FocusChild : PGtkWidget;

    Frame1 : TGtkContainerFrame1;

    { The list of children that requested a resize. }
    ResizeWidgets : PGSList;
  end;

  TGtkContainerClass = record
    ParentClass : TGtkWidgetClass;

    NChildArgs : GUint;

    Add, Remove : ^procedure (Container : PGtkContainer; Widget : PGtkWidget);
    CheckResize : ^procedure (Container : PGtkContainer);
    ForAll : ^procedure (Container : PGtkContainer; IncludeInternals : GBoolean; CallbackData : GPointer);
    Focus : ^function (Container : PGtkContainer; Direction : TGtkDirectionType) : GInt;
    SetFocusChild : ^procedure (Container : PGtkContainer; Widget : PGtkWidget);
    ChildType : ^function (Container : PGtkContainer) : TGtkType;
    SetChildArg, GetChildArg : ^procedure (Container : PGtkContainer; Child : PGtkWidget; Arg : PGtkArg; ArgID : GUint);
    CompositeName : ^function (Container : PGtkContainer; Child : PGtkWidget) : PGChar;

    { Padding for future expansion. }
    Pad1, Pad2 : TGtkFunction;
  end;

{ Application-level methods. }

function GtkContainerGetType : TGtkType;
asmname 'gtk_container_get_type';

procedure GtkContainerSetBorderWidth (Container : PGtkContainer; BorderWidth : GUint);
asmname 'gtk_container_set_border_width';

procedure GtkContainerAdd (Container : PGtkContainer; Widget : PGtkWidget);
asmname 'gtk_container_add';

procedure GtkContainerRemove (Container : PGtkContainer; Widget : PGtkWidget);
asmname 'gtk_container_remove';

procedure GtkContainerSetResizeMode (Container : PGtkContainer; ResizeMode : TGtkResizeMode);
asmname 'gtk_container_set_resize_mode';

procedure GtkContainerCheckResize (Container : PGtkContainer);
asmname 'gtk_container_check_resize';

procedure GtkContainerForeach (Container : PGtkContainer; Callback : TGtkCallback; CallbackData : GPointer);
asmname 'gtk_container_foreach';

procedure GtkContainerForeachFull (Container : PGtkContainer; Callback : TGtkCallback; Marshal : TGtkCallbackMarshal; CallbackData : GPointer; Notify : TGtkDestroyNotify);
asmname 'gtk_container_foreach_full';

function GtkContainerChildren (Container : PGtkContainer) : PGList;
asmname 'gtk_container_children';

function GtkContainerFocus (Container : PGtkContainer; Direction : TGtkDirectionType) : GInt;
asmname 'gtk_container_focus';

{ Widget-level methods. }

procedure GtkContainerSetReallocateRedraws (Container : PGtkContainer; NeedsRedraws : GBoolean);
asmname 'gtk_container_set_reallocate_redraws';

procedure GtkContainerSetFocusChild (Container : PGtkContainer; Child : PGtkWidget);
asmname 'gtk_container_set_focus_child';

procedure GtkContainerSetFocusVAdjustment (Container : PGtkContainer; Adjustment : PGtkAdjustment);
asmname 'gtk_container_set_focus_vadjustment';

procedure GtkContainerSetFocusHAdjustment (Container : PGtkContainer; Adjustment : PGtkAdjustment);
asmname 'gtk_container_set_focus_hadjustment';

procedure GtkContainerRegisterToplevel (Container : PGtkContainer);
asmname 'gtk_container_register_toplevel';

procedure GtkContainerUnregisterToplevel (Container : PGtkContainer);
asmname 'gtk_container_unregister_toplevel';

function GtkContainerGetToplevels : PGList;
asmname 'gtk_container_get_toplevels';

procedure GtkContainerResizeChildren (Container : PGtkContainer);
asmname 'gtk_container_resize_children';

function GtkContainerChildType (Container : PGtkContainer) : TGtkType;
asmname 'gtk_container_child_type';

{ The `arg_name' argument needs to be a const static string. }
procedure GtkContainerAddChildArgType (ArgName : PGChar; ArgType : TGtkType; ArgFlags, ArgID : GUint);
asmname 'gtk_container_add_child_arg_type';
     
{ Allocate a GtkArg array of size nargs that hold the
  names and types of the args that can be used with
  gtk_container_child_getv/gtk_container_child_setv.
  if (arg_flags!=NULL),
  (*arg_flags) will be set to point to a newly allocated
  guint array that holds the flags of the args.
  It is the callers response to do a
  g_free (returned_args); g_free (*arg_flags). }
function GtkContainerQueryChildArgs (ClassType : TGtkType; ArgFlags : PPGUint32; NArgs : PGUint) : PGtkArg;
asmname 'gtk_container_query_child_args';

{ gtk_container_child_getv() sets an arguments type and value, or just
  its type to GTK_TYPE_INVALID.
  if GTK_FUNDAMENTAL_TYPE (arg->type) == GTK_TYPE_STRING, it's the callers
  response to do a g_free (GTK_VALUE_STRING (arg)); }
procedure GtkContainerChildGetV (Container : PGtkContainer; Child : PGtkWidget; NArgs : GUint; Args : PGtkArg);
asmname 'gtk_container_child_getv';

procedure GtkContainerChildSetV (Container : PGtkContainer; Child : PGtkWidget; NArgs : GUint; Args : PGtkArg);
asmname 'gtk_container_child_setv';

{ gtk_container_add_with_args() takes a variable argument list of the form:
  (..., gchar *arg_name, ARG_VALUES, [repeatedly name/value pairs,] NULL)
  where ARG_VALUES type depend on the argument and can consist of
  more than one c-function argument. }
procedure GtkContainerAddWithArgs (Container : PGtkContainer; Widget : PGtkWidget; FirstArgName : PGChar; ...);
asmname 'gtk_container_add_with_args';

procedure GtkContainerAddV (Container : PGtkContainer; Widget : PGtkWidget; NArgs : GUint; Args : PGtkArg);
asmname 'gtk_container_addv';

procedure GtkContainerChildSet (Container : PGtkContainer; Widget : PGtkWidget; FirstArgName : PGChar; ...);
asmname 'gtk_container_child_set';
     
{ !! Non-public methods not implemented. }

implementation

function GTK_IS_CONTAINER (Obj : Pointer) : Boolean;
begin
  GTK_IS_CONTAINER := (Obj <> nil) and GTK_IS_CONTAINER_CLASS(PGtkTypeObject(Obj)^.Klass);
end;

function GTK_IS_CONTAINER_CLASS (Klass : Pointer) : Boolean;
begin
  GTK_IS_CONTAINER_CLASS := (Klass <> nil) and (GtkTypeIsA(PGtkTypeClass(Klass)^.TType, GTK_TYPE_CONTAINER) = G_TRUE);
end;

end.
