{
This unit is part of the Gtk+ units package.
Copyright (C) 2001  Nicola Girardi <nikke@swlibero.org>
                    Eike Lange <eike.lange@uni-essen.de>

This unit is free software; you can redistribute it and/or
modify it under the terms of the GNU unit General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This unit is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
unit General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this unit; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

Original GTK copyright and license information follows.
}

{
GTK - The GIMP Toolkit
Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
}

{
Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
file for a list of people on the GTK+ Team.  See the ChangeLog
files for a list of changes.  These files are distributed with
GTK+ at ftp://ftp.gtk.org/pub/gtk/.
}

Unit GtkCTree;

Interface

Uses
  Glib, GdkTypes, GtkClist, GtkEnums, GtkObject, GtkTypeUtils, GtkWidget;

Const

  { GtkCTreePos }
  GTK_CTREE_POS_BEFORE= 0;
  GTK_CTREE_POS_AS_CHILD=1;
  GTK_CTREE_POS_AFTER=2;

  { GtkCTreeLineStyle }
  GTK_CTREE_LINES_NONE=0;
  GTK_CTREE_LINES_SOLID=1;
  GTK_CTREE_LINES_DOTTED=2;
  GTK_CTREE_LINES_TABBED=3;

  { GtkCTreeExpanderStyle }
  GTK_CTREE_EXPANDER_NONE =0;
  GTK_CTREE_EXPANDER_SQUARE=1;
  GTK_CTREE_EXPANDER_TRIANGLE=2;
  GTK_CTREE_EXPANDER_CIRCULAR=3;
  
  { GtkCTreeExpansionType }
  GTK_CTREE_EXPANSION_EXPAND=0;
  GTK_CTREE_EXPANSION_EXPAND_RECURSIVE=1;
  GTK_CTREE_EXPANSION_COLLAPSE=2;
  GTK_CTREE_EXPANSION_COLLAPSE_RECURSIVE=3;
  GTK_CTREE_EXPANSION_TOGGLE=4;
  GTK_CTREE_EXPANSION_TOGGLE_RECURSIVE=5;


Type
  TGtkCTreePos = Integer;
  TGtkCTreeLineStyle = Integer;
  TGtkCTreeExpanderStyle = Integer;
  TGtkCTreeExpansionType = Integer;

  PGtkCTree = ^TGtkCTree;
  PGtkCTreeClass = ^TGtkCTreeClass;
  PGtkCTreeRow = ^TGtkCTreeRow;
  PGtkCTreeNode = ^TGtkCTreeNode;
  
  TGtkCTreeFunc = ^Procedure(ct:PGtkCTree; node:PGtkCTreeNode; data:gpointer);
  TGtkCTreeGNodeFunc = ^Function(ct:PGtkCTree; depth:guint; node:PGNode;
                            cnode:PGtkCTreeNode; data:gpointer):gboolean;
  TGtkCTreeCompareDragFunc = ^Function (ct:PGtkCTree; sourceNode,
                            newParent,newSibling:PGtkCTreeNode) : gboolean;


  TGtkCTree = Record
    clist : TGtkCList;
    linesGc : PGdkGC;
    treeIndent, treeSpacing, treeColumn : gint;
    lineStyle, expanderStyle : guint2;
    showStub : guint1;
    dragCompare : TGtkCTreeCompareDragFunc;
  End;

  TGtkCTreeClass = Record
    parentClass : TGtkCListClass;
    treeSelectRow : ^Procedure (ct:PGtkCTree; row:PGtkCTreeNode;col:gint);
    treeUnselectRow : ^Procedure (ct:PGtkCTree; row:PGtkCTreeNode; col;gint);
    treeExpand : ^Procedure (ct:PGtkCTree; node:PGtkCTreeNode);
    treeCollapse : ^Procedure (ct:PGtkCTree; node:PGtkCTreeNode);
    treeMove : ^Procedure (ct:PGtkCTree;
                              node,newParent,newSibling:PGtkCTreeNode);
    changeFocusRowExpansion : ^Procedure (ct:PgtkCTree;
                              action:TGtkCTreeExpansionType);
  End;
    
  TGtkCTreeRow = Record
    row : TGtkCListRow;
    parent, sibling, children : PGtkCTreeNode;
    pixmapClosed : PGdkPixmap;
    maskClosed : PGdkBitmap;
    pixmapOpened : PGdkPixmap;
    maskOpened : PGdkBitmap;
    level : guint16;
    isLeaf, expanded : guint1;
  End;

  TGtkCTreeNode = Record
    list : GList;
  End;


Function GTK_TYPE_CTREE : TGtkType;
  asmname 'gtk_ctree_get_type';

function GTK_IS_CTREE (Obj : Pointer) : Boolean;
function GTK_IS_CTREE_CLASS (Klass : Pointer) : Boolean;

function GTK_CTREE_ROW(Node : PGList) : PGtkCTreeRow;
function GTK_CTREE_NODE(Node : PGList) : PGtkCTreeNode;
function GTK_CTREE_NODE_NEXT(Node : PGList) : PGtkCTreeNode;
function GTK_CTREE_NODE_PREV(Node : PGList) : PGtkCTreeNode;
function GTK_CTREE_FUNC(Func : Pointer) : TGtkCTreeFunc;

 {**********************************************************
 *           Creation, insertion, deletion                 *
 **********************************************************} 

Function GtkCTreeGetType : TGtkType;
  asmname 'gtk_ctree_get_type';
Procedure GtkCTreeConstruct(ct:PGtkCTree; col,treeCol:gint; titles:ppgchar);
  asmname 'gtk_ctree_construct';
Function GtkCTreeNewWithTitles(col,treeCol:gint; titles:ppgchar)
                            : PGtkWidget;
  asmname 'gtk_ctree_new_with_titles';
Function GtkCTreeNew(col,treeCol:gint) : PGtkWidget;
  asmname 'gtk_ctree_new';
Function GtkCTreeInsertNode(ct:PGtkCTree; parent,sibl:PGtkCTreeNode; 
                            txt:ppgchar; spacing:guint8; 
                            pixClosed:PGdkPixmap; maskClosed:PGdkBitmap;
                            pixOpen:PGdkPixmap; maskOpen:PGdkBitmap;
                            isLeaf,expanded:gboolean):PGtkCTreeNode;
Procedure GtkCTreeRemoveNode(ct:PGtkCTree; node:PGtkCTreeNode);
  asmname 'gtk_ctree_remove_node';
Function GkCTreeInsertNode(ct:PGtkCTree; parent,sibl:PGtkCTreeNode; 
                            node:PGNode; f:TGtkCTreeGNodeFunc;
                            data:gpointer):PGtkCTreeNode;
  asmname 'gtk_ctree_insert_gnode';
Function GtkCTreeExportToGNode(ct:PGtkCTree; parent,sibl:PGNode; 
                            node:PGtkCTreeNode; f:TGtkCTreeGNodeFunc;
                            data:gpointer): PGNode;
  asmname 'gtk_ctree_export_to_gnode';

 {**********************************************************
 *  Generic recursive functions, querying / finding tree   *
 *  information                                            *
 **********************************************************} 
Procedure GtkCTreePostRecursive(ct:PGtkCTree; node:PGtkCTreeNode;
                            f:TGtkCTreeFunc; data:gpointer);
  asmname 'gtk_ctree_post_recursive';
Procedure GtkCTreePostRecursiveToDepth(ct:PGtkCTree; node:PGtkCTreeNode;
                            f:TGtkCTreeFunc; data:gpointer);
  asmname 'gtk_ctree_post_recursive_to_depth';
Procedure GtkCTreePreRecursive(ct:PGtkCTree; node:PGtkCTreeNode;
                            f:TGtkCTreeFunc; data:gpointer);
  asmname 'gtk_ctree_pre_recursive';
Procedure GtkCTreePreRecursiveToDepth(ct:PGtkCTree; node:PGtkCTreeNode;
                            f:TGtkCTreeFunc; data:gpointer);
  asmname 'gtk_ctree_pre_recursive_to_depth';
Function GtkCTreeIsViewable(ct:PGtkCTree; node:PGtkCTreeNode):gboolean;
  asmname 'gtk_ctree_is_viewable';
Function GtkCTreeLast(ct:PGtkCTree; node:PGtkCTreeNode):PGtkCTreeNode;
  asmname 'gtk_ctree_last';
Function GtkCTreeFindNotePtr(ct:PGtkCTree; node:PGtkCTreeRow):PGtkCTreeNode;
  asmname 'gtk_ctree_find_node_ptr';
Function GtkCTreeNodeNth(ct:PGtkCTree; row:guint) : PGtkCTreeNode;
  asmname 'gtk_ctree_node_nth';
Function GtkCTreeFind(ct:PGtkCTree; node,child:PGtkCTreeNode):gboolean;
  asmname 'gtk_ctree_find';
Function GtkCTreeIsAncestor(ct:PGtkCTree; node,child:PGtkCTreeNode):gboolean;
  asmname 'gtk_ctree_is_ancestor';
Function GtkCTreeFindByRowData(ct:PGtkCTree; node:PGtkCTreeNode;
                            data:gpointer) : PGtkCTreeNode;
  asmname 'gtk_ctree_find_by_row_data';

 { returns a GList of all GtkCTreeNodes with row->data == data. } 
Function GtkCTreeFindAllByRowData(ct:PGtkCTree; node:PGtkCTreeNode;
                            data:gpointer) : PGList;
  asmname 'gtk_ctree_find_all_by_row_data';
Function GtkCTreeFindByRowDataCustom(ct:PGtkCTree; node:PGtkCTreeNode;
                            data:gpointer; f:GCompareFunc):PGtkCTreeNode;
  asmname 'gtk_ctree_find_by_row_data_custom';

 { returns a GList of all GtkCTreeNodes with row->data == data. } 
Function GtkCTreeFindAllByRowDataCustom(ct:PGtkCTree; node:PGtkCTreeNode;
                            data:gpointer; f:GCompareFunc):PGList;
  asmname 'gtk_ctree_find_all_by_row_data_custom';
Function GtkCTreeIsHotSpot(ct:PGtkCTree; x,y:gint):gboolean;
  asmname 'gtk_ctree_is_hot_spot';

 {**********************************************************
 *   Tree signals : move, expand, collapse, (un)select     *
 **********************************************************} 
Procedure GtkCTreeMove(ct:PGtkCTree; node,parent,sibl:PGtkCTreeNode);
  asmname 'gtk_ctree_move';
Procedure GtkCTreeExpand(ct:PGtkCTree; node:PGtkCTreeNode);
  asmname 'gtk_ctree_expand';
Procedure GkCTreeExpandRecursive(ct:PGtkCTree; node:PGtkCTreeNode);
  asmname 'gtk_ctree_expand_recursive';
Procedure GtkCTreeExpandToDepth(ct:PGtkCTree; node:PGtkCTreeNode;
                            depth:gint);
  asmname 'gtk_ctree_expand_to_depth';
Procedure GtkCTreeCollapse(ct:PGtkCTree; node:PGtkCTreeNode);
  asmname 'gtk_ctree_collapse';
Procedure GtkCTreeCollapseRecursive(ct:PGtkCTree; node:PGtkCTreeNode);
  asmname 'gtk_ctree_collapse_recursive';
Procedure GtkCTreeCollapseToDepth(ct:PGtkCTree; node:PGtkCTreeNode;
                            depth:gint);
  asmname 'gtk_ctree_collapse_to_depth';
Procedure GtkCTreeToggleExpansion(ct:PGtkCTree; node:PGtkCTreeNode);
  asmname 'gtk_ctree_toggle_expansion';
Procedure GtkCTreeToggleExpansionRecursive(ct:PGtkCTree; node:PGtkCTreeNode);
  asmname 'gtk_ctree_toggle_expansion_recursive';
Procedure GtkCTreeSelect(ct:PGtkCTree; node:PGtkCTreeNode);
  asmname 'gtk_ctree_select';
Procedure GtkCTreeSelectRecursive(ct:PGtkCTree; node:PGtkCTreeNode);
  asmname 'gtk_ctree_select_recursive';
Procedure GtkCTreeUnselect(ct:PGtkCTree; node:PGtkCTreeNode);
  asmname 'gtk_ctree_unselect';
Procedure GtkCTreeUnselectRecursive(ct:PGtkCTree; node:PGtkCTreeNode);
  asmname 'gtk_ctree_unselect_recursive';
Procedure GtkCTreeRealSelectRecursive(ct:PGtkCTree; node:PGtkCTreeNode;
                            state:gint);
  asmname 'gtk_ctree_real_select_recursive';

 {**********************************************************
 *           Analogons of GtkCList functions               *
 **********************************************************} 
Procedure GtkCTreeNodeSetText(ct:PGtkCTree; node:PGtkCTreeNode;
                            col:gint; txt:PGChar);
  asmname 'gtk_ctree_node_set_text';
Procedure GtkCTreeNodeSetPixmap(ct:PGtkCTree; node:PGtkCTreeNode; col:gint;
                            pixmap:PGdkPixmap; mask:PGdkPixmap);
  asmname 'gtk_ctree_node_set_pixmap';
Procedure GtkCTreeNodeSetPixtext(ct:PGtkCTree; node:PGtkCTreeNode; col:gint;
                            txt:PGChar; spacing:guint8; 
                            pixmap:PGdkPixmap; mask:PGdkBitmap);
  asmname 'gtk_ctree_node_set_pixtext';
Procedure GtkCTreeSetNodeInfo(ct:PGtkCTree; node:PGtkCTreeNode; txt:PGChar;
                            spacing:guint8; pixClosed:PGdkPixmap;
                            maskClosed:PGdkBitmap; pixOpen:PGdkPixmap;
                            maskOpen:PGdkBitmap; isLeaf,expand:gboolean);
  asmname 'gtk_ctree_set_node_info';
Procedure GtkCTreeNodeSetShift(ct:PGtkCTree; node:PGtkCTreeNode;
                            col,vert,hori : gint);
  asmname 'gtk_ctree_node_set_shift';
Procedure GtkCTreeNodeSetSelectable(ct:PGtkCTree; node:PGtkCTreeNode;
                            selectable:gboolean);
  asmname 'gtk_ctree_node_set_selectable';
Function GtkCTreeNodeGetSelectable(ct:PGtkCTree; node:PGtkCTreeNode)
                            :gboolean;
  asmname 'gtk_ctree_node_get_selectable';
Function GtkCTreeNodeGetCellType(ct:PGtkCTree; node:PGtkCTreeNode;
                            col:gint) :TGtkCellType;
  asmname 'gtk_ctree_node_get_cell_type';
Function GkCTreeNodeGetText(ct:PGtkCTree; node:PGtkCTreeNode; col:gint;
                            txt:ppgchar):gint;
  asmname 'gtk_ctree_node_get_text';
Function GtkCTreeNodeGetPixmap(ct:PGtkCTree; node:PGtkCTreeNode; col:gint;
                            var pixmap:PGdkPixmap; var mask:PGdkBitmap):gint;
  asmname 'gtk_ctree_node_get_pixmap';
Function GtkCTreeNodeGetPixtext(ct:PGtkCTree; node:PGtkCTreeNode;col:gint;
                            var TText:pgchar; spacing:guint8;
                            var pixmap:PGdkPixmap; var mask:PGdkBitmap):gint;
  asmname 'gtk_ctree_node_get_pixtext';
Function GtkCTreeGetNodeInfo(ct:PGtkCTree; node:PGtkCTreeNode; var TText:pgchar;
                            spacing:pguint8; var pixClose:PGdkPixmap;
                            var maskClose:PGdkBitmap; var pixOpen:PGdkPixmap;
                            var maskOpen:PGdkBitmap; var isleaf,expanded:gboolean)
                            :gint;
  asmname 'gtk_ctree_get_node_info';
Procedure GtkCTreeNodeSetRowStyle(ct:PGtkCTree; node:PGtkCTreeNode;
                            style:PGtkStyle);
  asmname 'gtk_ctree_node_set_row_style';
Function GtkCTreeNodeGetRowStyle(ct:PGtkCTree; node:PGtkCTreeNode):PGtkStyle;
  asmname 'gtk_ctree_node_get_row_style';
Procedure GtkCTreeNodeSetCellType(ct:PGtkCTree; node:PGtkCTreeNode;
                            col:gint; style:PGtkStyle);
  asmname 'gtk_ctree_node_set_cell_type';
Function GtkCTreeNodeGetCellType(ct:PGtkCTree; node:PGtkCTreeNode;
                            col:gint) : TGtkCellType;
  asmname 'gtk_ctree_node_get_cell_type';
Procedure GTkCTreeNodeSetforeground(ct:PGtkCTree; node:PGtkCTreeNode;
                            color:PGdkColor);
  asmname 'gtk_ctree_node_set_foreground';
Procedure GtkCTreeNodeSetBackground(ct:PGtkCTree; node:PGtkCTreeNode;
                            color:PGdkColor);
  asmname 'gtk_ctree_node_set_background';
Procedure GtkCTreeNodeSetRowData(ct:PGtkCTree; node:PGtkCTreeNode;
                            data:gpointer);
  asmname 'gtk_ctree_node_set_row_data';
Procedure GtkCTreeNodeSetRowDataFull(ct:PGtkCTree; node:PGtkCTreeNode;
                            data:gpointer; destroy:TGtkDestroyNotify);
  asmname 'gtk_ctree_node_set_row_data_full';
Function GtkCTreeNodeGetRowData(ct:PGtkCTree; node:PGtkCTreeNode):gpointer;
  asmname 'gtk_ctree_node_get_row_data';
Procedure GtkCTreeNodeMoveto(ct:PGtkCTree; node:PGtkCTreeNode; col:gint;
                            rowAlign,colAlign:gfloat);
  asmname 'gtk_ctree_node_moveto';
Function GtkCTreeNodeIsVisible(ct:PGtkCTree; node:PGtkCTreeNode)
                            :TGtkVisibility;
  asmname 'gtk_ctree_node_is_visible';

 {**********************************************************
 *             GtkCTree specific functions                 *
 **********************************************************} 
Procedure GtkCTreeSetIndent(ct:PGtkCTree; i:gint);
  asmname 'gtk_ctree_set_indent';
Procedure GtkCTreeSetSpacing(ct:PGtkCTree;  s:gint);
  asmname 'gtk_ctree_set_spacing';
Procedure GtkCTreeSetShowStub(ct:PGtkCTree; show:gboolean);
  asmname 'gtk_ctree_set_show_stub';
Procedure GtkCTreeSetLineStyle(ct:PGtkCTree; lineStyle:TGtkCTreeLineStyle);
  asmname 'gtk_ctree_set_line_style';
Procedure GtkCTreeSetExpanderStyle(ct:PGtkCTree; ex:TGtkCTreeExpanderStyle);
  asmname 'gtk_ctree_set_expander_style';
Procedure GtkCTreeSetDragCompareFunc(ct:PGtkCTree; f:TGtkCTreeCompareDragFunc);
  asmname 'gtk_ctree_set_drag_compare_func';

 {**********************************************************
 *             Tree sorting functions                      *
 **********************************************************} 
Procedure GtkCTreeSortNode(ct:PGtkCTree; node:PGtkCTreeNode);
  asmname 'gtk_ctree_sort_node';
Procedure GtkCTreeSortRecursive(ct:PGtkCTree; nod:PGtkCTreeNode);
  asmname 'gtk_ctree_sort_recursive';


Implementation

function GTK_IS_CTREE (Obj : Pointer) : Boolean;
begin
  GTK_IS_CTREE := (Obj <> nil) and GTK_IS_CTREE_CLASS(PGtkTypeObject(Obj)^.Klass);
end;

function GTK_IS_CTREE_CLASS (Klass : Pointer) : Boolean;
begin
  GTK_IS_CTREE_CLASS := (Klass <> nil) and (GtkTypeIsA(PGtkTypeClass(Klass)^.TType, GTK_TYPE_CTREE) = G_TRUE);
end;

function GTK_CTREE_ROW(Node : PGList) : PGtkCTreeRow;
begin
  GTK_CTREE_ROW := Node^.Data;
end;

function GTK_CTREE_NODE(Node : PGList) : PGtkCTreeNode;
begin
  GTK_CTREE_NODE := PGtkCTreeNode(Node);
end;

function GTK_CTREE_NODE_NEXT(Node : PGList) : PGtkCTreeNode;
begin
  GTK_CTREE_NODE_NEXT := PGtkCTreeNode(Node^.Next);
end;

function GTK_CTREE_NODE_PREV(Node : PGList) : PGtkCTreeNode;
begin
  GTK_CTREE_NODE_PREV := PGtkCTreeNode(Node^.Prev);
end;

function GTK_CTREE_FUNC(Func : Pointer) : TGtkCTreeFunc;
begin
  GTK_CTREE_FUNC := Func;
end;

End.
