{
This unit is part of the Gtk+ units package.
Copyright (C) 2001  Nicola Girardi <nikke@swlibero.org>
                    Eike Lange <eike.lange@uni-essen.de>

This unit is free software; you can redistribute it and/or
modify it under the terms of the GNU unit General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This unit is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
unit General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this unit; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

Original GTK copyright and license information follows.
}

{
GTK - The GIMP Toolkit
Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
}

{
Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
file for a list of people on the GTK+ Team.  See the ChangeLog
files for a list of changes.  These files are distributed with
GTK+ at ftp://ftp.gtk.org/pub/gtk/.
}

unit GtkEditable;

interface

uses Glib, GdkTypes, GtkObject, GtkWidget;

function GTK_TYPE_EDITABLE : TGtkType;
asmname 'gtk_editable_get_type';

function GTK_IS_EDITABLE (Obj : GPointer) : Boolean;
function GTK_IS_EDITABLE_CLASS (Klass : GPointer) : Boolean;

type
  PGtkEditableClass = ^TGtkEditableClass;
  PGtkEditable      = ^TGtkEditable;

  TGtkTextFunction = ^procedure (Editable : PGtkEditable; Time : GUint32);

  TGtkEditable = record
    Widget : TGtkWidget;

    { Public }
    CurrentPos : GUint;
    SelectionStartPos, SelectionEndPos : GUint;
    HasSelection : GUint1;

    { Private }
    Editable, Visible : GUint1;

    IC : PGdkIC;
    ICAttr : PGdkICAttr;

    ClipboardText : PGChar;
  end;

  TGtkEditableClass = record
    ParentClass : TGtkWidgetClass;

    { Signals for notification/filtering of changes. }
    Changed : ^procedure (Editable : PGtkEditable);
    InsertText : ^procedure (Editable : PGtkEditable; TText : PGChar; Len : GInt; Position : PGInt);
    DeleteText : ^procedure (Editable : PGtkEditable; StartPos, EndPos : GInt);

    { Bindings actions. }
    Activate : ^procedure (Editable : PGtkEditable);
    SetEditable : ^procedure (Editable : PGtkEditable; IsEditable : GBoolean);
    MoveCursor : ^procedure (Editable : PGtkEditable; x, y : GInt);
    MoveWord : ^procedure (Editable : PGtkEditable; n : GInt);
    MovePage : ^procedure (Editable : PGtkEditable; x, y : GInt);
    MoveToRow : ^procedure (Editable : PGtkEditable; Row : GInt);
    MoveToColumn : ^procedure (Editable : PGtkEditable; Col : GInt);

    KillChar, KillWord, KillLine : ^procedure (Editable : PGtkEditable; Direction : GInt);

    CutClipboard, CopyClipboard, PasteClipboard : ^procedure (Editable : PGtkEditable);

    { Virtual functions. get_chars is in paricular not a signal because
      it returns malloced memory. The others are not signals because
      they would not be particularly useful as such. (All changes to
      selection and position do not go through these functions) }
    UpdateText : ^procedure (Editable : PGtkEditable; StartPos, EndPos : GInt);
    GetChars : ^function (Editable : PGtkEditable; StartPos, EndPos : GInt) : PGChar;
    SetSelection : ^procedure (Editable : PGtkEditable; StartPos, EndPos : GInt);
    SetPosition : ^procedure (Editable : PGtkEditable; Position : GInt);
  end;

function GtkEditableGetType : TGtkType;
asmname 'gtk_editable_get_type';

procedure GtkEditableSelectRegion (Editable : PGtkEditable; StartPos, EndPos : GInt);
asmname 'gtk_editable_select_region';

procedure GtkEditableInsertText (Editable : PGtkEditable; NewText : PGChar; NewTextLen : GInt; Position : PGInt);
asmname 'gtk_editable_insert_text';

procedure GtkEditableDeleteText (Editable : PGtkEditable; StartPos, EndPos : GInt);
asmname 'gtk_editable_delete_text';

function GtkEditableGetChars (Editable : PGtkEditable; StartPos, EndPos : GInt) : PGChar;
asmname 'gtk_editable_get_chars';

procedure GtkEditableCutClipboard (Editable : PGtkEditable);
asmname 'gtk_editable_cut_clipboard';

procedure GtkEditableCopyClipboard (Editable : PGtkEditable);
asmname 'gtk_editable_copy_clipboard';

procedure GtkEditablePasteClipboard (Editable : PGtkEditable);
asmname 'gtk_editable_paste_clipboard';

procedure GtkEditableClaimSelection (Editable : PGtkEditable; Claim : GBoolean; Time : GUint32);
asmname 'gtk_editable_claim_selection';

procedure GtkEditableDeleteSelection (Editable : PGtkEditable);
asmname 'gtk_editable_delete_selection';

procedure GtkEditableChanged (Editable : PGtkEditable);
asmname 'gtk_editable_changed';

procedure GtkEditableSetPosition (Editable : PGtkEditable; Position : GInt);
asmname 'gtk_editable_set_position';

function GtkEditableGetPosition (Editable : PGtkEditable) : GInt;
asmname 'gtk_editable_get_position';

procedure GtkEditableSetEditable (Editable : PGtkEditable; IsEditable : GBoolean);
asmname 'gtk_editable_set_editable';

implementation

function GTK_IS_EDITABLE (Obj : GPointer) : Boolean;
begin
  GTK_IS_EDITABLE := (Obj <> nil) and (GTK_IS_EDITABLE_CLASS(PGtkTypeObject(Obj)^.Klass));
end;

function GTK_IS_EDITABLE_CLASS (Klass : GPointer) : Boolean;
begin
  GTK_IS_EDITABLE_CLASS := (Klass <> nil) and (GtkTypeIsA(PGtkTypeClass(Klass)^.TType, GTK_TYPE_EDITABLE) = G_TRUE);
end;

end.
