{
This unit is part of the Gtk+ units package.
Copyright (C) 2001  Nicola Girardi <nikke@swlibero.org>
                    Eike Lange <eike.lange@uni-essen.de>

This unit is free software; you can redistribute it and/or
modify it under the terms of the GNU unit General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This unit is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
unit General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this unit; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

Original GTK copyright and license information follows.
}

{
GTK - The GIMP Toolkit
Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
}

{
Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
file for a list of people on the GTK+ Team.  See the ChangeLog
files for a list of changes.  These files are distributed with
GTK+ at ftp://ftp.gtk.org/pub/gtk/.
}

unit GtkEntry;

interface

uses Glib, GdkTypes, GtkEditable, GtkObject, GtkWidget;

function GTK_TYPE_ENTRY : TGtkType;
asmname 'gtk_entry_get_type';

function GTK_IS_ENTRY (Obj : GPointer) : Boolean;
function GTK_IS_ENTRY_CLASS (Klass : GPointer) : Boolean;

type
  PGtkEntryClass = ^TGtkEntryClass;
  PGtkEntry      = ^TGtkEntry;

  TGtkEntry = record
    Editable : TGtkEditable;

    TextArea : PGdkWindow;
    BackingPixmap : PGdkPixmap;
    Cursor : PGdkCursor;
    Text : PGdkWChar;

    TextSize, TextLength, TextMaxLen : GUint16;
    ScrollOffset : GInt;

    { Deprecated - See editable->visible }
    Visible : GUint1;

    Timer : GUint32;
    Button : GUint;

    { The x-offset of each character (including the last insertion position)
      only valid when the widget is realized. }
    CharOffset : PGChar;

    { Same as 'text', but in multibyte. }
    TextMB : PGChar;

    { If true, 'text' and 'text_mb' are not coherent. }
    TextMBDirty : GUint1;
    { If true, we use the encoding of wchar_t as the encoding of 'text'.
      Otherwise we use the encoding of multi-byte characters instead. }
    UseWChar : GUint1;
  end;

  TGtkEntryClass = record
    ParentClass : TGtkEditableClass;
  end;

function GtkEntryGetType : TGtkType;
asmname 'gtk_entry_get_type';

function GtkEntryNew : PGtkWidget;
asmname 'gtk_entry_new';

function GtkEntryNewWithMaxLength (Max : GUint16) : PGtkWidget;
asmname 'gtk_entry_new_with_max_length';

procedure GtkEntrySetText (Entry : PGtkEntry; Text : PGChar);
asmname 'gtk_entry_set_text';

procedure GtkEntryAppendText (Entry : PGtkEntry; Text : PGChar);
asmname 'gtk_entry_append_text';

procedure GtkEntryPrependText (Entry : PGtkEntry; Text : PGChar);
asmname 'gtk_entry_prepend_text';

procedure GtkEntrySetPosition (Entry : PGtkEntry; Position : GInt);
asmname 'gtk_entry_set_position';

{ Returns a reference to the text. }
function GtkEntryGetText (Entry : PGtkEntry) : PGChar;
asmname 'gtk_entry_get_text';

procedure GtkEntrySelectRegion (Entry : PGtkEntry; StartPos, EndPos : GInt);
asmname 'gtk_entry_select_region';

procedure GtkEntrySetVisibility (Entry : PGtkEntry; Visible : Gboolean);
asmname 'gtk_entry_set_visibility';

procedure GtkEntrySetEditable (Entry : PGtkEntry; Editable : GBoolean);
asmname 'gtk_entry_set_editable';

{ Text is truncated if needed. }
procedure GtkEntrySetMaxLength (Entry : PGtkEntry; Max : GUint16);
asmname 'gtk_entry_set_max_length';

implementation

function GTK_IS_ENTRY (Obj : GPointer) : Boolean;
begin
  GTK_IS_ENTRY := (Obj <> nil) and (GTK_IS_ENTRY_CLASS(PGtkTypeObject(Obj)^.Klass));
end;

function GTK_IS_ENTRY_CLASS (Klass : GPointer) : Boolean;
begin
  GTK_IS_ENTRY_CLASS := (Klass <> nil) and (GtkTypeIsA(PGtkTypeClass(Klass)^.TType, GTK_TYPE_ENTRY) = G_TRUE);
end;

end.
