{
This unit is part of the Gtk+ units package.
Copyright (C) 2001  Nicola Girardi <nikke@swlibero.org>
                    Eike Lange <eike.lange@uni-essen.de>

This unit is free software; you can redistribute it and/or
modify it under the terms of the GNU unit General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This unit is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
unit General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this unit; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

Original GTK copyright and license information follows.
}

{
GTK - The GIMP Toolkit
Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
}

{
Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
file for a list of people on the GTK+ Team.  See the ChangeLog
files for a list of changes.  These files are distributed with
GTK+ at ftp://ftp.gtk.org/pub/gtk/.
}

Unit GtkFontsel;

Interface

Uses
  Glib, GdkTypes, GtkEnums, GtkObject, GtkWindow, GtkNotebook, GtkWidget;

Const
{  This is the number of properties which we keep in the properties array,
   i.e. Weight, Slant, Set Width, Spacing, Charset & Foundry. } 
  GTK_NUM_FONT_PROPERTIES = 6;

{  This is the number of properties each style has i.e. Weight, Slant,
   Set Width, Spacing & Charset. Note that Foundry is not included,
   since it is the same for all styles of the same FontInfo. } 
  GTK_NUM_STYLE_PROPERTIES = 5;

{  Used to determine whether we are using point or pixel sizes. } 
  { GtkFontMetricType }
  GTK_FONT_METRIC_PIXELS = 0;
  GTK_FONT_METRIC_POINTS = 1;

{  Used for determining the type of a font style, and also for setting filters.
   These can be combined if a style has bitmaps and scalable fonts available.} 
  { GtkFontType }
  GTK_FONT_BITMAP               = 1;
  GTK_FONT_SCALABLE             = 1 shl 1;
  GTK_FONT_SCALABLE_BITMAP      = 1 shl 2;
  GTK_FONT_ALL                  = 16#07;

{  These are the two types of filter available - base and user. The base
   filter is set by the application and can't be changed by the user. } 
  GTK_NUM_FONT_FILTERS = 2;
  { GtkFontFilterType }
  GTK_FONT_FILTER_BASE = 0;
  GTK_FONT_FILTER_USER = 1;

Type
  TGtkFontMetricType = Integer;
  TGtkFontType = Integer;
  TGtkFontFilterType = Integer;
  PGtkFontSelection = ^TGtkFontSelection;
  PGtkFontSelectionClass = ^TGtkFontSelectionClass;
  PGtkFontSelectionDialog = ^TGtkFontSelectionDialog;
  PGtkFontSelectionDialogClass = ^TGtkFontSelectionDialogClass;
  PGtkFontFilter = ^TGtkFontFilter;

{  These hold the arrays of current filter settings for each property.
   If nfilters is 0 then all values of the property are OK. If not the
   filters array contains the indexes of the valid property values. } 
  TGtkFontFilter = Record
    FontType : gint;
    PropertyFilters  : Array[0..GTK_NUM_FONT_PROPERTIES-1] Of pguint16;
    PropertyNFilters : Array[0..GTK_NUM_FONT_PROPERTIES-1] Of guint16;
  End;

  TGtkFontSelection = Record
    Notebook : TGtkNotebook;
    
    {  These are on the font page. } 
    mainVbox, fontLabel, fontEntry, fontClist, fontStyleEntry,
      fontStyleClist, sizeEntry, sizeClist, pixelsButton, pointsButton, 
      filterButton, previewEntry, messageLabel : PGtkWidget;
    
    {  These are on the font info page. } 
    infoVbox, infoClist, requestedFontName, 
      actualFontName : PGtkWidget;
  
    {  These are on the filter page. } 
    filterVbox, typeBitmapsButton, typeScalableButton, 
      typeScaledBitmapsButton : PGtkWidget;
    FilterCLists : array[0..GTK_NUM_FONT_PROPERTIES-1] Of PGtkWidget;
    
    font : PGdkFont;
    fontIndex, style : gint;
    Metric : TGtkFontMetricType;
    {  The size is either in pixels or deci-points, depending on the metric. } 
    Size : gint;
  
    {  This is the last size explicitly selected. When the user selects different
     fonts we try to find the nearest size to this. } 
    SelectedSize : gint;
  
    {  These are the current property settings. They are indexes into the
     strings in the GtkFontSelInfo properties array. } 
    PropertyValues : array [0..GTK_NUM_STYLE_PROPERTIES] Of guint16;
  
    {  These are the base and user font filters. } 
    Filters : Array [0..GTK_NUM_FONT_FILTERS-1] Of TGtkFontFilter;
  End; 

  TGtkFontSelectionClass = Record
    ParentClass : TGtkNotebookClass;
  End;

  TGtkFontSelectionDialog = Record
    window : TGtkWindow;
    fontsel, mainVbox, actionArea,
    okbutton : PGtkWidget;
    {  The 'Apply' button is not shown by default but you can show/hide it. } 
    applyButton, cancelbutton : PGtkWidget;
    {  If the user changes the width of the dialog, we turn auto-shrink off. } 
    dialogWidth : gint;
    autoResize : gboolean;
  End;
  
  TGtkFontSelectionDialogClass = Record
    parentClass : TGtkWindowClass;
  End;
  
  
{ -- Functions and Procedures -- }

Function GTK_TYPE_FONT_SELECTION : TGtkType;
  asmname 'gtk_font_selection_get_type';

function GTK_IS_FONT_SELECTION (Obj : Pointer) : Boolean;
function GTK_IS_FONT_SELECTION_CLASS (Klass : Pointer) : Boolean;

Function GTK_TYPE_FONT_SELECTION_DIALOG : TGtkType;
  asmname 'gtk_font_selection_dialog_get_type';

function GTK_IS_FONT_SELECTION_DIALOG (Obj : Pointer) : Boolean;
function GTK_IS_FONT_SELECTION_DIALOG_CLASS (Klass : Pointer) : Boolean;

{ ****************************************************************************
 * GtkFontSelection functions.
 *   see the comments in the GtkFontSelectionDialog functions.
 ****************************************************************************} 

Function GtkFontSelectionGetType : TGtkType;
  asmname 'gtk_font_selection_get_type';
Function GtkFontSelectionNew : PGtkWidget;
  asmname 'gtk_font_selection_new';
Function GtkFontSelectionGetFontName(f:PGtkFontSelection):pgchar;
  asmname 'gtk_font_selection_get_font_name';
Function GtkFontSelectionGetFont(f:PGtkFontSelection):PGdkFont;
  asmname 'gtk_font_selection_get_font';
Function GtkFontSelectionSetFontName(f:PGtkFontSelection;
                            fontname:PGChar):gboolean;
  asmname 'gtk_font_selection_set_font_name';
Procedure GtkFontSelectionSetFilter(fsel:PGtkFontSelection;
                            filterType:TGtkFontFilterType;
                            ftype:TGtkFontType; fondries, weights, slants,
                            setwidths, spacings, charsets:ppgchar);
  asmname 'gtk_font_selection_set_filter';
Function GtkFontSelectionGetPreviewText(fsel:PGtkFontSelection):pgchar;
  asmname 'gtk_font_selection_get_preview_text';
Procedure GtkFontSelectionSetPreviewText(fsel:PGtkFontSelection;
                            text:PGChar);
  asmname 'gtk_font_selection_set_preview_text';

{ ****************************************************************************
 * GtkFontSelectionDialog functions.
 *   most of these functions simply call the corresponding function in the
 *   GtkFontSelection.
 ****************************************************************************} 

Function GtkFontSelectionDialogGetType : TGtkType;
  asmname 'gtk_font_selection_dialog_get_type';
Function GtkfontSelectionDialogNew(title:PGChar) : PGtkWidget;
  asmname 'gtk_font_selection_dialog_new';

{  This returns the X Logical Font Description fontname, or NULL if no font
   is selected. Note that there is a slight possibility that the font might not
   have been loaded OK. You should call gtk_font_selection_dialog_get_font()
   to see if it has been loaded OK.
   You should g_free() the returned font name after you're done with it. } 
Function GtkFontSelectionDialogGetFontName(fs:PGtkFontSelectionDialog)
                            :PGChar;
  asmname 'gtk_font_selection_dialog_get_font_name';

{  This will return the current GdkFont, or NULL if none is selected or there
   was a problem loading it. Remember to use gdk_font_ref/unref() if you want
   to use the font (in a style, for example). } 
Function GtkFontSelectionDialogGetFont (fsd:PGtkFontSelectionDialog)
                            :PGdkFont;
  asmname 'gtk_font_selection_dialog_get_font';

{  This sets the currently displayed font. It should be a valid X Logical
   Font Description font name (anything else will be ignored), e.g.
   "-adobe-courier-bold-o-normal--25-*-*-*-*-*-*-*" 
   It returns TRUE on success. } 
Function GtkFontSelectionDialogSetFontName(fs:PGtkFontSelectionDialog;
                            fname : PGChar) : gboolean;
  asmname 'gtk_font_selection_dialog_set_font_name';

{  This sets one of the font filters, to limit the fonts shown. The filter_type
   is GTK_FONT_FILTER_BASE or GTK_FONT_FILTER_USER. The font type is a
   combination of the bit flags GTK_FONT_BITMAP, GTK_FONT_SCALABLE and
   GTK_FONT_SCALABLE_BITMAP (or GTK_FONT_ALL for all font types).
   The foundries, weights etc. are arrays of strings containing property
   values, e.g. 'bold', 'demibold', and *MUST* finish with a NULL.
   Standard long names are also accepted, e.g. 'italic' instead of 'i'.

   e.g. to allow only fixed-width fonts ('char cell' or 'monospaced') to be
   selected use:

  gchar *spacings[] = [ "c", "m", NULL ];
  gtk_font_selection_dialog_set_filter (GTK_FONT_SELECTION_DIALOG (fontsel),
                                       GTK_FONT_FILTER_BASE, GTK_FONT_ALL,
                                       NULL, NULL, NULL, NULL, spacings, NULL);

  to allow only true scalable fonts to be selected use:

  gtk_font_selection_dialog_set_filter (GTK_FONT_SELECTION_DIALOG (fontsel),
                                       GTK_FONT_FILTER_BASE, GTK_FONT_SCALABLE,
                                       NULL, NULL, NULL, NULL, NULL, NULL);
} 
Procedure GtkFontSelectionDialogSetFilter(fsel:PGtkFontSelection;
                            filterType:TGtkFontFilterType;
                            ftype:TGtkFontType; foundries, weights, slants,
                            setwidths, spacings, charsets:ppgchar);
  asmname 'gtk_font_selection_dialog_set_filter';

{  This returns the text in the preview entry. You should copy the returned
   text if you need it. } 
Function GtkFontSelectionDialogGetPreviewText(fs:PGtkFontSelectiondialog)
                            : PGChar;
  asmname 'gtk_font_selection_dialog_get_preview_text';

{  This sets the text in the preview entry. It will be copied by the entry,
   so there's no need to g_strdup() it first. } 
Procedure GtkFontSelectionDialogSetPreviewText(fs:PGtkFontSelectiondialog;
                            t: PGChar);
  asmname 'gtk_font_selection_dialog_set_preview_text';


Implementation

function GTK_IS_FONT_SELECTION (Obj : Pointer) : Boolean;
begin
  GTK_IS_FONT_SELECTION := (Obj <> nil) and GTK_IS_FONT_SELECTION_CLASS(PGtkTypeObject(Obj)^.Klass);
end;

function GTK_IS_FONT_SELECTION_CLASS (Klass : Pointer) : Boolean;
begin
  GTK_IS_FONT_SELECTION_CLASS := (Klass <> nil) and (GtkTypeIsA(PGtkTypeClass(Klass)^.TType, GTK_TYPE_FONT_SELECTION) = G_TRUE);
end;

function GTK_IS_FONT_SELECTION_DIALOG (Obj : Pointer) : Boolean;
begin
  GTK_IS_FONT_SELECTION_DIALOG := (Obj <> nil) and GTK_IS_FONT_SELECTION_DIALOG_CLASS(PGtkTypeObject(Obj)^.Klass);
end;

function GTK_IS_FONT_SELECTION_DIALOG_CLASS (Klass : Pointer) : Boolean;
begin
  GTK_IS_FONT_SELECTION_DIALOG_CLASS := (Klass <> nil) and (GtkTypeIsA(PGtkTypeClass(Klass)^.TType, GTK_TYPE_FONT_SELECTION_DIALOG) = G_TRUE);
end;

End.
