{
This unit is part of the Gtk+ units package.
Copyright (C) 2001  Nicola Girardi <nikke@swlibero.org>
                    Eike Lange <eike.lange@uni-essen.de>

This unit is free software; you can redistribute it and/or
modify it under the terms of the GNU unit General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This unit is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
unit General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this unit; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

Original GTK copyright and license information follows.
}

{
GTK - The GIMP Toolkit
Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
}

{
Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
file for a list of people on the GTK+ Team.  See the ChangeLog
files for a list of changes.  These files are distributed with
GTK+ at ftp://ftp.gtk.org/pub/gtk/.
}

unit GtkMenu;

interface

uses Glib, GtkObject, GtkAccelGroup, GtkMenuShell, GtkWidget;

function GTK_TYPE_MENU : TGtkType;
asmname 'gtk_menu_get_type';

function GTK_IS_MENU (Obj : Pointer) : Boolean;
function GTK_IS_MENU_CLASS (Klass : Pointer) : Boolean;

type
  PGtkMenuClass = ^TGtkMenuClass;
  PGtkMenu      = ^TGtkMenu;

  TGtkMenuPositionFunc = ^procedure (Menu : PGtkMenu; x, y : PGInt; UserData : GPointer);
  TGtkMenuDetachFunc = ^procedure (AttachWidget : PGtkWidget; Menu : PGtkMenu);

  TGtkMenuClass = record
    ParentClass : TGtkMenuShellClass;
  end;

  TGtkMenuFrame1 = packed record
    TornOff : GUint1;
  end;

  TGtkMenu = record
    MenuShell : TGtkMenuShell;
    ParentMenuItem, OldActiveMenuItem : PGtkWidget;
    AccelGroup : PGtkAccelGroup;
    PositionFunc : TGtkMenuPositionFunc;
    PositionFuncData : GPointer;

    { Do _not_ touch these widgets directly. We hide the reference
      count from the toplevel to the menu, so it must be restored
      before operating on these widgets. }
    Topleve, TearoffWindow : PGtkWidget;

    Frame1 : TGtkMenuFrame1;
  end;

function GtkMenuGetType : TGtkType;
asmname 'gtk_menu_get_type';

function GtkMenuNew : PGtkWidget;
asmname 'gtk_menu_new';

{ Wrappers for the Menu Shell operations. }
procedure GtkMenuAppend (Menu : PGtkMenu; Child : PGtkWidget);
asmname 'gtk_menu_append';

procedure GtkMenuPrepend (Menu : PGtkMenu; Child : PGtkWidget);
asmname 'gtk_menu_prepend';

procedure GtkMenuInsert (Menu : PGtkMenu; Child : PGtkWidget; Position : GInt);
asmname 'gtk_menu_insert';

{ Display the menu onscreen. }
procedure GtkMenuPopup (Menu : PGtkMenu; ParentMenuShell, ParentMenuItem : PGtkWidget; Func : TGtkMenuPositionFunc; Data : GPointer; Button : GUint; ActivateTime : GUint32);
asmname 'gtk_menu_popup';

{ Position the menu according to its position function. Called
  from gtkmenuitem.c when a menu-item changes its allocation. }
procedure GtkMenuReposition (Menu : PGtkMenu);
asmname 'gtk_menu_reposition';

procedure GtkMenuPopdown (Menu : PGtkMenu);
asmname 'gtk_menu_popdown';

{ Keep track of the last menu item selected. (For the purposes
  of the option menu. }
function GtkMenuGetActive (Menu : PGtkMenu) : PGtkWidget;
asmname 'gtk_menu_get_active';

procedure GtkMenuSetActive (Menu : PGtkMenu; Idx : GUint);
asmname 'gtk_menu_set_active';


{ Set/get the acclerator group that holds global accelerators (should
  be added to the corresponding toplevel with gtk_window_add_accel_group(). }
procedure GtkMenuSetAccelGroup (Menu : PGtkMenu; AccelGroup : PGtkAccelGroup);
asmname 'gtk_menu_set_accel_group';

function GtkMenuGetAccelGroup (Menu : PGtkMenu) : PGtkAccelGroup;
asmname 'gtk_menu_get_accel_group';

{ Get the accelerator group that is used internally by the menu for
  underline accelerators while the menu is popped up. }
function GtkMenuGetULineAccelGroup (Menu : PGtkMenu) : PGtkAccelGroup;
asmname 'gtk_menu_get_uline_accel_group';

function GtkMenuEnsureULineAccelGroup (Menu : PGtkMenu) : PGtkAccelGroup;
asmname 'gtk_menu_ensure_uline_accel_group';

{ A reference count is kept for a widget when it is attached to
  a particular widget. This is typically a menu item; it may also
  be a widget with a popup menu - for instance, the Notebook widget. }
procedure GtkMenuAttachToWidget (Menu : PGtkMenu; AttachWidget : PGtkWidget; Detacher : TGtkMenuDetachFunc);
asmname 'gtk_menu_attach_to_widget';

procedure GtkMenuDetach (Menu : PGtkMenu);
asmname 'gtk_menu_detach';

{ This should be dumped in favor of data set when the menu is popped
  up - that is currently in the ItemFactory code, but should be
  in the Menu code. }
function GtkMenuGetAttachWidget (Menu : PGtkMenu) : PGtkWidget;
asmname 'gtk_menu_get_attach_widget';

procedure GtkMenuSetTearoffState (Menu : PGtkMenu; TornOff : GBoolean);
asmname 'gtk_menu_set_tearoff_state';

{ This sets the window manager title for the window that
  appears when a menu is torn off. }
procedure GtkMenuSetTitle (Menu : PGtkMenu; Title : PGChar);
asmname 'gtk_menu_set_title';

procedure GtkMenuReorderChild (Menu : PGtkMenu; Child : PGtkWidget; Position : GInt);
asmname 'gtk_menu_reorder_child';

implementation

function GTK_IS_MENU (Obj : Pointer) : Boolean;
begin
  GTK_IS_MENU := (Obj <> nil) and GTK_IS_MENU_CLASS(PGtkTypeObject(Obj)^.Klass);
end;

function GTK_IS_MENU_CLASS (Klass : Pointer) : Boolean;
begin
  GTK_IS_MENU_CLASS := (Klass <> nil) and (GtkTypeIsA(PGtkTypeClass(Klass)^.TType, GTK_TYPE_MENU) = G_TRUE);
end;

end.
