{
This unit is part of the Gtk+ units package.
Copyright (C) 2001  Nicola Girardi <nikke@swlibero.org>
                    Eike Lange <eike.lange@uni-essen.de>

This unit is free software; you can redistribute it and/or
modify it under the terms of the GNU unit General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This unit is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
unit General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this unit; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

Original GTK copyright and license information follows.
}

{
GTK - The GIMP Toolkit
Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
}

{
Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
file for a list of people on the GTK+ Team.  See the ChangeLog
files for a list of changes.  These files are distributed with
GTK+ at ftp://ftp.gtk.org/pub/gtk/.
}

unit GtkObject;

interface

uses Glib, GtkEnums;

{ !! Please access the members directly.
  GTK_OBJECT_TYPE(obj)              (GTK_OBJECT (obj)->klass->type)
  GTK_OBJECT_SIGNALS(obj)           (GTK_OBJECT (obj)->klass->signals)
  GTK_OBJECT_NSIGNALS(obj)          (GTK_OBJECT (obj)->klass->nsignals)
}

function GTK_IS_OBJECT (Obj : GPointer) : Boolean;
function GTK_IS_OBJECT_CLASS (Klass : GPointer) : Boolean;

{ GtkObject only uses the first 4 bits of the flags field.
  Derived objects may use the remaining bits. Though this
  is a kinda nasty break up, it does make the size of
  derived objects smaller. }

const
  GTK_DESTROYED   = 1 shl 0;
  GTK_FLOATING    = 1 shl 1;
  GTK_CONNECTED   = 1 shl 2;
  GTK_CONSTRUCTED = 1 shl 3;
type
  PGtkObjectFlags = ^TGtkObjectFlags;
  TGtkObjectFlags = Integer;

const
  GTK_ARG_READABLE       = 1 shl 0;
  GTK_ARG_WRITABLE       = 1 shl 1;
  GTK_ARG_CONSTRUCT      = 1 shl 2;
  GTK_ARG_CONSTRUCT_ONLY = 1 shl 3;
  GTK_ARG_CHILD_ARG      = 1 shl 4;
  GTK_ARG_MASK           = 16#1f;
  { Aliases. }
  GTK_ARG_READWRITE      = GTK_ARG_READABLE or GTK_ARG_WRITABLE;
type
  PGtkArgFlags = ^TGtkArgFlags;
  TGtkArgFlags = Integer;

{ The GtkObject structure is the base of the Gtk+ objects hierarchy,
  it ``inherits'' from the GtkTypeObject by mirroring its fields,
  which must always be kept in sync completely. The GtkObject defines
  the few basic items that all derived classes contain. }

{ The GtkObjectClass is the base of the Gtk+ objects classes hierarchy,
  it ``inherits'' from the GtkTypeClass by mirroring its fields, which
  must always be kept in sync completely. The GtkObjectClass defines
  the basic necessities for the object inheritance mechanism to work.
  Namely, the `signals' and `nsignals' fields as well as the function
  pointers, required to end an object's lifetime. }

type
  PGtkObjectClass = ^TGtkObjectClass;
  PGtkObject      = ^TGtkObject;
  PGtkArg         = ^TGtkArg;

  { !! Imported from gtktypeutils.pas. }
  TGtkSignalFunc       = Pointer; { !! Notice: not `procedure'. }
  TGtkFunction         = Pointer; { @@ ^function (Data : GPointer) : GInt; }
  TGtkDestroyNotify    = ^procedure (Data : GPointer);
  TGtkCallbackMarshal  = ^procedure (Obj : PGtkObject; Data : GPointer; NArgs : GUint; Args : PGtkArg);
  TGtkSignalMarshaller = ^procedure (Obj : PGtkObject; Func : TGtkSignalFunc; FuncData : GPointer; Args : PGtkArg);

  { !! Imported from gtktypeutils.pas. }
  PGtkType = ^TGtkType; { A GtkType holds a unique type ID. }
  TGtkType = Integer;

  PSignalRecord = ^TSignalRecord;
  TSignalRecord = record
    F : TGtkSignalFunc;
    D : GPointer;
  end;

  PArgsRecord = ^TArgsRecord;
  TArgsRecord = record
    NArgs : GInt;
    Args : PGtkArg;
  end;

  PCallbackDataRecord = ^TCallbackDataRecord;
  TCallbackDataRecord = record
    Marshal : TGtkCallbackMarshal;
    Data : GPointer;
    Notify : TGtkDestroyNotify;
  end;

  PCCallbackDataRecord = ^TCallbackDataRecord;
  TCCallbackDataRecord = record
    Func : TGtkfunction;
    FuncData : GPointer;
  end;

  PForeignDataRecord = ^TForeignDataRecord;
  TForeignDataRecord = record
    Data : GPointer;
    Notify : TGtkDestroyNotify;
  end;

  TGtkArg = record
    TType : TGtkType;
    Name : PGChar;
    D : record
      case LongInt of
        0 : ( CharData : GChar );
        1 : ( UCharData : GUChar );
        2 : ( BoolData : GBoolean );
        3 : ( IntData : GInt );
        4 : ( UIntData : GUint );
        5 : ( LongData : GLong );
        6 : ( ULongData : GULong );
        7 : ( FloatData : GFloat );
        8 : ( DoubleData : GDouble );
        9 : ( StringData : PGChar );
        10 : ( PointerData : GPointer );
        11 : ( ObjectData : PGtkObject );
        12 : ( SignalData : TSignalRecord );
        13 : ( ArgsData : TArgsRecord );
        14 : ( CallbackData : TCallbackDataRecord );
        15 : ( CCallbackData : TCCallbackDataRecord );
        16 : ( ForeignData : TForeignDataRecord );
      end;
  end;

  { Deprecated. }
  TGtkArgGetFunc = ^procedure(OObject : PGtkObject; Arg : PGtkArg; N : GUint);
  TGtkArgSetFunc = ^procedure(OObject : PGtkObject; Arg : PGtkArg; N : GUint);

  { A GtkTypeObject defines the minimum structure requirements
    for type instances. Type instances returned from gtk_type_new ()
    and initialized through a GtkObjectInitFunc need to directly inherit
    from this structure or at least copy its fields one by one. }
  PGtkTypeClass  = ^TGtkTypeClass;
  PGtkTypeObject = ^TGtkTypeObject;

  TGtkTypeObject = record
    { A pointer to the objects class. This will actually point to
      the derived objects class struct (which will be derived from
      GtkTypeClass). }
    Klass : PGtkTypeClass;
  end;

  { A GtkTypeClass defines the minimum structure requirements for
    a types class. Classes returned from gtk_type_class () and
    initialized through a GtkClassInitFunc need to directly inherit
    from this structure or at least copy its fields one by one. }
  TGtkTypeClass = record
    { The type identifier for the objects class. There is
      one unique identifier per class. }
    TType : TGtkType;
  end;

  { !! End of foreign code. }

  TGtkObject = record
    { GtkTypeObject related fields. }
    Klass : PGtkObjectClass;

    { 32 bits of flags. GtkObject only uses 4 of these bits and
      GtkWidget uses the rest. This is done because structs are
      aligned on 4 or 8 byte boundaries. If a new bitfield were
      used in GtkWidget much space would be wasted. }
    Flags : GUint32;

    { Reference count. }
    RefCount : GUint;

    { A list of keyed data pointers, used for e.g. the list of
      signal handlers or an object's user_data. }
    ObjectData : PGData;
  end;

  TGtkObjectClass = record
    { GtkTypeClass fields. }
    TType : TGtkType;

    { The signals this object class handles. "signals"
      is an array of signal ID's. }
    Signals : PGUint;
  
    { The number of signals listed in "signals". }
    NSignals : GUint;
  
    { The number of arguments per class. }
    NArgs : GUint;
    ConstructArgs : PGSList;
  
    { Non overridable class methods to set and get per class arguments. }
    SetArg, GetArg : ^procedure (Obj : PGtkObject; Arg : PGtkArg; ArgID : GUint);

    { The functions that will end an objects life time. In one way ore
      another all three of them are defined for all objects. If an
      object class overrides one of the methods in order to perform class
      specific destruction then it must still invoke its superclass'
      implementation of the method after it is finished with its
      own cleanup. (See the destroy function for GtkWidget for
      an example of how to do this). }
    Shutdown, Destroy, Finalize : ^procedure (Obj: PGtkObject);
  end;

{ Macros for extracting bits from the object_flags field. }
function GTK_OBJECT_DESTROYED (Obj : PGtkObject) : Boolean;
function GTK_OBJECT_FLOATING (Obj : PGtkObject) : Boolean;
function GTK_OBJECT_CONNECTED (Obj : PGtkObject) : Boolean;
function GTK_OBJECT_CONSTRUCTED (Obj : PGtkObject) : Boolean;

{ Macros for setting and clearing bits in the object_flags field of GtkObject. }
procedure GTK_OBJECT_SET_FLAGS(Obj : Pointer; Flag : LongInt);
procedure GTK_OBJECT_UNSET_FLAGS(Obj : Pointer; Flag : LongInt);

function GtkObjectGetType : TGtkType;
asmname 'gtk_object_get_type';

{ Append a user defined signal without default handler to a class. }
function GtkObjectClassUserSignalNew (Klass : PGtkObjectClass; Name : PGChar; SignalFlags : TGtkSignalRunType; Marshaller : TGtkSignalMarshaller; ReturnVal : TGtkType; NParams : GUint; ...) : GUint;
asmname 'gtk_object_class_user_signal_new';

function GtkObjectClassUserSignalNewV (Klass : PGtkObjectClass; Name : PGChar; SignalFlags : TGtkSignalRunType; Marshaller : TGtkSignalMarshaller; ReturnVal : TGtkType; NParams : GUint; Params : PGtkType) : GUint;
asmname 'gtk_object_class_user_signal_newv';

function GtkObjectNew (TType : TGtkType; FirstArgName : PGChar; ...) : PGtkObject;
asmname 'gtk_object_new';

function GtkObjectNewV (TType : TGtkType; NArgs : GUint; Args : PGtkArg) : PGtkObject;
asmname 'gtk_object_newv';

procedure GtkObjectDefaultConstruct (Obj : PGtkObject);
asmname 'gtk_object_default_construct';

procedure GtkObjectConstructed (Obj : PGtkObject);
asmname 'gtk_object_constructed';

procedure GtkObjectSink (Obj : PGtkObject);
asmname 'gtk_object_sink';

procedure GtkObjectRef (Obj : PGtkObject);
asmname 'gtk_object_ref';

procedure GtkObjectUnref (Obj : PGtkObject);
asmname 'gtk_object_unref';

procedure GtkObjectWeakRef (Obj : PGtkObject; Notify : TGtkDestroyNotify; Data : GPointer);
asmname 'gtk_object_weakref';

procedure GtkObjectWeakUnref (Obj : PGtkObject; Notify : TGtkDestroyNotify; Data : GPointer);
asmname 'gtk_object_weakunref';

procedure GtkObjectDestroy (Obj : PGtkObject);
asmname 'gtk_object_destroy';

{ gtk_object_getv() sets an arguments type and value, or just
  its type to GTK_TYPE_INVALID.
  if GTK_FUNDAMENTAL_TYPE (arg->type) == GTK_TYPE_STRING, it's
  the callers response to do a g_free (GTK_VALUE_STRING (arg)); }
procedure GtkObjectGetV(Obj : PGtkObject; NArgs : GUint; Args : PGtkArg);
asmname 'gtk_object_getv';

{ gtk_object_get() sets the variable values pointed to by the adresses
  passed after the argument names according to the arguments value.
  if GTK_FUNDAMENTAL_TYPE (arg->type) == GTK_TYPE_STRING, it's
  the callers response to do a g_free (retrived_value); }
procedure GtkObjectGet (Obj : PGtkObject; FirstArgName : PGChar; ...);
asmname 'gtk_object_get';

{ gtk_object_set() takes a variable argument list of the form:
  (..., gchar *arg_name, ARG_VALUES, [repeatedly name/value pairs,] NULL)
  where ARG_VALUES type depend on the argument and can consist of
  more than one c-function argument. }
procedure GtkObjectSet (Obj : PGtkObject; FirstArgName : PGChar; ...);
asmname 'gtk_object_set';
procedure GtkObjectSetV (Obj : PGtkObject; NArgs : GUint; Args : PGtkArg);
asmname 'gtk_object_setv';

{ Allocate a GtkArg array of size nargs that hold the
  names and types of the args that can be used with
  gtk_object_set/gtk_object_get. if (arg_flags!=NULL),
  ( *arg_flags) will be set to point to a newly allocated
  guint array that holds the flags of the args.
  It is the callers response to do a
  g_free (returned_args); g_free ( *arg_flags). }
function GtkObjectQueryArgs (ClassType : TGtkType; var ArgFlags : PGUint32; var NArgs : GUint) : PGtkArg;
asmname 'gtk_object_query_args';

{ Set 'data' to the "object_data" field of the object. The data is
  indexed by the "key". If there is already data associated with
  "key" then the new data will replace it. If 'data' is NULL then
  this call is equivalent to 'gtk_object_remove_data'. The
  gtk_object_set_data_full variant acts just the same, but takes an
  additional argument which is a function to be called when the data
  is removed. `gtk_object_remove_data' is equivalent to the above,
  where 'data' is NULL. `gtk_object_get_data' gets the data associated
  with "key". }

procedure GtkObjectSetData (Obj : PGtkObject; Key : PGChar; Data : GPointer);
asmname 'gtk_object_set_data';

procedure GtkObjectSetDataFull (Obj : PGtkObject; Key : PGChar; Data : GPointer; Notify : TGtkDestroyNotify);
asmname 'gtk_object_set_data_full';

function  GtkObjectGetData (Obj : PGtkObject; Key : PGChar) : GPointer;
asmname 'gtk_object_get_data';

procedure GtkObjectRemoveData (Obj : PGtkObject; Key : PGChar);
asmname 'gtk_object_remove_data';

procedure GtkObjectRemoveNoNotify (Obj : PGtkObject; Key : PGChar);
asmname 'gtk_object_remove_no_notify';

{ Set/get the "user_data" object data field of "object". It should
  be noted that these functions are no different than calling
  `gtk_object_set_data'/`gtk_object_get_data' with a key of "user_data".
  They are merely provided as a convenience. }
procedure GtkObjectSetUserData (Obj : PGtkObject; Data : GPointer);
asmname 'gtk_object_set_user_data';

function GtkObjectGetUserData (Obj : PGtkObject) : GPointer;
asmname 'gtk_object_get_user_data';

{ Object-level methods }

{ Append "signals" to those already defined in "class". }
procedure GtkObjectClassAddSignals (Klass : PGtkObjectClass; Signals : PGUint; NSignals : GUint);
asmname 'gtk_object_class_add_signals';

procedure GtkObjectAddArgType (ArgName : PGChar; ArgType : TGtkType; ArgFlags, ArgID : GUint);
asmname 'gtk_object_add_arg_type';

{ Object data method variants that operate on key ids. }
procedure GtkObjectSetDataByID (Obj : PGtkObject; DataID : GQuark; Data : GPointer);
asmname 'gtk_object_set_data_by_id';

procedure GtkObjectSetDataByIDFull (Obj : PGtkObject; DataID : GQuark; Data : GPointer; DestroyNotify : TGtkDestroyNotify);
asmname 'gtk_object_set_data_by_id_full';

function GtkObjectGetDataByID (Obj : PGtkObject; DataID : GQuark) : GPointer;
asmname 'gtk_object_get_data_by_id';

procedure GtkObjectRemoveDataByID (Obj : PGtkObject; DataID : GQuark);
asmname 'gtk_object_remove_data_by_id';

procedure GtkObjectRemoveNoNotifyByID (Obj : PGtkObject; DataID : GQuark);
asmname 'gtk_object_remove_no_notify_by_id';

{ !! Ignored.
 #define gtk_object_data_try_key     g_quark_try_string
 #define gtk_object_data_force_id    g_quark_from_string
}

{ !! Non-public methods not implemented. }

{ !! From gtktypeutils.pas. }
function GtkTypeIsA (OType, IsAType : TGtkType) : GBoolean;
asmname 'gtk_type_is_a';

{ !! COPIED (and not cut) from gtktypeutils.pas }
const { Base type node of the object system }
  GTK_TYPE_OBJECT = 21;

implementation

function GTK_IS_OBJECT (Obj : GPointer) : Boolean;
begin
  GTK_IS_OBJECT := (Obj <> nil) and (GTK_IS_OBJECT_CLASS(PGtkTypeObject(Obj)^.Klass));
end;

function GTK_IS_OBJECT_CLASS (Klass : GPointer) : Boolean;
begin
  GTK_IS_OBJECT_CLASS := (Klass <> nil) and (GtkTypeIsA(PGtkTypeClass(Klass)^.TType, GTK_TYPE_OBJECT) = G_TRUE);
end;

function GTK_OBJECT_DESTROYED(Obj : PGtkObject) = B : Boolean;
begin
  B := ((Obj^.Flags and GTK_DESTROYED) <> 0);
end;

function GTK_OBJECT_FLOATING(Obj : PGtkObject) = B : Boolean;
begin
  B := ((Obj^.Flags and GTK_FLOATING) <> 0);
end;

function GTK_OBJECT_CONNECTED(Obj : PGtkObject) = B : Boolean;
begin
  B := ((Obj^.Flags and GTK_CONNECTED) <> 0);
end;

function GTK_OBJECT_CONSTRUCTED(Obj : PGtkObject) = B : Boolean;
begin
  B := ((Obj^.Flags and GTK_CONSTRUCTED) <> 0);
end;

procedure GTK_OBJECT_SET_FLAGS(Obj : Pointer; Flag : LongInt);
begin
  PGtkObject(Obj)^.Flags := PGtkObject(Obj)^.Flags or Flag;
end;

procedure GTK_OBJECT_UNSET_FLAGS(Obj : Pointer; Flag : LongInt);
begin
  PGtkObject(Obj)^.Flags := PGtkObject(Obj)^.Flags and (not Flag);
end;

end.
