{
This unit is part of the Gtk+ units package.
Copyright (C) 2001  Nicola Girardi <nikke@swlibero.org>
                    Eike Lange <eike.lange@uni-essen.de>

This unit is free software; you can redistribute it and/or
modify it under the terms of the GNU unit General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This unit is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
unit General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this unit; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

Original GTK copyright and license information follows.
}

{
GTK - The GIMP Toolkit
Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
}

{
Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
file for a list of people on the GTK+ Team.  See the ChangeLog
files for a list of changes.  These files are distributed with
GTK+ at ftp://ftp.gtk.org/pub/gtk/.
}

unit GtkRange;

interface

uses Glib, GdkTypes, GtkEnums, GtkObject, GtkAdjustment, GtkWidget;

function GTK_TYPE_RANGE : TGtkType;
asmname 'gtk_range_get_type';

function GTK_IS_RANGE (Obj : Pointer) : Boolean;
function GTK_IS_RANGE_CLASS (Klass : Pointer) : Boolean;

type
  PGtkRangeClass = ^TGtkRangeClass;
  PGtkRange      = ^TGtkRange;

  TGtkRangeClass = record
    ParentClass : TGtkWidgetClass;
    SliderWidth, StepperSize, StepperSliderSpacing, MinSliderSize : GInt;
    Trough, Slider, StepForw, StepBack : GUint8;

    DrawBackground, ClearBackground, DrawTrough, DrawSlider, DrawStepForw, DrawStepBack, SliderUpdate : ^procedure (Range : PGtkRange);
    TroughClick : ^function (Range : PGtkRange; x, y : GInt; JumpPerc : GFloat) : GInt;
    TroughKeys : ^function (Range : PGtkRange; Key : PGdkEventKey; Scroll : PGtkScrollType; Trough : PGtkTroughType) : GInt;
    Motion : ^procedure (Range : PGtkRange; xDelta, yDelta : GInt);
    Timer : ^function (Range : PGtkRange) : GInt;
  end;

  TGtkRangeFrame1 = packed record
    Policy : GUint2;
    ScrollType, InChild, ClickChild : GUint3;
    NeedTimer : GUint1;
  end;

  TGtkRange = record
    Widget : TGtkWidget;
    Trough, Slider, StepForw, StepBack : PGdkWindow;

    xClickPoint, yClickPoint : Gint16;
    Button : GUint8;
    Digits : Gint8;

    Frame1 : TGtkRangeFrame1;

    Timer : GUint32;
    OldValue, OldLower, OldUpper, OldPageSize : GFloat;
    Adjustment : PGtkAdjustment;
  end;

function GtkRangeGetType : TGtkType;
asmname 'gtk_range_get_type';

function GtkRangeGetAdjustment (Range : PGtkRange) : PGtkAdjustment;

procedure GtkRangeSetUpdatePolicy (Range : PGtkRange; Policy : TGtkUpdateType);
asmname 'gtk_range_set_update_policy';

procedure GtkRangeSetAdjustment (Range : PGtkRange; Adjustment : PGtkAdjustment);
asmname 'gtk_range_set_adjustment';

procedure GtkRangeDrawBackground (Range : PGtkRange);
asmname 'gtk_range_draw_background';

procedure GtkRangeClearBackground (Range : PGtkRange);
asmname 'gtk_range_clear_background';

procedure GtkRangeDrawTrough (Range : PGtkRange);
asmname 'gtk_range_draw_trough';

procedure GtkRangeDrawSlider (Range : PGtkRange);
asmname 'gtk_range_draw_slider';

procedure GtkRangeDrawStepForw (Range : PGtkRange);
asmname 'gtk_range_draw_step_forw';

procedure GtkRangeDrawStepBack (Range : PGtkRange);
asmname 'gtk_range_draw_step_back';

procedure GtkRangeSliderUpdate (Range : PGtkRange);
asmname 'gtk_range_slider_update';

function GtkRangeTroughClick (Range : PGtkRange; x, y : GInt; JumpPerc : PGFloat) : GInt;
asmname 'gtk_range_trough_click';

procedure GtkRangeDefaultHSliderUpdate(Range : PGtkRange);
asmname 'gtk_range_default_hslider_update';

procedure GtkRangeDefaultVSliderUpdate(Range : PGtkRange);
asmname 'gtk_range_default_vslider_update';

function GtkRangeDefaultHTroughClick (Range : PGtkRange; x, y : GInt; JumpPerc : PGFloat) : GInt;
asmname 'gtk_range_default_htrough_click';

function GtkRangeDefaultVTroughClick (Range : PGtkRange; x, y : GInt; JumpPerc : PGFloat) : GInt;
asmname 'gtk_range_default_vtrough_click';

procedure GtkRangeDefaultHMotion (Range : PGtkRange; xDelta, yDelta : GInt);
asmname 'gtk_range_default_hmotion';

procedure GtkRangeDefaultVMotion (Range : PGtkRange; xDelta, yDelta : GInt);
asmname 'gtk_range_default_vmotion';

implementation

function GTK_IS_RANGE (Obj : Pointer) : Boolean;
begin
  GTK_IS_RANGE := (Obj <> nil) and GTK_IS_RANGE_CLASS(PGtkTypeObject(Obj)^.Klass);
end;

function GTK_IS_RANGE_CLASS (Klass : Pointer) : Boolean;
begin
  GTK_IS_RANGE_CLASS := (Klass <> nil) and (GtkTypeIsA(PGtkTypeClass(Klass)^.TType, GTK_TYPE_RANGE) = G_TRUE);
end;

end.
