{
This unit is part of the Gtk+ units package.
Copyright (C) 2001  Nicola Girardi <nikke@swlibero.org>
                    Eike Lange <eike.lange@uni-essen.de>

This unit is free software; you can redistribute it and/or
modify it under the terms of the GNU unit General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This unit is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
unit General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this unit; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

Original GTK copyright and license information follows.
}

{
GTK - The GIMP Toolkit
Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
}

{
Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
file for a list of people on the GTK+ Team.  See the ChangeLog
files for a list of changes.  These files are distributed with
GTK+ at ftp://ftp.gtk.org/pub/gtk/.
}

Unit GtkText;

Interface

Uses
  Glib, GdkTypes, GtkAdjustment, GtkEditable, GtkObject, GtkWidget;

function GTK_TYPE_TEXT : TGtkType;
asmname 'gtk_text_get_type';

function GTK_IS_TEXT (Obj : Pointer) : Boolean;
function GTK_IS_TEXT_CLASS (Klass : Pointer) : Boolean;

(* @@@@@@@ return value could be gchar or gdkwchar...
see implementation
function GTK_TEXT_INDEX(T : PGtkText; I : Integer) : ???
*********)

Type
  PGtkTextFont = Pointer; { !! Warning. }
  PGtkPropertyMark = ^TGtkPropertyMark;
  PGtkText = ^TGtkText;
  PGtkTextClass = ^TGtkTextClass;
  
  TGtkPropertyMark = Record
    { Position in list. } 
    pproperty : PGList;
    { Offset into that property. } 
    offset : guint;
    { Current index. } 
    index : guint;
  End;

  UnionText = Record
    case longint of
      1:(wc:PgdkWChar);
      2:(ch:pguchar);
  End;

  TGtkText = record
    Editable : TGtkEditable;
    textArea : PGdkWindow;
    hadj, vadj : PGtkAdjustment;
    gc : PGdkGC;
    lineWrapBitmap, lineArrowBitmap : PGdkPixmap;
    
    { GAPPED TEXT SEGMENT } 
    
    { The text, a single segment of text a'la emacs, with a gap
    * where insertion occurs. } 
    TText : UnionText;
    { The allocated length of the text segment. } 
    TextLen : guint;
    { The gap position, index into address where a char
    * should be inserted. } 
    GapPosition : guint;
    { The gap size, s.t. *(text + gap_position + gap_size) is
    * the first valid character following the gap. } 
    GapSize : guint;
    { The last character position, index into address where a
    * character should be appeneded.  Thus, text_end - gap_size
    * is the length of the actual data. } 
    TextEnd : guint;
    
    { LINE START CACHE } 

    { A cache of line-start information.  Data is a LineParam*. } 
    lineStartCache : PGList;
    { Index to the start of the first visible line. } 
    firstLineStartIndex : guint;
    { The number of pixels cut off of the top line. } 
    firstCutPixels : guint;
    { First visible horizontal pixel. } 
    firstOnscreenHorPixel : guint;
    { First visible vertical pixel. } 
    firstOnscreenVerPixel : guint;
    
    { FLAGS } 

    { True iff this buffer is wrapping lines, otherwise it is using a
    * horizontal scrollbar. } 
    lineWrap, wordWrap : guint1;
    { If a fontset is supplied for the widget, use_wchar become true,
    * and we use GdkWchar as the encoding of text. } 
    useWchar : guint1;
    { Frozen, don't do updates. }
    freezeCount : guint;
    
    { TEXT PROPERTIES } 
    
    { A doubly-linked-list containing TextProperty objects. } 
    textProperties : PGList;
    { The end of this list. } 
    textProperties_end : PGList;
    { The first node before or on the point along with its offset to
    * the point and the buffer's current point.  This is the only
    * PropertyMark whose index is guaranteed to remain correct
    * following a buffer insertion or deletion. } 
    point : TGtkPropertyMark;
    
    { SCRATCH AREA } 

    scratchBuffer : UnionText;
    scratchBufferLen : guint;
    
    { SCROLLING } 
    
    lastVerValue : gint;
    
    { CURSOR } 
    
    cursorPosX : gint;        { Position of cursor. } 
    cursorPosY : gint;        { Baseline of line cursor is drawn on. } 
    cursorMark : TGtkPropertyMark;         { Where it is in the buffer. }
    cursorChar : GdkWChar;         { Character to redraw. } 
    cursorCharOffset : gchar;  { Distance from baseline of the font. } 
    cursorVirtualX : gint;    { Where it would be if it could be. } 
    cursorDrawnLevel : gint;  { How many people have undrawn. } 
    
    { Current Line } 
    
    currentLine : PGList ;
    
    { Tab Stops } 
    tabStops : PGList;
    defaultTabWidth : gint;
    currentFont : PGtkTextFont;         { Text font for current style } 
    { Timer used for auto-scrolling off ends } 
    timer : gint;
    button : guint;                      { currently pressed mouse button } 
    bgGc : PGdkGC;                      { gc for drawing background pixmap } 
  End;

  TGtkTextClass = record
    ParentClass : TGtkEditableClass;
    SetScrollAdjustments : ^Procedure (t:PGtkText; h,v:TGtkAdjustment);
  End;

Function GtkTextType : TGtkType;
  asmname 'gtk_text_get_type';
Function GtkTextNew(hadj, vadj:PGtkAdjustment) : PGtkWidget;
  asmname 'gtk_text_new';
Procedure GtkTextSetEditable(t:PGtkText; edi:gboolean);
  asmname 'gtk_text_set_editable';
Procedure GtkTextSetWordWrap(t:PGtkText; ww:gint);
  asmname 'gtk_text_set_word_wrap';
Procedure GtkTextSetLineWrap(t:PGtkText; lw:gint);
  asmname 'gtk_text_set_line_wrap';
Procedure GtkTextSetAdjustments(t:PGtkText; h,v:PGtkAdjustment);
  asmname 'gtk_text_set_adjustments';
Procedure GkTxtSetPoint(t:PGtkText; idx:guint);
  asmname 'gtk_text_set_point';
Function GtkTextGetPoint(t:PGtkText) : guint;
  asmname 'gtk_text_get_point';
Function GtkTextGetLength(t:PGtkText) : guint;
  asmname 'gtk_text_get_length';
Procedure GtkTextFreeze(t:PGtkText);
  asmname 'gtk_text_freeze';
Procedure GtkTextThaw(t:PGtkText);
  asmname 'gtk_text_thaw';
Procedure GtkTextInsert(t:PGtkText; font:PGdkFont; fore,back:PGdkColor;
                            chars:PGChar; lenght:gint);
  asmname 'gtk_text_insert';
Function GtkTextBackwardDelete(t:PGtkText; nchars:guint):gint;
  asmname 'gtk_text_backward_delete';
Function GtkTextForwardDelete(t:PGtkText; nchars:guint) :gint;
  asmname 'gtk_text_forward_delete';

Implementation

function GTK_IS_TEXT (Obj : Pointer) : Boolean;
begin
  GTK_IS_TEXT := (Obj <> nil) and GTK_IS_TEXT_CLASS(PGtkTypeObject(Obj)^.Klass);
end;

function GTK_IS_TEXT_CLASS (Klass : Pointer) : Boolean;
begin
  GTK_IS_TEXT_CLASS := (Klass <> nil) and (GtkTypeIsA(PGtkTypeClass(Klass)^.TType, GTK_TYPE_TEXT) = G_TRUE);
end;

(* @@@@@@@ return value could be gchar or gdkwchar...
function GTK_TEXT_INDEX(T : PGtkText; I : Integer) : ???
begin
  if T^.UseWChar <> G_FALSE then
    begin
      if I < T^.GapPosition then
        GTK_TEXT_INDEX := T^.TText.WC[I]
      else
        GTK_TEXT_INDEX := T^.Text.WC[I+T^.GapSize];
    end
  else
    begin
      if I < T^.GapPosition then
        GTK_TEXT_INDEX := T^.Text.CH[I]
      else
        GTK_TEXT_INDEX := T^.Text.CH[I+T^.GapSize];
    end;
end;
******)

End.
