{
This unit is part of the Gtk+ units package.
Copyright (C) 2001  Nicola Girardi <nikke@swlibero.org>
                    Eike Lange <eike.lange@uni-essen.de>

This unit is free software; you can redistribute it and/or
modify it under the terms of the GNU unit General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This unit is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
unit General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this unit; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

Original GTK copyright and license information follows.
}

{
GTK - The GIMP Toolkit
Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
}

{
Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
file for a list of people on the GTK+ Team.  See the ChangeLog
files for a list of changes.  These files are distributed with
GTK+ at ftp://ftp.gtk.org/pub/gtk/.
}

Unit GtkTree;

Interface

Uses
  GLib, GdkTypes, GtkContainer, GtkEnums, GtkObject, GtkTypeUtils, GtkWidget;

Function GTK_TYPE_TREE : TGtkType;
  asmname 'gtk_tree_get_type';

function GTK_IS_TREE (Obj : Pointer) : Boolean;
function GTK_IS_TREE_CLASS (Klass : Pointer) : Boolean;
function GTK_IS_ROOT_TREE(Obj : Pointer) : Boolean;

Const
  { GtkTreeViewMode }
  GTK_TREE_VIEW_LINE=0;   { default view mode } 
  GTK_TREE_VIEW_ITEM=1;

Type
  TGtkTreeViewMode = Integer;

  PGtkTree = ^TGtkTree;
  PGtkTreeClass = ^TGtkTreeClass;

  TGtkTree = Record
    Container : TGtkContainer;
    Children : PGList;
    RootTree : PGtkTree;  { owner of selection list }
    TreeOwner : PGtkWidget;
    Selection : PGList;
    Level, IndentValue, CurrentIndent : guint;
    SelectionMode : guint2;
    ViewMode, ViewLine : guint1;
  End;
  
  TGtkTreeClass = Record
    ParentClass : TGtkContainerClass;
    SelectionChanged : ^Procedure(tree : PGtkTree);
    SelectChild : ^Procedure (tree : PGtkTree;child : PGtkWidget);
    UnselectChild : ^Procedure (tree : PGtkTree;child : PGtkWidget);
  End;

function GTK_TREE_ROOT_TREE(Obj : Pointer) : PGtkTree;
function GTK_TREE_SELECTION(Obj : Pointer) : PGList;

Function GtkTreeGetType : TGtkType;
  asmname 'gtk_tree_get_type';
Function gtkTreeNew : PGtkWidget;
  asmname 'gtk_tree_new';
Procedure GtkTreeAppend(t:PGtkTree; w:PGtkWidget);
  asmname 'gtk_tree_append';
Procedure GtkTreePrepend(t:PGtkTree; w:PGtkWidget);
  asmname 'gtk_tree_prepend';
Procedure GtkTreeInsert(t:PGtkTree; w:PGtkWidget; posi:gint);
  asmname 'gtk_tree_insert';
Procedure GtkTreeRemoveItems(t:PGtkTree; w:PGtkWidget);
  asmname 'gtk_tree_remove_items';
Procedure GtkTreeClearItems(t:PGtkTree; beginn, endd : gint);
  asmname 'gtk_tree_clear_items';
Procedure GtkTreeSelectItem(t:PGtkTree; itm : gint);
  asmname 'gtk_tree_select_item';
Procedure GtkTreeUnselectItem(t:PGtkTree; i:gint);
  asmname 'gtk_tree_unselect_item';
Procedure GtkTreeSelectChild(t:PGtkTree; i:PGtkWidget);
  asmname 'gtk_tree_select_child';
Procedure GtkTreeUnselectChild(t:PGtkTree; i:PGtkWidget);
  asmname 'gtk_tree_unselect_child';
Function GtkTreeChildPosition(t:PGtkTree; c:PGtkWidget):gint;
  asmname 'gtk_tree_child_position';
Procedure GtkTreeSetSelectionMode(t:PGtkTree; m:TGtkSelectionMode);
  asmname 'gtk_tree_set_selection_mode';
Procedure GtkTreeSetViewMode(t:PGtkTree; m:TGtkTreeViewMode);
  asmname 'gtk_tree_set_view_mode';
Procedure GtkTreeSetViewLines(t:PGtkTree; flag:guint);
  asmname 'gtk_tree_set_view_lines';

{ deprecated function, use gtk_container_remove instead. }
procedure GtkTreeRemoveItem(Tree : PGtkTree; Child : PGtkWidget);
asmname 'gtk_tree_remove_item';

Implementation

function GTK_IS_TREE (Obj : Pointer) : Boolean;
begin
  GTK_IS_TREE := (Obj <> nil) and GTK_IS_TREE_CLASS(PGtkTypeObject(Obj)^.Klass);
end;

function GTK_IS_TREE_CLASS (Klass : Pointer) : Boolean;
begin
  GTK_IS_TREE_CLASS := (Klass <> nil) and (GtkTypeIsA(PGtkTypeClass(Klass)^.TType, GTK_TYPE_TREE) = G_TRUE);
end;

function GTK_IS_ROOT_TREE(Obj : Pointer) : Boolean;
begin
  GTK_IS_ROOT_TREE := (PGtkObject(PGtkTree(Obj)^.RootTree) = PGtkObject(Obj));
end;

function GTK_TREE_ROOT_TREE(Obj : Pointer) : PGtkTree;
begin
  if PGtkTree(Obj)^.RootTree <> nil then
    GTK_TREE_ROOT_TREE := PGtkTree(Obj)^.RootTree
  else
    GTK_TREE_ROOT_TREE := PGtkTree(Obj);
end;

function GTK_TREE_SELECTION(Obj : Pointer) : PGList;
begin
  GTK_TREE_SELECTION := GTK_TREE_ROOT_TREE(Obj)^.Selection;
end;

End.
