{
This unit is part of the Gtk+ units package.
Copyright (C) 2001  Nicola Girardi <nikke@swlibero.org>
                    Eike Lange <eike.lange@uni-essen.de>

This unit is free software; you can redistribute it and/or
modify it under the terms of the GNU unit General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This unit is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
unit General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this unit; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

Original GTK copyright and license information follows.
}

{
GTK - The GIMP Toolkit
Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
}

{
Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
file for a list of people on the GTK+ Team.  See the ChangeLog
files for a list of changes.  These files are distributed with
GTK+ at ftp://ftp.gtk.org/pub/gtk/.
}

unit GtkTypeUtils;

interface

uses Glib, GtkObject;

{ Fundamental Types }
const
  GTK_TYPE_INVALID = 0;
  GTK_TYPE_NONE    = 1;
  
  { Flat types }
  GTK_TYPE_CHAR    = 2;
  GTK_TYPE_UCHAR   = 3;
  GTK_TYPE_BOOL    = 4;
  GTK_TYPE_INT     = 5;
  GTK_TYPE_UINT    = 6;
  GTK_TYPE_LONG    = 7;
  GTK_TYPE_ULONG   = 8;
  GTK_TYPE_FLOAT   = 9;
  GTK_TYPE_DOUBLE  = 10;
  GTK_TYPE_STRING  = 11;
  GTK_TYPE_ENUM    = 12;
  GTK_TYPE_FLAGS   = 13;
  GTK_TYPE_BOXED   = 14;
  GTK_TYPE_POINTER = 15;
  
  { Structured types }
  GTK_TYPE_SIGNAL     = 16;
  GTK_TYPE_ARGS       = 17;
  GTK_TYPE_CALLBACK   = 18;
  GTK_TYPE_C_CALLBACK = 19;
  GTK_TYPE_FOREIGN    = 20;

  { Base type node of the object system }
  GTK_TYPE_OBJECT = 21;

type { This refers to the previous enumeration. }
  TGtkFundamentalType = Integer;

{ Bounds definitions for type sets, these are provided to distinct
  between fundamental types with if() statements, and to build
  up foreign fundamentals. }
const
  GTK_TYPE_FLAT_FIRST       = GTK_TYPE_CHAR;
  GTK_TYPE_FLAT_LAST        = GTK_TYPE_POINTER;
  GTK_TYPE_STRUCTURED_FIRST = GTK_TYPE_SIGNAL;
  GTK_TYPE_STRUCTURED_LAST  = GTK_TYPE_FOREIGN;
  GTK_TYPE_FUNDAMENTAL_LAST = GTK_TYPE_OBJECT;
  GTK_TYPE_FUNDAMENTAL_MAX  = 32;

{ @@ GTK_STRUCT_OFFSET moved to gtk.inc because $define'd macros
     can't be exported to other units. }

{ !! Exported (P|T)GtkType to gtkobject.pas. }

function GTK_TYPE_IDENTIFIER : TGtkType;
asmname 'gtk_identifier_get_type';

function GtkIdentifierGetType : TGtkType;
asmname 'gtk_identifier_get_type';

{ Macros }
function GTK_TYPE_MAKE(ParentT, SeqNo : TGtkType) : TGtkType;
function GTK_FUNDAMENTAL_TYPE(TType : TGtkType) : TGtkFundamentalType;
function GTK_TYPE_SEQNO(TType : TGtkType) : TGtkType;

type
  TGtkClassInitFunc = Pointer; { !! Because function/proc type-casting isn't allowed. }
  TGtkObjectInitFunc = Pointer; { !! Because function/proc type-casting isn't allowed. }

{ !! Exported some pointers to procedures or functions to gtkobject.pas. }
{ !! Exported GtkType(Object|Class) to gtkobject.pas. }
{ !! Exported GtkArg declaration to gtkobject.pas. }

{ Argument value access macros, these must not contain casts,
  to allow the usage of these macros in combination with the
  adress operator, e.g. &GTK_VALUE_CHAR (*arg) }

{ Flat values }
function GTK_VALUE_CHAR(const a : TGtkArg) : GChar;
function GTK_VALUE_UCHAR(const a : TGtkArg) : GUChar;
function GTK_VALUE_BOOL(const a : TGtkArg) : GBoolean;
function GTK_VALUE_INT(const a : TGtkArg) : GInt;
function GTK_VALUE_UINT(const a : TGtkArg) : GUint;
function GTK_VALUE_LONG(const a : TGtkArg) : GLong;
function GTK_VALUE_ULONG(const a : TGtkArg) : GULong;
function GTK_VALUE_FLOAT(const a : TGtkArg) : GFloat;
function GTK_VALUE_DOUBLE(const a : TGtkArg) : GDouble;
function GTK_VALUE_ENUM(const a : TGtkArg) : Integer;
function GTK_VALUE_FLAGS(const a : TGtkArg) : Integer;
function GTK_VALUE_BOXED(const a : TGtkArg) : GPointer;
function GTK_VALUE_POINTER(const a : TGtkArg) : GPointer;
function GTK_VALUE_OBJECT(const a : TGtkArg): GPointer;

{ Structured values }
function GTK_VALUE_SIGNAL(const a : TGtkArg) : TSignalRecord;
function GTK_VALUE_ARGS(const a : TGtkArg) : TArgsRecord;
function GTK_VALUE_CALLBACK(const a : TGtkArg) : TCallbackDataRecord;
function GTK_VALUE_C_CALLBACK(const a : TGtkArg) : TCCallbackDataRecord;
function GTK_VALUE_FOREIGN(const a : TGtkArg) : TForeignDataRecord;

{ Return location macros, these all narow down to
  pointer types, because return values need to be
  passed by reference }

{ Flat values }
function GTK_RETLOC_CHAR(const a : TGtkArg) : PGChar;
function GTK_RETLOC_UCHAR(const a : TGtkArg) : PGUChar;
function GTK_RETLOC_BOOL(const a : TGtkArg) : PGBoolean;
function GTK_RETLOC_INT(const a : TGtkArg) : PGInt;
function GTK_RETLOC_UINT(const a : TGtkArg) : PGUint;
function GTK_RETLOC_LONG(const a : TGtkArg) : PGLong;
function GTK_RETLOC_ULONG(const a : TGtkArg) : PGULong;
function GTK_RETLOC_FLOAT(const a : TGtkArg) : PGFloat;
function GTK_RETLOC_DOUBLE(const a : TGtkArg) : PGDouble;
function GTK_RETLOC_ENUM(const a : TGtkArg) : PGInt;
function GTK_RETLOC_FLAGS(const a : TGtkArg) : PGUint;
function GTK_RETLOC_BOXED(const a : TGtkArg) : PGPointer;

type
  PGtkTypeInfo = ^TGtkTypeInfo;
  TGtkTypeInfo = record
    TypeName : PGChar;
    ObjectSize, ClassSize : GUint;
    ClassInitFunc : TGtkClassInitFunc;
    ObjectInitFunc : TGtkObjectInitFunc;
    Reserved1, Reserved2 : GPointer;
    BaseClassInitFunc : TGtkClassInitFunc;
  end;

type
  PGtkTypeQuery = ^TGtkTypeQuery;
  TGtkTypeQuery = record
    TType : TGtkType;
    TypeName : PGChar;
    ObjectSize, ClassSize : GUint;
  end;

type
  PGtkEnumValue = ^TGtkEnumValue;
  PGtkFlagValue = ^TGtkFlagValue;
  TGtkEnumValue = record
    VValue : GUint;
    ValueName, ValueNick : PGChar;
  end;
  TGtkFlagValue = TGtkEnumValue;

procedure GtkTypeInit;
asmname 'gtk_type_init';

function GtkTypeUnique(ParentType : TGtkType; TypeInfo : PGtkTypeInfo) : TGtkType;
asmname 'gtk_type_unique';

procedure GtkTypeSetChunkAlloc(TType : TGtkType; NChunks : GUint);
asmname 'gtk_type_set_chunk_alloc';

function GtkTypeName(TType : GUint) : PGChar;
asmname 'gtk_type_name';

function GtkTypeFromName(Name : PGChar) : TGtkType;
asmname 'gtk_type_from_name';

function GtkTypeParent(TType : TGtkType) : TGtkType;
asmname 'gtk_type_parent';

function GtkTypeClass(TType : TGtkType) : GPointer;
asmname 'gtk_type_class';

function GtkTypeParentClass(TType : TGtkType) : GPointer;
asmname 'gtk_type_parent_class';

function GtkTypeChildrenTypes(TType : TGtkType) : PGList;
asmname 'gtk_type_children_types';

function GtkTypeNew(TType : TGtkType) : GPointer;
asmname 'gtk_type_new';

procedure GtkTypeFree(TType : TGtkType; Mem : GPointer);
asmname 'gtk_type_free';

procedure GtkTypeDescribeHeritage(TType : TGtkType);
asmname 'gtk_type_describe_heritage';

procedure GtkTypeDescribeTree(TType : TGtkType; ShowSize : GBoolean);
asmname 'gtk_type_describe_tree';

{ !! GtkTypeIsA written in gtkobject.pas. }

function GtkTypeCheckObjectCast(TypeObject : PGtkTypeObject; CastType : TGtkType) : PGtkTypeObject;
asmname 'gtk_type_check_object_cast';

function GtkTypeCheckClassCast(Klass : PGtkTypeClass; CastType : TGtkType) : PGtkTypeClass;
asmname 'gtk_type_check_class_cast';

function GtkTypeRegisterEnum(TypeName : PGChar; Values : PGtkEnumValue) : TGtkType;
asmname 'gtk_type_register_enum';

function GtkTypeRegisterFlags(TypeName : PGChar; Values : PGtkFlagValue) : TGtkType;
asmname 'gtk_type_register_flags';

function GtkTypeEnumGetValues(EnumType : TGtkType) : PGtkEnumValue;
asmname 'gtk_type_enum_get_values';

function GtkTypeEnumGetValues(FlagsType : TGtkType) : PGtkFlagValue;
asmname 'gtk_type_flags_get_values';

function GtkTypeEnumFindValue(EnumType : TGtkType; ValueName : PGChar) : PGtkEnumValue;
asmname 'gtk_type_enum_find_value';

function GtkTypeFlagsFindValue(FlagsType : TGtkType; ValueName : PGChar) : PGtkFlagValue;
asmname 'gtk_type_flags_find_value';

{ Set the argument collector alias for foreign fundamentals. }
procedure GtkTypeSetVarArgsType(ForeignType, VarArgsType : TGtkType);
asmname 'gtk_type_set_varargs_type';

function GtkTypeGetVarArgsType(ForeignType : TGtkType) : TGtkType;
asmname 'gtk_type_get_varargs_type';

{ Report internal information about a type. The caller has the
  responsibility to invoke a subsequent g_free (returned_data); but
  must not modify data pointed to by the members of GtkTypeQuery }
function GtkTypeQuery(TType : TGtkType) : PGtkTypeQuery;
asmname 'gtk_type_query';

implementation

function GTK_TYPE_MAKE(ParentT, SeqNo : TGtkType) : TGtkType;
begin
  GTK_TYPE_MAKE := (SeqNo shl 8) or (GTK_FUNDAMENTAL_TYPE(ParentT));
end;

function GTK_FUNDAMENTAL_TYPE(TType : TGtkType) : TGtkFundamentalType;
begin
  GTK_FUNDAMENTAL_TYPE := TGtkFundamentalType(TType and 16#ff);
end;

function GTK_TYPE_SEQNO(TType : TGtkType) : TGtkType;
begin
  if TType > 16#ff then
    GTK_TYPE_SEQNO := TType shr 8
  else
    GTK_TYPE_SEQNO := TType;
end;

function GTK_VALUE_CHAR(const a : TGtkArg) : GChar;
begin
  GTK_VALUE_CHAR := a.d.CharData;
end;

function GTK_VALUE_UCHAR(const a : TGtkArg) : GUChar;
begin
  GTK_VALUE_UCHAR := a.d.UCharData;
end;

function GTK_VALUE_BOOL(const a : TGtkArg) : GBoolean;
begin
  GTK_VALUE_BOOL := a.d.BoolData;
end;

function GTK_VALUE_INT(const a : TGtkArg) : GInt;
begin
  GTK_VALUE_INT := a.d.IntData;
end;

function GTK_VALUE_UINT(const a : TGtkArg) : GUint;
begin
  GTK_VALUE_UINT := a.d.UIntData;
end;

function GTK_VALUE_LONG(const a : TGtkArg) : GLong;
begin
  GTK_VALUE_LONG := a.d.LongData;
end;

function GTK_VALUE_ULONG(const a : TGtkArg) : GULong;
begin
  GTK_VALUE_ULONG := a.d.ULongData;
end;

function GTK_VALUE_FLOAT(const a : TGtkArg) : GFloat;
begin
  GTK_VALUE_FLOAT := a.d.FloatData;
end;

function GTK_VALUE_DOUBLE(const a : TGtkArg) : GDouble;
begin
  GTK_VALUE_DOUBLE := a.d.DoubleData;
end;

function GTK_VALUE_ENUM(const a : TGtkArg) : Integer;
begin
  GTK_VALUE_ENUM := a.d.IntData;
end;

function GTK_VALUE_FLAGS(const a : TGtkArg) : Integer;
begin
  GTK_VALUE_FLAGS := a.d.UIntData;
end;

function GTK_VALUE_BOXED(const a : TGtkArg) : GPointer;
begin
  GTK_VALUE_BOXED := a.d.PointerData;
end;

function GTK_VALUE_POINTER(const a : TGtkArg) : GPointer;
begin
  GTK_VALUE_POINTER := a.d.PointerData;
end;

function GTK_VALUE_OBJECT(const a : TGtkArg) : GPointer;
begin
  GTK_VALUE_OBJECT := a.d.ObjectData;
end;

function GTK_VALUE_SIGNAL(const a : TGtkArg) : TSignalRecord;
begin
  GTK_VALUE_SIGNAL := a.d.SignalData;
end;

function GTK_VALUE_ARGS(const a : TGtkArg) : TArgsRecord;
begin
  GTK_VALUE_ARGS := a.d.ArgsData;
end;

function GTK_VALUE_CALLBACK(const a : TGtkArg) : TCallbackDataRecord;
begin
  GTK_VALUE_CALLBACK:=a.d.CallbackData;
end;

function GTK_VALUE_C_CALLBACK(const a : TGtkArg) : TCCallbackDataRecord;
begin
  GTK_VALUE_C_CALLBACK := a.d.CCallbackData;
end;

function GTK_VALUE_FOREIGN(const a : TGtkArg) : TForeignDataRecord;
begin
  GTK_VALUE_FOREIGN := a.d.ForeignData;
end;

function GTK_RETLOC_CHAR(const a : TGtkArg) : PGChar;
begin
  GTK_RETLOC_CHAR := PGChar(a.d.PointerData);
end;

function GTK_RETLOC_UCHAR(const a : TGtkArg) : PGUChar;
begin
  GTK_RETLOC_UCHAR := PGUChar(a.d.PointerData);
end;

function GTK_RETLOC_BOOL(const a : TGtkArg) : PGBoolean;
begin
  GTK_RETLOC_BOOL := PGBoolean(a.d.PointerData);
end;

function GTK_RETLOC_INT(const a : TGtkArg) : PGInt;
begin
  GTK_RETLOC_INT := PGInt(a.d.PointerData);
end;

function GTK_RETLOC_UINT(const a : TGtkArg) : PGUint;
begin
  GTK_RETLOC_UINT := PGUint(a.d.PointerData);
end;

function GTK_RETLOC_LONG(const a : TGtkArg) : PGLong;
begin
  GTK_RETLOC_LONG := PGLong(a.d.PointerData);
end;

function  GTK_RETLOC_ULONG(const a : TGtkArg) : PGULong;
begin
  GTK_RETLOC_ULONG := PGULong(a.d.PointerData);
end;

function GTK_RETLOC_FLOAT(const a : TGtkArg) : PGFloat;
begin
  GTK_RETLOC_FLOAT:=PGFloat(a.d.PointerData);
end;

function GTK_RETLOC_DOUBLE(const a : TGtkArg) : PGDouble;
begin
  GTK_RETLOC_DOUBLE := PGDouble(a.d.PointerData);
end;

function GTK_RETLOC_ENUM(const a : TGtkArg) : PGInt;
begin
  GTK_RETLOC_ENUM := PGInt(a.d.PointerData);
end;

function GTK_RETLOC_FLAGS(const a : TGtkArg) : PGUint;
begin
  GTK_RETLOC_FLAGS := PGUint(a.d.PointerData);
end;

function GTK_RETLOC_BOXED(const a : TGtkArg) : PGPointer;
begin
  GTK_RETLOC_BOXED := PGPointer(a.d.PointerData);
end;

function GTK_RETLOC_POINTER(const a : TGtkArg) : PGPointer;
begin
  GTK_RETLOC_POINTER := PGPointer(a.d.PointerData);
end;

end.
