{
This unit is part of the Gtk+ units package.
Copyright (C) 2001  Nicola Girardi <nikke@swlibero.org>
                    Eike Lange <eike.lange@uni-essen.de>

This unit is free software; you can redistribute it and/or
modify it under the terms of the GNU unit General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This unit is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
unit General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this unit; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

Original GTK copyright and license information follows.
}

{
GTK - The GIMP Toolkit
Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
}

{
Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
file for a list of people on the GTK+ Team.  See the ChangeLog
files for a list of changes.  These files are distributed with
GTK+ at ftp://ftp.gtk.org/pub/gtk/.
}

unit GtkWidget;

interface

uses Glib, GdkTypes, GtkAccelGroup, GtkAdjustment, GtkEnums, GtkObject, GtkTypeUtils;

{ The flags that are used by GtkWidget on top of the
  flags field of GtkObject. }

const
  GTK_TOPLEVEL         = 1 shl 4;
  GTK_NO_WINDOW        = 1 shl 5;
  GTK_REALIZED         = 1 shl 6;
  GTK_MAPPED           = 1 shl 7;
  GTK_VISIBLE          = 1 shl 8;
  GTK_SENSITIVE        = 1 shl 9;
  GTK_PARENT_SENSITIVE = 1 shl 10;
  GTK_CAN_FOCUS        = 1 shl 11;
  GTK_HAS_FOCUS        = 1 shl 12;
  GTK_CAN_DEFAULT      = 1 shl 13;
  GTK_HAS_DEFAULT      = 1 shl 14;
  GTK_HAS_GRAB         = 1 shl 15;
  GTK_RC_STYLE         = 1 shl 16;
  GTK_COMPOSITE_CHILD  = 1 shl 17;
  GTK_NO_REPARENT      = 1 shl 18;
  GTK_APP_PAINTABLE    = 1 shl 19;
  GTK_RECEIVES_DEFAULT = 1 shl 20;
type
  PGtkWidgetFlags = ^TGtkWidgetFlags;
  TGtkWidgetFlags = Integer;

function GTK_TYPE_WIDGET : TGtkType;
asmname 'gtk_widget_get_type';

{ !! Please access the ^.State and ^.SavedState without involving macros. }

{ Macros for extracting the widget flags from GtkWidget. }
function GTK_WIDGET_TOPLEVEL(Wid : Pointer) : Boolean;
function GTK_WIDGET_NO_WINDOW(Wid : Pointer) : Boolean;
function GTK_WIDGET_REALIZED(Wid : Pointer) : Boolean;
function GTK_WIDGET_MAPPED(Wid : Pointer) : Boolean;
function GTK_WIDGET_VISIBLE(Wid : Pointer) : Boolean;
function GTK_WIDGET_DRAWABLE(Wid : Pointer) : Boolean;
function GTK_WIDGET_SENSITIVE(Wid : Pointer) : Boolean;
function GTK_WIDGET_PARENT_SENSITIVE(Wid : Pointer) : Boolean;
function GTK_WIDGET_IS_SENSITIVE(Wid : Pointer) : Boolean;
function GTK_WIDGET_CAN_FOCUS(Wid : Pointer) : Boolean;
function GTK_WIDGET_HAS_FOCUS(Wid : Pointer) : Boolean;
function GTK_WIDGET_CAN_DEFAULT(Wid : Pointer) : Boolean;
function GTK_WIDGET_HAS_DEFAULT(Wid : Pointer) : Boolean;
function GTK_WIDGET_HAS_GRAB(Wid : Pointer) : Boolean;
function GTK_WIDGET_RC_STYLE(Wid : Pointer) : Boolean;
function GTK_WIDGET_COMPOSITE_CHILD(Wid : Pointer) : Boolean;
function GTK_WIDGET_APP_PAINTABLE(Wid : Pointer) : Boolean;
function GTK_WIDGET_RECEIVES_DEFAULT(Wid : Pointer) : Boolean;

{ Macros for setting and clearing widget flags. }
procedure GTK_WIDGET_SET_FLAGS(Wid : Pointer; Flag : LongInt);
procedure GTK_WIDGET_UNSET_FLAGS(Wid : Pointer; Flag : LongInt);

{ !! Taken from gtkrcstyle.pas to solve interdependencies. }

const
  GTK_RC_FG   = 1 shl 0;
  GTK_RC_BG   = 1 shl 1;
  GTK_RC_TEXT = 1 shl 2;
  GTK_RC_BASE = 1 shl 3;
type
  PGtkRcFlags = ^TGtkRcFlags;
  TGtkRcFlags = Integer;

{ !! End of foreign code. }

type
  { A requisition is a desired amount of space which a widget may request. }
  PGtkRequisition = ^TGtkRequisition;
  TGtkRequisition  = record
    Width, Height : GInt16;
  end;

  { An allocation is a size and position. Where a widget can ask for a
    desired size, it is actually given this amount of space at the
    specified position. }
  PGtkAllocation = ^TGtkAllocation;
  TGtkAllocation  = record
    x, y : GInt16;
    Width, Height : GUint16;
  end;

  { The contents of a selection are returned in a GtkSelectionData
    structure. selection/target identify the request.
    type specifies the type of the return; if length < 0, and
    the data should be ignored. This structure has object semantics -
    no fields should be modified directly, they should not be created
    directly, and pointers to them should not be stored beyond the duration of
    a callback. (If the last is changed, we'll need to add reference
    counting.) The time field gives the timestamp at which the data was sent. }
  PGtkSelectionData = ^TGtkSelectionData;
  TGtkSelectionData  = record
    Selection, Target, TType : GdkAtom;

    Format : GInt;
    Data   : PGUChar;
    Length : GInt;
  end;

  { The widget is the base of the tree for displayable objects.
    (A displayable object is one which takes up some amount
    of screen real estate). It provides a common base and interface
    which actual widgets must adhere to. }
  PPGtkWidget = ^PGtkWidget;
  PGtkWidget = ^TGtkWidget;

  { !! Taken from gtktheme.pas and gtkstyle.pas and gtkrcstyle.pas
       to solve interdependencies. }
  PGtkThemeEngine = ^TGtkThemeEngine;
  PGtkStyleClass = ^TGtkStyleClass;
  PGtkRcStyle = ^TGtkRcStyle;
  PGtkStyle = ^TGtkStyle;

  flagArray = array[0..4] of TGtkRcFlags;
  pixnArray = array[0..4] of PGChar;
  colArray = array[0..4] of GdkColor;

  TGtkRcStyle = record
    Name, FontName, FontsetName : PGChar;
    BgPixmapName : pixnArray;
    ColorFlags : flagArray;
    Fg, Bg, Txt, Base : colArray;
    Engine : PGtkThemeEngine;
    EngineData : GPointer;
  end;

  TGtkThemeEngine = record
    { Fill in engine_data pointer in a GtkRcStyle by parsing contents
      of brackets. Returns G_TOKEN_NONE if succesfull, otherwise returns
      the token it expected but didn't get. }
    ParseRcStyle : ^function (Scanner : PGScanner; RcStyle : PGtkRcStyle) : GUint;

    { Combine RC style data from src into dest. If
      dest->engine_data is NULL, it should be initialized to default
      values. }
    MergeRcStyle : ^procedure (Dest, Src : PGtkRcStyle);

    { Fill in style->engine_data from rc_style->engine_data. }
    RcStyleToStyle : ^procedure (Style : PGtkStyle; RcStyle : PGtkRcStyle);

    { Duplicate engine_data from src to dest. The engine_data will
      not subsequently be modified except by a call to realize_style()
      so if realize_style() does nothing, refcounting is appropriate. }
    DuplicateStyle : ^procedure (Dest, Src : PGtkStyle);

    { If style needs to initialize for a particular colormap/depth
      combination, do it here. style->colormap/style->depth will have
      been set at this point, and style itself initialized for
      the colormap. }
    RealizeStyle : ^procedure (NewStyle : PGtkStyle);

    { If style needs to clean up for a particular colormap/depth
      combination, do it here. }
    UnrealizeStyle : ^procedure (NewStyle : PGtkStyle);

    { Clean up rc_style->engine_data before rc_style is destroyed. }
    DestroyRcStyle : ^procedure (RcStyle : PGtkRcStyle);

    { Clean up style->engine_data before style is destroyed. }
    DestroyStyle : ^procedure (Style : PGtkStyle);

    SetBackground : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType);
  end;

  TGtkStyleClass = record
    XThickNess, YThickNess : GInt;

    DrawHLine : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType; Area : PGdkRectangle; Widget : PGtkWidget; Detail : PGChar; X1, X2, Y : GInt);
    DrawVLine : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType; Area : PGdkRectangle; Widget : PGtkWidget; Detail : PGChar; Y1, Y2, X : GInt);
    DrawShadow : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType; ShadowType : TGtkShadowType; Area : PGdkRectangle; Widget : PGtkWidget; Detail : PGChar; X, Y, Width, Height : GInt);
    DrawPolygon : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType; ShadowType : TGtkShadowType; Area : PGdkRectangle; Widget : PGtkWidget; Detail : PGChar; Point : PGdkPoint; NPoints : GInt; Fill : GBoolean);
    DrawArrow : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType; ShadowType : TGtkShadowType; Area : PGdkRectangle; Widget : PGtkWidget; Detail : PGChar; ArrowType : TGtkArrowType; Fill : GBoolean; X, Y, Width, Height : GInt);
    DrawDiamond : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType; ShadowType : TGtkShadowType; Area : PGdkRectangle; Widget : PGtkWidget; Detail : PGChar; X, Y, Width, Height : GInt);
    DrawOval : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType; ShadowType : TGtkShadowType; Area : PGdkRectangle; Widget : PGtkWidget; Detail : PGChar; X, Y, Width, Height : GInt);
    DrawString : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType; ShadowType : TGtkShadowType; Area : PGdkRectangle; Widget : PGtkWidget; Detail : PGChar; X, Y : GInt; Str : PGChar);
    DrawBox : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType; ShadowType : TGtkShadowType; Area : PGdkRectangle; Widget : PGtkWidget; Detail : PGChar; X, Y, Width, Height : GInt);
    DrawFlaxBox : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType; ShadowType : TGtkShadowType; Area : PGdkRectangle; Widget : PGtkWidget; Detail : PGChar; X, Y, Width, Height : GInt);
    DrawCheck : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType; ShadowType : TGtkShadowType; Area : PGdkRectangle; Widget : PGtkWidget; Detail : PGChar; X, Y, Width, Height : GInt);
    DrawOption : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType; ShadowType : TGtkShadowType; Area : PGdkRectangle; Widget : PGtkWidget; Detail : PGChar; X, Y, Width, Height : GInt);
    DrawCross : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType; ShadowType : TGtkShadowType; Area : PGdkRectangle; Widget : PGtkWidget; Detail : PGChar; X, Y, Width, Height : GInt);
    DrawRamp : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType; ShadowType : TGtkShadowType; Area : PGdkRectangle; Widget : PGtkWidget; Detail : PGChar; X, Y, Width, Height : GInt);
    DrawTab : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType; ShadowType : TGtkShadowType; Area : PGdkRectangle; Widget : PGtkWidget; Detail : PGChar; X, Y, Width, Height : GInt);
    DrawShadowGap : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType; ShadowType : TGtkShadowType; Area : PGdkRectangle; Widget : PGtkWidget; Detail : PGChar; X, Y, Width, Height : GInt; GapSide : TGtkPositionType; GapX, GapWidth : GInt);
    DrawBoxGap : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType; ShadowType : TGtkShadowType; Area : PGdkRectangle; Widget : PGtkWidget; Detail : PGChar; X, Y, Width, Height : GInt; GapSide : TGtkPositionType; GapX, GapWidth : GInt);
    DrawExtension : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType; ShadowType : TGtkShadowType; Area : PGdkRectangle; Widget : PGtkWidget; Detail : PGChar; X, Y, Width, Height : GInt; GapSide : TGtkPositionType);
    DrawFocus : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType; ShadowType : TGtkShadowType; Area : PGdkRectangle; Widget : PGtkWidget; Detail : PGChar; X, Y, Width, Height : GInt);
    DrawSlider : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType; ShadowType : TGtkShadowType; Area : PGdkRectangle; Widget : PGtkWidget; Detail : PGChar; X, Y, Width, Height : GInt; Orientation : TGtkOrientation);
    DrawHandle : ^procedure (Style : PGtkStyle; Window : PGdkWindow; StateType : TGtkStateType; ShadowType : TGtkShadowType; Area : PGdkRectangle; Widget : PGtkWidget; Detail : PGChar; X, Y, Width, Height : GInt; Orientation : TGtkOrientation);
  end;

  gcArray = array[0..4] of PGdkGC;
  pixArray = array[0..4] of PGdkPixmap;

  TGtkStyle = record
    Klass : PGtkStyleClass;

    Fg, Bg, Light, Dark, Mid, TText, Base : colArray;
    Black, White : GdkColor;

    Font : PGdkFont;

    FgGC, BgGC, LightGC, DarkGC, MidGC, TextGC, BaseGC : gcArray;
    BlackGC, WhiteGC : PGdkGC;

    BgPixmap : pixArray;

    { Private. }
    RefCount, AttachCount, Depth : GInt;
    Colormap : PGdkColormap;
    Engine : PGtkThemeEngine;
    EngineData : GPointer;
    RcStyle : PGtkRCStyle; { The Rc style from which this style was created. }
    Styles : PGSList;
  end;

  { !! End of foreing code. }

  TGtkWidget = record
    { The object structure needs to be the first element in the widget
      structure in order for the object mechanism to work correctly. This
      allows a GtkWidget pointer to be cast to a GtkObject pointer. }
    OObject : TGtkObject;

    { 16 bits of internally used private flags.
      this will be packed into the same 4 byte alignment frame that
      state and saved_state go. We therefore don't waste any new
      space on this. }
    PrivateFlags : GUint16;

    { The state of the widget. There are actually only
      5 widget states (defined in "gtkenums.h"). }
    State : GUint8;
  
    { The saved state of the widget. When a widgets state
      is changed to GTK_STATE_INSENSITIVE via
      "gtk_widget_set_state" or "gtk_widget_set_sensitive"
      the old state is kept around in this field. The state
      will be restored once the widget gets sensitive again. }
    SavedState : GUint8;
  
    { The widget's name. If the widget does not have a name
      (the name is NULL), then its name (as returned by
      "gtk_widget_get_name") is its classes name.
      Among other things, the widget name is used to determine
      the style to use for a widget. }
    Name : PGChar;

    { The style for the widget. The style contains the
      colors the widget should be drawn in for each state
      along with graphics contexts used to draw with and
      the font to use for text. }
    Style : PGtkStyle;

    { The widgets desired size. }
    Requisition : TGtkRequisition;

    { The widgets allocated size. }
    Allocation : TGtkAllocation;

    { The widgets window or its parent window if it does
      not have a window. (Which will be indicated by the
      GTK_NO_WINDOW flag being set). }
    Window : PGdkWindow;

    { The widgets parent. }
    Parent : PGtkWidget;
  end;

  TGtkCallback = ^procedure (Widget : PGtkWidget; Data : GPointer);

  PGtkWidgetClass = ^TGtkWidgetClass;
  TGtkWidgetClass = record
    { The object class structure needs to be the first
      element in the widget class structure in order for
      the class mechanism to work correctly. This allows a
      GtkWidgetClass pointer to be cast to a GtkObjectClass
      pointer. }
    ParentClass : TGtkObjectClass;

    { The signal to emit when a widget of this class is activated,
      gtk_widget_activate() handles the emission.
      Implementation of this signal is optional. }
    ActivateSignal : GUint;

    { This signal is emitted  when a widget of this class is added
      to a scrolling aware parent, gtk_widget_set_scroll_adjustments()
      handles the emission. Implementation of this signal is optional. }
    SetScrollAdjustmentsSignal : GUint;
  
    { Basics. }
    Show, ShowAll, Hide, HideAll, Map, Unmap, Realize, Unrealize : ^procedure (Widget : PGtkWidget);
    Draw : ^procedure (Widget : PGtkWidget; Area : PGdkRectangle);
    DrawFocus, DrawDefault: ^procedure (Widget : PGtkWidget);
    SizeRequest : ^procedure (Widget : PGtkWidget; Requisition : PGtkRequisition);
    SizeAllocate : ^procedure (Widget : PGtkWidget; Allocation : PGtkAllocation);
    StateChanged : ^procedure (Widget : PGtkWidget; PreviousState : TGtkStateType);
    ParentSet : ^procedure (Widget, PreviousParent : PGtkWidget);
    StyleSet : ^procedure (Widget : PGtkWidget; PreviousStyle : PGtkStyle);
  
    { Accelerators. }
    AddAccelerator : ^function (Widget : PGtkWidget; AccelSignalID : GUint; AccelGroup : PGtkAccelGroup; AccelKey : GUint; AccelMods : GdkModifierType; AccelFlags : TGtkAccelFlags) : GInt;
    RemoveAccelerator : ^procedure (Widget : PGtkWidget; AccelGroup : PGtkAccelGroup; AccelKey : GUint; AccelMods : GdkModifierType);

    { Explicit focus. }
    GrabFocus : ^procedure (Widget : PGtkWidget);

    { Events. }
    Event : ^function (Widget : PGtkWidget; Event : PGdkEvent) : GInt;
    ButtonPressEvent, ButtonReleaseEvent : ^function (Widget : PGtkWidget; Event : PGdkEventButton) : GInt;
    MotionNotifyEvent : ^function (Widget : PGtkWidget; Event : PGdkEventMotion) : GInt;
    DeleteEvent, DestroyEvent : ^function (Widget : PGtkWidget; Event : PGdkEventAny) : GInt;
    ExposeEvent : ^function (Widget : PGtkWidget; Event : PGdkEventExpose) : GInt;
    KeyPressEvent, KeyReleaseEvent : ^function (Widget : PGtkWidget; Event : PGdkEventKey) : GInt;
    EnterNotifyEvent, LeaveNotifyEvent : ^function (Widget : PGtkWidget; Event : PGdkEventCrossing) : GInt;
    ConfigureEvent : ^function (Widget : PGtkWidget; Event : PGdkEventConfigure) : GInt;
    FocusInEvent, FocusOutEvent : ^function (Widget : PGtkWidget; Event : PGdkEventFocus) : GInt;
    MapEvent, UnmapEvent : ^function (Widget : PGtkWidget; Event : PGdkEventAny) : GInt;
    PropertyNotifyEvent : ^function (Widget : PGtkWidget; Event : PGdkEventProperty) : GInt;
    SelectionClearEvent, SelectionRequestEvent, SelectionNotifyEvent : ^function (Widget : PGtkWidget; Event : PGdkEventSelection) : GInt;
    ProximityInEvent, ProximityOutEvent : ^function (Widget : PGtkWidget; Event : PGdkEventProximity) : GInt;
    VisibilityNotifyEvent : ^function (Widget : PGtkWidget; Event : PGdkEventVisibility) : GInt;
    ClientEvent : ^function (Widget : PGtkWidget; Event : PGdkEventClient) : GInt;
    NoExposeEvent : ^function (Widget : PGtkWidget; Event : PGdkEventAny) : GInt;

    { Selection. }
    SelectionGet : ^procedure (Widget : PGtkWidget; SelectionData : PGtkSelectionData; Info, Time : GUint);
    SelectionReceived : ^procedure (Widget : PGtkWidget; SelectionData : PGtkSelectionData; Time : GUint);

    { Source side drag signals. }
    DragBegin, DragEnd : ^procedure (Widget : PGtkWidget; Context : PGdkDragContext);
    DragDataGet : ^procedure (Widget : PGtkWidget; Context : PGdkDragContext; SelectionData : PGtkSelectionData; Info, Time : GUint);
    DragDataDelete : ^procedure (Widget : PGtkWidget; Context : PGdkDragContext);

    { Target side drag signals. }
    DragLeave : ^procedure (Widget : PGtkWidget; Context : PGdkDragContext; Time : GUint);
    DragMotion, DragDrop : ^function (Widget : PGtkWidget; Context : PGdkDragContext; x, y : GInt; Time : GUint) : GBoolean;
    DragDataReceived : ^procedure (Widget : PGtkWidget; Context : PGdkDragContext; x, y : GInt; SelectionData : PGtkSelectionData; Info, Time : GUint);

    { Action signals. }
    DebugMsg : ^procedure (Widget : PGtkWidget; Message : PGChar);

    { Padding for future expandsion. }
    Pad1, Pad2, Pad3, Pad4 : TGtkFunction;
  end;

type
  PGtkWidgetAuxInfo = ^TGtkWidgetAuxInfo;
  TGtkWidgetAuxInfo = record
    x, y, Widget, Height : GInt16;
  end;

type
  PGtkWidgetShapeInfo = ^TGtkWidgetShapeInfo;
  TGtkWidgetShapeInfo = record
    OffsetX, OffsetY : GInt16;
    ShapeMask : PGdkBitmap;
  end;

function GtkWidgetGetType : TGtkType;
asmname 'gtk_widget_get_type';

function GtkWidgetNew (TType : TGtkType; FirstArgName : PGChar; ...) : PGtkWidget;
asmname 'gtk_widget_new';

function GtkWidgetNewV (TType : TGtkType; NArgs : GUint; Args : PGtkArg) : PGtkWidget;
asmname 'gtk_widget_newv';

procedure GtkWidgetRef (Widget : PGtkWidget);
asmname 'gtk_widget_ref';

procedure GtkWidgetUnref (Widget : PGtkWidget);
asmname 'gtk_widget_unref';

procedure GtkWidgetDestroy (Widget : PGtkWidget);
asmname 'gtk_widget_destroy';

procedure GtkWidgetDestroyed (Widget : PGtkWidget; WidgetPointer : PPGtkWidget);
asmname 'gtk_widget_destroyed';

procedure GtkWidgetGet (Widget : PGtkWidget; Arg : PGtkArg);
asmname 'gtk_widget_get';

procedure GtkWidgetGetV (Widget : PGtkWidget; NArgs : GUint; Args : PGtkArg);
asmname 'gtk_widget_getv';

procedure GtkWidgetSet (Widget : PGtkWidget; FirstArgName : PGChar; ...);
asmname 'gtk_widget_set';

procedure GtkWidgetSetV (Widget : PGtkWidget; NArgs : GUint; Args : PGtkArg);
asmname 'gtk_widget_setv';

procedure GtkWidgetUnparent (Widget: PGtkWidget);
asmname 'gtk_widget_unparent';

procedure GtkWidgetShow (Widget: PGtkWidget);
asmname 'gtk_widget_show';

procedure GtkWidgetShowNow (Widget: PGtkWidget);
asmname 'gtk_widget_show_now';

procedure GtkWidgetHide (Widget: PGtkWidget);
asmname 'gtk_widget_hide';

procedure GtkWidgetShowAll (Widget: PGtkWidget);
asmname 'gtk_widget_show_all';

procedure GtkWidgetHideAll (Widget: PGtkWidget);
asmname 'gtk_widget_hide_all';

procedure GtkWidgetMap (Widget: PGtkWidget);
asmname 'gtk_widget_map';

procedure GtkWidgetUnmap (Widget: PGtkWidget);
asmname 'gtk_widget_unmap';

procedure GtkWidgetRealize (Widget: PGtkWidget);
asmname 'gtk_widget_realize';

procedure GtkWidgetUnrealize (Widget: PGtkWidget);
asmname 'gtk_widget_unrealize';

{ Queuing draws. }
procedure GtkWidgetQueueDraw (Widget: PGtkWidget);
asmname 'gtk_widget_queue_draw';

procedure GtkWidgetQueueDrawArea (Widget: PGtkWidget; x, y, Width, Height : GInt);
asmname 'gtk_widget_queue_draw_area';

procedure GtkWidgetQueueClear (Widget: PGtkWidget);
asmname 'gtk_widget_queue_clear';

procedure GtkWidgetQueueClearArea (Widget: PGtkWidget; x, y, Width, Height : GInt);
asmname 'gtk_widget_queue_clear_area';

procedure GtkWidgetQueueResize (Widget: PGtkWidget);
asmname 'gtk_widget_queue_resize';

procedure GtkWidgetDraw (Widget: PGtkWidget; Area : PGdkRectangle);
asmname 'gtk_widget_draw';

procedure GtkWidgetDrawFocus (Widget: PGtkWidget);
asmname 'gtk_widget_draw_focus';

procedure GtkWidgetDrawDefault (Widget: PGtkWidget);
asmname 'gtk_widget_draw_default';

procedure GtkWidgetSizeRequest (Widget: PGtkWidget; Requisition : PGtkRequisition);
asmname 'gtk_widget_size_request';

procedure GtkWidgetSizeAllocate (Widget: PGtkWidget; Allocation : PGtkAllocation);
asmname 'gtk_widget_size_allocate';

procedure GtkWidgetGetChildRequisition (Widget: PGtkWidget; Requisition : PGtkRequisition);
asmname 'gtk_widget_get_child_requisition';

procedure GtkWidgetAddAccelerator (Widget: PGtkWidget; AccelSignal : PGChar; AccelGroup : PGtkAccelGroup; AccelKey, AccelMods : GUint; AccelFlags : TGtkAccelFlags);
asmname 'gtk_widget_add_accelerator';

procedure GtkWidgetRemoveAccelerator (Widget: PGtkWidget; AccelGroup : PGtkAccelGroup; AccelKey, AccelMods : GUint);
asmname 'gtk_widget_remove_accelerator';

procedure GtkWidgetRemoveAccelerators (Widget : PGtkWidget; AccelSignal : PGChar; VisibleOnly : GBoolean);
asmname 'gtk_widget_remove_accelerators';

function GtkWidgetAcceleratorSignal (Widget : PGtkWidget; AccelGroup : PGtkAccelGroup; AccelKey, AccelMods : GUint) : GUint;
asmname 'gtk_widget_accelerator_signal';

procedure GtkWidgetLockAccelerators (Widget: PGtkWidget);
asmname 'gtk_widget_lock_accelerators';

procedure GtkWidgetUnlockAccelerators (Widget: PGtkWidget);
asmname 'gtk_widget_unlock_accelerators';

function GtkWidgetAcceleratorsLocked (Widget: PGtkWidget) : GBoolean;
asmname 'gtk_widget_accelerators_locked';

function GtkWidgetEvent (Widget : PGtkWidget; Event : PGdkEvent) : GInt;
asmname 'gtk_widget_event';

function GtkWidgetActivate (Widget: PGtkWidget) : GBoolean;
asmname 'gtk_widget_activate';

function GtkWidgetSetScrollAdjustments (Widget : PGtkWidget; HAdj, VAdj : PGtkAdjustment) : GBoolean;
asmname 'gtk_widget_set_scroll_adjustments';

procedure GtkWidgetReparent (Widget, NewParent : PGtkWidget);
asmname 'gtk_widget_reparent';

procedure GtkWidgetPopup (Widget: PGtkWidget; x, y : GInt);
asmname 'gtk_widget_popup';

function GtkWidgetIntersect (Widget : PGtkWidget; Area, Intersection : PGdkRectangle) : GInt;
asmname 'gtk_widget_intersect';

procedure GtkWidgetGrabFocus (Widget: PGtkWidget);
asmname 'gtk_widget_grab_focus';

procedure GtkWidgetGrabDefault (Widget: PGtkWidget);
asmname 'gtk_widget_grab_default';

procedure GtkWidgetSetName (Widget: PGtkWidget; Name : PGChar);
asmname 'gtk_widget_set_name';

function GtkWidgetGetName (Widget : PGtkWidget) : PGChar;
asmname 'gtk_widget_get_name';

procedure GtkWidgetSetState (Widget: PGtkWidget; State : TGtkStateType);
asmname 'gtk_widget_set_state';

procedure GtkWidgetSetSensitive (Widget: PGtkWidget; Sensitive : GBoolean);
asmname 'gtk_widget_set_sensitive';

procedure GtkWidgetSetAppPaintable (Widget : PGtkWidget; AppPaintable : GBoolean);
asmname 'gtk_widget_set_app_paintable';

procedure GtkWidgetSetParent (Widget, Parent : PGtkWidget);
asmname 'gtk_widget_set_parent';

procedure GtkWidgetSetParentWindow (Widget: PGtkWidget; ParentWindow : PGdkWindow);
asmname 'gtk_widget_set_parent_window';

function GtkWidgetGetParentWindow (Widget : PGtkWidget) : PGdkWindow;
asmname 'gtk_widget_get_parent_window';

procedure GtkWidgetSetUPosition (Widget : PGtkWidget; x, y : GInt);
asmname 'gtk_widget_set_uposition';

procedure GtkWidgetSetUSize (Widget : PGtkWidget; Width, Height : GInt);
asmname 'gtk_widget_set_usize';

procedure GtkWidgetSetEvents (Widget: PGtkWidget; Events : GInt);
asmname 'gtk_widget_set_events';

procedure GtkWidgetAddEvents (Widget: PGtkWidget; Events : GInt);
asmname 'gtk_widget_add_events';

procedure GtkWidgetSetExtensionEvents (Widget: PGtkWidget; Mode : GdkExtensionMode);
asmname 'gtk_widget_set_extension_events';

function GtkWidgetGetExtensionEvents (Widget : PGtkWidget) : GdkExtensionMode;
asmname 'gtk_widget_get_extension_events';

function GtkWidgetGetToplevel (Widget: PGtkWidget) : PGtkWidget;
asmname 'gtk_widget_get_toplevel';

function GtkWidgetGetAncestor (Widget: PGtkWidget; WidgetType : TGtkType) : PGtkWidget;
asmname 'gtk_widget_get_ancestor';

function GtkWidgetGetColormap (Widget: PGtkWidget) : PGdkColormap;
asmname 'gtk_widget_get_colormap';

function GtkWidgetGetVisual (Widget: PGtkWidget) : PGdkVisual;
asmname 'gtk_widget_get_visual';

{ The following functions must not be called on an already
  realized widget. Because it is possible that somebody
  can call get_colormap() or get_visual() and save the
  result, these functions are probably only safe to
  call in a widget's init() function. }
procedure GtkWidgetSetColormap (Widget : PGtkWidget; Colormap : PGdkColormap);
asmname 'gtk_widget_set_colormap';

procedure GtkWidgetSetVisual (Widget : PGtkWidget; Visual : PGdkVisual);
asmname 'gtk_widget_set_visual';

function GtkWidgetGetEvents (Widget : PGtkWidget) : GInt;
asmname 'gtk_widget_get_events';

procedure GtkWidgetGetPointer (Widget : PGtkWidget; x, y : PGInt);
asmname 'gtk_widget_get_pointer';

function GtkWidgetIsAncestor (Widget, Ancestor : PGtkWidget) : GBoolean;
asmname 'gtk_widget_is_ancestor';

{ Hide widget and return TRUE. }
function GtkWidgetHideOnDelete (Widget : PGtkWidget) : GInt;
asmname 'gtk_widget_hide_on_delete';

{ Widget styles. }
procedure GtkWidgetSetStyle (Widget : PGtkWidget; Style : PGtkStyle);
asmname 'gtk_widget_set_style';

procedure GtkWidgetSetRcStyle (Widget: PGtkWidget);
asmname 'gtk_widget_set_rc_style';

procedure GtkWidgetEnsureStyle (Widget: PGtkWidget);
asmname 'gtk_widget_ensure_style';

function GtkWidgetGetStyle (Widget : PGtkWidget) : PGtkStyle;
asmname 'gtk_widget_get_style';

procedure GtkWidgetRestoreDefaultStyle (Widget : PGtkWidget);
asmname 'gtk_widget_restore_default_style';

procedure GtkWidgetModifyStyle (Widget : PGtkWidget; Style : PGtkRcStyle);
asmname 'gtk_widget_modify_style';

{ Handle composite names for GTK_COMPOSITE_CHILD widgets,
  the returned name is newly allocated. }
procedure GtkWidgetSetCompositeName (Widget : PGtkWidget; Name : PGChar);
asmname 'gtk_widget_set_composite_name';

function GtkWidgetGetCompositeName (Widget : PGtkWidget) : PGChar;
asmname 'gtk_widget_get_composite_name';
     
{ Descend recursively and set rc-style on all widgets without user styles. }
procedure GtkWidgetResetRcStyles (Widget : PGtkWidget);
asmname 'gtk_widget_reset_rc_styles';

{ Push/pop pairs, to change default values upon a widget's creation.
  This will override the values that got set by the
  gtk_widget_set_default_* () functions. }

procedure GtkWidgetPushStyle (Style : PGtkStyle);
asmname 'gtk_widget_push_style';

procedure GtkWidgetPushColormap (CMap : PGdkColorMap);
asmname 'gtk_widget_push_colormap';

procedure GtkWidgetPushVisual (Visual : PGdkVisual);
asmname 'gtk_widget_push_visual';

procedure GtkWidgetPushCompositeChild;
asmname 'gtk_widget_push_composite_child';

procedure GtkWidgetPopCompositeChild;
asmname 'gtk_widget_pop_composite_child';

procedure GtkWidgetPopStyle;
asmname 'gtk_widget_pop_style';

procedure GtkWidgetPopColormap;
asmname 'gtk_widget_pop_colormap';

procedure GtkWidgetPopVisual;
asmname 'gtk_widget_pop_visual';

{ Set certain default values to be used at widget creation time. }
procedure GtkWidgetSetDefaultStyle (Style : PGtkStyle);
asmname 'gtk_widget_set_default_style';

procedure GtkWidgetSetDefaultColormap (Colormap : PGdkColormap);
asmname 'gtk_widget_set_default_colormap';

procedure GtkWidgetSetDefaultVisual (Visual : PGdkVisual);
asmname 'gtk_widget_set_default_visual';

function GtkWidgetGetDefaultStyle : PGtkStyle;
asmname 'gtk_widget_get_default_style';

function GtkWidgetGetDefaultColormap : PGdkColormap;
asmname 'gtk_widget_get_default_colormap';

function GtkWidgetGetDefaultVisual : PGdkVisual;
asmname 'gtk_widget_get_default_visual';

{ Counterpart to gdk_window_shape_combine_mask. }
procedure GtkWidgetShapeCombineMask (Widget : PGtkWidget; ShapeMask : PGdkBitmap; OffsetX, OffsetY : GInt);
asmname 'gtk_widget_shape_combine_mask';

{ !! Internal functions not implemented. }

{ Compute a widget's path in the form "GtkWindow.MyLabel", and
  return newly alocated strings. }
procedure GtkWidgetPath (Widget : PGtkWidget; PathLength : PGUint; Path, PathReversed: PPGChar);
asmname 'gtk_widget_path';

procedure GtkWidgetClassPath (Widget : PGtkWidget; PathLength : PGUint; Path, PathReversed: PPGChar);
asmname 'gtk_widget_class_path';

implementation

function GTK_WIDGET_TOPLEVEL(Wid : Pointer) = B : Boolean;
begin
  B := ((PGtkObject(Wid)^.Flags and GTK_TOPLEVEL) <> 0);
end;

function GTK_WIDGET_NO_WINDOW(Wid : Pointer) = B : Boolean;
begin
  B := ((PGtkObject(Wid)^.Flags and GTK_NO_WINDOW) <> 0);
end;

function GTK_WIDGET_REALIZED(Wid : Pointer) = B : Boolean;
begin
  B := ((PGtkObject(Wid)^.Flags and GTK_REALIZED) <> 0);
end;

function GTK_WIDGET_MAPPED(Wid : Pointer) = B : Boolean;
begin
  B := ((PGtkObject(Wid)^.Flags and GTK_MAPPED) <> 0);
end;

function GTK_WIDGET_VISIBLE(Wid : Pointer) = B : Boolean;
begin
  B := ((PGtkObject(Wid)^.Flags and GTK_VISIBLE) <> 0);
end;

function GTK_WIDGET_DRAWABLE(Wid : Pointer) = B : Boolean;
begin
  B := (GTK_WIDGET_VISIBLE(Wid) and GTK_WIDGET_MAPPED(Wid));
end;

function GTK_WIDGET_SENSITIVE(Wid : Pointer) = B : Boolean;
begin
  B := ((PGtkObject(Wid)^.Flags and GTK_SENSITIVE) <> 0);
end;

function GTK_WIDGET_PARENT_SENSITIVE(Wid : Pointer) = B : Boolean;
begin
  B := ((PGtkObject(Wid)^.Flags and GTK_PARENT_SENSITIVE) <> 0);
end;

function GTK_WIDGET_IS_SENSITIVE(Wid : Pointer) = B : Boolean;
begin
  B := (GTK_WIDGET_SENSITIVE(Wid) and GTK_WIDGET_PARENT_SENSITIVE(Wid));
end;

function GTK_WIDGET_CAN_FOCUS(Wid : Pointer) = B : Boolean;
begin
  B := ((PGtkObject(Wid)^.Flags and GTK_CAN_FOCUS) <> 0);
end;

function GTK_WIDGET_HAS_FOCUS(Wid : Pointer) = B : Boolean;
begin
  B := ((PGtkObject(Wid)^.Flags and GTK_HAS_FOCUS) <> 0);
end;

function GTK_WIDGET_CAN_DEFAULT(Wid : Pointer) = B : Boolean;
begin
  B := ((PGtkObject(Wid)^.Flags and GTK_CAN_DEFAULT) <> 0);
end;

function GTK_WIDGET_HAS_DEFAULT(Wid : Pointer) = B : Boolean;
begin
  B := ((PGtkObject(Wid)^.Flags and GTK_HAS_DEFAULT) <> 0);
end;

function GTK_WIDGET_HAS_GRAB(Wid : Pointer) = B : Boolean;
begin
  B := ((PGtkObject(Wid)^.Flags and GTK_HAS_GRAB) <> 0);
end;

function GTK_WIDGET_RC_STYLE(Wid : Pointer) = B : Boolean;
begin
  B := ((PGtkObject(Wid)^.Flags and GTK_RC_STYLE) <> 0);
end;

function GTK_WIDGET_COMPOSITE_CHILD(Wid : Pointer) = B : Boolean;
begin
  B := ((PGtkObject(Wid)^.Flags and GTK_COMPOSITE_CHILD) <> 0);
end;

function GTK_WIDGET_APP_PAINTABLE(Wid : Pointer) = B : Boolean;
begin
  B := ((PGtkObject(Wid)^.Flags and GTK_APP_PAINTABLE) <> 0);
end;

function GTK_WIDGET_RECEIVES_DEFAULT(Wid : Pointer) = B : Boolean;
begin
  B := ((PGtkObject(Wid)^.Flags and GTK_RECEIVES_DEFAULT) <> 0);
end;

procedure GTK_WIDGET_SET_FLAGS(Wid : Pointer; Flag : LongInt);
begin
  PGtkObject(Wid)^.Flags := PGtkObject(Wid)^.Flags or Flag;
end;

procedure GTK_WIDGET_UNSET_FLAGS(Wid : Pointer; Flag : LongInt);
begin
  PGtkObject(Wid)^.Flags := PGtkObject(Wid)^.Flags and (not Flag);
end;

end.
