{
This unit is part of the Gtk+ units package.
Copyright (C) 2001  Nicola Girardi <nikke@swlibero.org>
                    Eike Lange <eike.lange@uni-essen.de>

This unit is free software; you can redistribute it and/or
modify it under the terms of the GNU unit General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This unit is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
unit General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this unit; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

Original GTK copyright and license information follows.
}

{
GTK - The GIMP Toolkit
Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.
}

{
Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
file for a list of people on the GTK+ Team.  See the ChangeLog
files for a list of changes.  These files are distributed with
GTK+ at ftp://ftp.gtk.org/pub/gtk/.
}

unit GtkWindow;

interface

uses Glib, GdkTypes, GtkAccelGroup, GtkBin, GtkEnums, GtkObject, GtkWidget;

function GTK_TYPE_WINDOW : TGtkType;
asmname 'gtk_window_get_type';

function GTK_IS_WINDOW (Obj : Pointer) : Boolean;
function GTK_IS_WINDOW_CLASS (Klass : Pointer) : Boolean;

type
  PGtkWindowClass = ^TGtkWindowClass;
  PGtkWindow      = ^TGtkWindow;

  TGtkWindowClass = record
    ParentClass : TGtkBinClass;
    SetFocus : ^procedure (Window : PGtkWindow; Focus : PGtkWidget);
  end;

  TGtkWindowFrame1 = packed record
    AllowShrink, AllowGrow, AutoShrink, HandlingResize : GUint1;
    Position : GUint2;
    UsePosition, Modal : GUint1;
    { Set if the window or any descendent of it, has the focus. }
    WindowHasFocus : GUint1;
    { Set if !window_has_focus, but events are being sent to the
      window because the pointer is in it. (Typically, no window
      manager is running.) }
    WindowHasPointerFocus : GUint1
  end;

  TGtkWindow = record
    Bin : TGtkBin;

    Title, WMClassName, WMClassClass : PGChar;
    TType : TGtkWindowType;

    FocusWidget, DefaultWidget : PGtkWidget;
    TransientParent : PGtkWindow;

    ResizeCount : GUShort;

    Frame1 : TGtkWindowFrame1;
  end;

function GtkWindowGetType : TGtkType;
asmname 'gtk_window_get_type';

function GtkWindowNew (TType : TGtkWindowType) : PGtkWidget;
asmname 'gtk_window_new';

procedure GtkWindowSetTitle (Window : PGtkWindow; Title : PGChar);
asmname 'gtk_window_set_title';

procedure GtkWindowSetWMClass (Window : PGtkWindow; WMClassName, WMClassClass : PGChar);
asmname 'gtk_window_set_wmclass';

procedure GtkWindowSetPolicy (Window : PGtkWindow; AllowShrink, AllowGrow, AutoShrink : GInt);
asmname 'gtk_window_set_policy';

procedure GtkWindowAddAccelGroup (Window : PGtkWindow; AccelGroup : PGtkAccelGroup);
asmname 'gtk_window_add_accel_group';

procedure GtkWindowRemoveAccelGroup (Window : PGtkWindow; AccelGroup : PGtkAccelGroup);
asmname 'gtk_window_remove_accel_group';

procedure GtkWindowSetPosition (Window : PGtkWindow; Position : TGtkWindowPosition);
asmname 'gtk_window_set_position';

function GtkWindowActivateFocus (Window : PGtkWindow) : GInt;
asmname 'gtk_window_activate_focus';

function GtkWindowActivateDefault (Window : PGtkWindow) : GInt;
asmname 'gtk_window_activate_default';

procedure GtkWindowSetTransientFor (Window : PGtkWindow; Parent : PGtkWindow);
asmname 'gtk_window_set_transient_for';

procedure GtkWindowSetGeometryHints (Window : PGtkWindow; GeometryWidget : PGtkWidget; Geometry : PGdkGeometry; GeomMask : GdkWindowHints);
asmname 'gtk_window_set_geometry_hints';

{ The following differs from gtk_widget_set_usize, in that
  gtk_widget_set_usize() overrides the requisition, so sets a minimum
  size, while this only sets the size requested from the WM. }
procedure GtkWindowSetDefaultSize (Window : PGtkWindow; Width, Height : GInt);
asmname 'gtk_window_set_default_size';

{ If window is set modal, input will be grabbed when show and released when hide. }
procedure GtkWindowSetModal (Window : PGtkWindow; Modal : GBoolean);
asmname 'gtk_window_set_modal';

{ !! Internal functions not implemented. }

implementation

function GTK_IS_WINDOW (Obj : Pointer) : Boolean;
begin
  GTK_IS_WINDOW := (Obj <> nil) and GTK_IS_WINDOW_CLASS(PGtkTypeObject(Obj)^.Klass);
end;

function GTK_IS_WINDOW_CLASS (Klass : Pointer) : Boolean;
begin
  GTK_IS_WINDOW_CLASS := (Klass <> nil) and (GtkTypeIsA(PGtkTypeClass(Klass)^.TType, GTK_TYPE_WINDOW) = G_TRUE);
end;

end.
