/*
xpm2pas.l Lexer to convert XPM files to Pascal arrays.
Copyright (C) 2001 Nicola Girardi <nikke@swlibero.org>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

%option noyywrap
%pointer

%x Start
%x Lines

%{

#include <stdio.h>
#include <string.h>

static char *xpm_id = NULL;
static int flag = 0;

%}

COMMENT_1 \/\*.*\*\/
COMMENT_2 \/\/.*\n
COMMENT ({COMMENT_1}|{COMMENT_2})\n*

ID [_A-Za-z][_A-Za-z0-9]*

DECL "static"[\ \t]+"char"[\ \t]*\*[\ \t]*{ID}\[\][\ \t]*\=[\ \t]*\{[\ \t]*\n*

NORMLINE \".*\"[\ \t\n]*,[\ \t\n]*
LASTLINE \".*\"[\ \t\n]*\}[\ \t\n]*;[\ \t\n]*

%%
                        BEGIN(Start);

<*>{COMMENT}

<Start>{DECL}           {
                          char *q, *p = strrchr(yytext, '[');
                          if (p)
                                  *p = '\0';
                          else {
                                  fprintf(stderr, "Unexpected string (0)\n");
                                  exit(1);
                          }
                          p = strchr(yytext, '*') + 1;
                          if (!p) {
                                  fprintf(stderr, "Unexpected string (1)\n");
                                  exit(1);
                          }
                          while (*p == ' ')
                                  p++;
                          while ((q = strrchr(yytext, ' ')))
                                  *q = '\0';
                          xpm_id = strdup(p);
                          BEGIN(Lines);
                        }

<Lines>{NORMLINE}       {
                          if (!flag) { /* This is the first time! */
                                  int h, ncols;
                                  char *p, *q;

                                  flag = 1;
                                  q = strdup(yytext);
                                  q++;

                                  for (; *q == ' '; q++);
                                  for (; *q != ' '; q++);
                                  for (; *q == ' '; q++);
                                  /* Now we are pointing at the height string. */
                                  p = strchr(q, ' ');
                                  *p = '\0';
                                  h = atoi(q);
                                  q = p+1;
                                  for (; *q == ' '; q++);
                                  /* Now we are pointing at the ncols string. */
                                  p = strchr(q, ' ');
                                  *p = '\0';
                                  ncols = atoi(q);

                                  fprintf(stdout, "var\n  %s : array[0..%d] of PChar = (\n", xpm_id, h+ncols);
                                  free(xpm_id);
                          }

                          fprintf(stdout, "    %s", yytext);
                        }

<Lines>{LASTLINE}       {
                          char *p = strrchr(yytext, '}');
                          if (!p) {
                                  fprintf(stderr, "Unexpected string.\n");
                                  exit(-1);
                          }
                          *p = ')';
                          fprintf(stdout, "    %s", yytext);
                          exit(0);
                        }

%%
